///////////////////////////////////////////////////////////////////////////////
/**
 * @file 	config.h
 * @author	Cal(it)2
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2011, The Regents of the University of California
 * All Rights Reserve
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 *
 * Change History:
 * 		Rev Date        	Author              Description of Change
 * @endverbatim
 */
////////////////////////////////////////////////////////////////////////////////

#ifndef _P25_CONFIG_H
#define _P25_CONFIG_H

#include "p25/constants.h"

//#define _DEBUG


const char             p25_cntrl_header[2] = {0x0A,0xA0};

/** @brief Default Network Access Code. */
const unsigned short   NAC = 0x0293;
/** @brief Default Talk-group ID. */
const unsigned short   TGID = 0x0001;
/** @brief Default Unit ID. */
const unsigned char    UID[3] = {0x00,0x00,0x01};
/** @brief Default Emergency Indicator. */
const unsigned char    EBIT = 0x0;
/** @brief Default Algorithm ID. */
const unsigned char    ALGID = 0x80;
const unsigned short   KID = 0x0000;
/** @brief Default Manufacturer ID. */
const unsigned char    MFID = 0x00;
/** @brief Default Message Indicator. */
const unsigned char    MI[9] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

/** @brief Unit ID designated for group calls to everyone. */
const unsigned char    EVERYONE_UID[3] = {0xFF,0xFF,0xFF};
/** @brief Talk-group ID that includes everyone. */
const unsigned short   EVERYONE_TGID = 0xFFFF;
/** @brief No NAC Mode NAC constant. */
const unsigned short   P25_NO_NAC_MODE = 0x0F7E;

#pragma pack(push,1)
/** @brief Struct used for data passed between GUI and OSSIE.
  * @details GUI sends data one element at a time, so UInterface struct needs to be packed.
  */
typedef struct {
    double          cfreq; 	///< Center frequency
    unsigned short  nac;	///< Network Access Code
    unsigned short  tgId;	///< Talk-group ID
    unsigned short  keyId;	///< Key ID
    unsigned char   mfId;	///< Manufacturer ID
    unsigned char   algId;	///< Algorithm ID
    unsigned char   ebit;	///< Emergency Bit
    unsigned char   unitId[3];	///< Unit ID
    unsigned char   dstId[3];	///< Destination ID
    unsigned char   mi[9];	///< Message Indicator
    unsigned char   tfpath[255];///< Test File path
    unsigned char  	tfmode;	///< Test File Mode
    unsigned char   ptt;	///< Push to talk status
} UInterface;
#pragma pack(pop)


#endif
