/******************************************************************************
     Name:  golay.h
   Author:  Anthony Nwokafor
     Date:  6/24/09
  History:  6/24/09  New
This module contains three functions:
	golay_24_12_encode ( word )    encodes (24,12,8) extended  Golay code
    golay_18_6_encode ( word )    encodes (18, 6,8) shortened Golay code
    golay_24_12_decode ( word )    decodes (24,12,8) extended  Golay code
    golay_18_6_decode ( word )    decodes (18, 6,8) shortened Golay code
The encode functions return a long int value that is 18 or 24 bits long
for the respective function.
The decode functions return an int value that is 6 or 12 bits long
for the respective function.
******************************************************************************/
#ifndef _P25_ECC_GOLAY_H
#define _P25_ECC_GOLAY_H

const int N = 24;
const int K = 12;
const int P = 12;

const long G[12] = {040006165,
						 020003073,
						 010007550,
						  04003664,
						  02001732,
						  01006631,
						   0403315,
						   0201547,
						   0106706,
							045227,
							024476,
							014353};
								 
const long B[12] = {06165,
                         03073,
                         07550,
                         03664,
                         01732,
                         06631,
                         03315,
                         01547,
                         06706,
                         05227,
                         04476,
                         04353};
						 
const long BT[12] = {05117,
                          07550,
                          03664,
                          01732,
                           0755,
                          05271,
                          07423,
                          06706,
                          03343,
                          04476,
                          02237,
                          06165};
								  
const int code_word_mask[12] = {04000,
                                 02000,
                                 01000,
                                  0400,
                                  0200,
                                  0100,
                                   040,
                                   020,
                                   010,
                                    04,
                                    02,
                                    01};
									
long golay_18_6_encode ( int );
long golay_24_12_encode ( int );

int golay_24_12_decode ( long * );
int golay_18_6_decode ( long * );

long syndrome ( long );
#endif
