///////////////////////////////////////////////////////////////////////////////
/**
 * @file 	bit_fifo.h
 * @author	Cal(it)2
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2011, The Regents of the University of California
 * All Rights Reserve
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 *
 * Change History:
 * 		Rev Date        	Author              Description of Change
 * @endverbatim
 */
////////////////////////////////////////////////////////////////////////////////

#ifndef _P25_FORMAT_BIT_FIFO_H
#define _P25_FORMAT_BIT_FIFO_H

#define BF_SIZE_DFT		1728*4 //DSP version had 1728*2
#define BF_SIZE_MAX		1728*10

/**
 * @brief Bit FIFO struct
 */
struct bit_fifo { //DSP verison used unsigned int instead of int
	int	byte_size;   ///< Size of bit fifo in bytes
	int	head_index;  ///< Head Index
	int	head_offset; ///< Head Offset
	int	tail_index;  ///< Tail Index
	int	tail_offset; ///< Tail Offset
	char	*bytes;  ///< Contents of bit fifo
};

/** @brief Allocate bit fifo, size in bits, the actual size may be larger. */
struct bit_fifo *bf_alloc(int size);
/** @brief Free bit fifo */
void bf_free(struct bit_fifo *bf);
/** @brief Reset bit fifo */
void bf_reset(struct bit_fifo *bf);
/** @brief Check if bit fifo is empty */
int bf_empty(struct bit_fifo *bf);
/** @brief Check if bit fifo is full */
int bf_full(struct bit_fifo *bf);    
/** @brief Get number of bits in bit fifo */
int bf_size(struct bit_fifo *bf);
/** @brief Get available space in bit fifo */
int bf_space(struct bit_fifo *bf);

/* These access functions are dangerous. */
/** @brief Bit fifo access function for putting bits into fifo */
void __bf_put(struct bit_fifo *bf, char b);
/** @brief Bit fifo access function for getting bits from fifo */
char __bf_get(struct bit_fifo *bf);

/* Get n bits at a time, start r/w from [0] */
/** @brief Put bits into bit fifo */
int bf_put_n(struct bit_fifo *bf, char *bits, int n);
/** @brief Get bits from bit fifo */
int bf_get_n(struct bit_fifo *bf, char *bits, int n);

/* Functions for transmit fifo. */
/** @brief Imports bits into fifo */
int bf_import(struct bit_fifo *bf, char *bytes, int bit_len);
/** @brief Exports bits from fifo */
int bf_export(struct bit_fifo *bf, char *bytes, int bit_len);

/** @brief Dumps contents of bit fifo */
void bf_dump(struct bit_fifo *bf);

#endif

