///////////////////////////////////////////////////////////////////////////////
/**
 * @file 	bit_stream.h
 * @author	Cal(it)2
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2011, The Regents of the University of California
 * All Rights Reserve
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 *
 * Change History:
 * 		Rev Date        	Author              Description of Change
 * @endverbatim
 */
////////////////////////////////////////////////////////////////////////////////

#ifndef _BIT_STREAM_H
#define _BIT_STREAM_H

/* FIXME: optimize bit stream */

static void *input_task(void *arg);
static void *output_task(void *arg);

/*
 * @brief Read bits from GPP
 * 
 * @param bits: array of bits, each bit is represented by a byte
 * @param n:	 number of bits to read
 *
 * @return The number of bits read or -1 if error
 */
int bs_input(char *bits, int n);

/*
 * @brief Write bits to GPP
 * 
 * @param bits: array of bits, each bit is represented by a byte
 * @paramn:	 number of bits to write
 *
 * @return The number of bits write or -1 if error
 */
int bs_output(char *bits, int n);

/*
 * @brief Flush the remaining bits in the bit
 *                   stream to GPP
 */
void bs_output_flush(int mode);

/* 
 * @brief Initialize bit stream
 *
 * @return 0 if success, -1 if failed
 */
int bs_init(void);

/*
 * @brief Close bit stream
 */
void bs_exit(void);

#endif 
