///////////////////////////////////////////////////////////////////////////////
/**
 * @file 	int_stream.cpp
 * @author	Cal(it)2
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2011, The Regents of the University of California
 * All Rights Reserve
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 *
 * Change History:
 * 		Rev Date        	Author              Description of Change
 * @endverbatim
 */
////////////////////////////////////////////////////////////////////////////////

#include <cstdio>
#include <cstring>
#include <iostream>
#include <cmath>
#include <stdlib.h>

#include "../../intfc.hpp"
#include "format/int_stream.h"


FILE *tfile;
int first_time = 1;

#define BLOB_LEN		480 // because of UHD
#define FIX_SHIFT 		12

static float rd_buf[BLOB_LEN];
static float wr_buf[BLOB_LEN];
static int wr_idx = 0;
static int rd_idx = BLOB_LEN;
/*
 * @brief Send a integer to UHD
 * @param val: pointer of the integer to send
 * @return 0 on success, negative on failure
 *
 * @details We receive *val as a Fixed Point integer, but convert to float before sending to UHD/Radio
 */
int is_output(int *val)
{
	int ret = 0;

	wr_buf[wr_idx++] = ((float)(*val)) / pow(2, FIX_SHIFT);

	if (wr_idx == BLOB_LEN) {
		ret = dsp::intfc::write_uhdi((char *)wr_buf, sizeof(wr_buf));

		memset(wr_buf, 0, sizeof(wr_buf));
		wr_idx = 0;

		if (ret < 0) {
			printf("write_uhdi failed %d\n", ret);
			return -1;
		}
	}
	return 0;
}

/*
 * @brief Flush all data in the stream
 */
int is_output_flush(void)
{
	int ret = 0;

	ret = dsp::intfc::write_uhdi((char *)wr_buf, sizeof(wr_buf));

	memset(wr_buf, 0, sizeof(wr_buf));
	wr_idx = 0;

	if (ret < 0)
		printf("write_uhdi failed %d\n", ret);

	return ret;
}

/*
 * @brief Read a integer from UHD stream
 * @param val: pointer to save the integer
 * @return 0 on success, negative on failure
 *
 *
 */
int is_input(int *val)
{
//	if (first_time) {
//		tfile = fopen("tfile.dat", "w");
//		first_time = 0;
//		//fprintf(tfile, "rd_buf_currxx = %f, orig_val = %d, val = %d\n", rd_buf_curr, tmp, *val);
//	}

	int ret = 0;
	float rd_buf_curr = 0;

	if (rd_idx == BLOB_LEN) {
		memset(rd_buf, 0, sizeof(rd_buf));
		ret = dsp::intfc::read_uhdi((char*)rd_buf, sizeof(rd_buf));
		rd_idx = 0;

		if (ret < 0) {
			printf("read_uhdi failed %d\n", ret);
			return -1;
		}
	}

	// Convert float format to Fixed Point Integer by multiplying with scaling factor
	rd_buf_curr = rd_buf[rd_idx++];
	if (rd_buf_curr > 7.9) {
		rd_buf_curr = 7.9;
	}
	else if (rd_buf_curr < -7.9) {
		rd_buf_curr = -7.9;
	}

	*val = (int)(rd_buf_curr * pow(2, FIX_SHIFT));
	//*val = (int)floor(rd_buf_curr * pow(2, FIX_SHIFT) + 0.5);

        if (*val > 32767) {
		printf("*val = %d, rd_buf_curr = %f\n", *val, rd_buf_curr);
        }
        else if (*val < -32768) {
		printf("*val = %d, rd_buf_curr = %f\n", *val, rd_buf_curr);
        }

	//fprintf(tfile, "%f, %d, %d\n", rd_buf_curr, tmp, *val);

	return 0;
}

/*
 * @brief Reset UHD input stream
 */
void is_input_reset(void)
{
	rd_idx = BLOB_LEN;
}

