///////////////////////////////////////////////////////////////////////////////
/**
 * @file 	libsys.cpp
 * @brief	System function definitions.
 * @author	Cal(it)2
 * @author  Sivaramakrishnan Chandrasekaran <bscrk29@gmail.com>
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2009, The Regents of the University of California
 * All Rights Reserve
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 *
 * Change History:
 * 		Rev Date        	Author              Description of Change
 * @endverbatim
 */
///////////////////////////////////////////////////////////////////////////////

#include <iostream>
#include <cstring>

#include "libsys.hpp"
#include "modem.hpp"

using namespace std;

/**
 * @brief   The thread for Transmit.
 * @details The thread is controlled by input data
 * 			availability at the GPP interface. The thread continues execution
 * 			until there is data (bitstream) availabe from the GPP, otherwise
 * 			it waits on input from GPP.
 * @param   tx_obj the Tx object of the Tx chain
 */
void *tx_task_start(void *tx_obj){

	dsp::modem::tx *tx_chain = static_cast<dsp::modem::tx*>(tx_obj);

	tx_chain->tx_task();

}

/**
 * @brief 	The thread for Receive.
 * @details The thread is controlled by input data availability
 * 			at the UHD/Radio interface. The thread continues execution until
 * 			there is data (float stream) availabe from the Radio, otherwise
 * 			it waits on input from Radio.
 * @param   rx_obj the Rx object of the Rx chain
 */
void *rx_task_start(void *rx_obj){

	dsp::modem::rx *rx_chain = static_cast<dsp::modem::rx*>(rx_obj);

	rx_chain->rx_task();

}

//Not used. To be removed.
int sys_init(void)
{
	cout << "sys_init\n";
	return 0;
}

//Not used. To be removed.
void sys_exit(int error)
{
	cout << "sys_exit\n";
}
