///////////////////////////////////////////////////////////////////////////////
/**
 * @file 	libsys.hpp
 * @brief	Header file for systemwide constants and functions used to
 * 			control operation of the DSP module.
 * @author	Cal(it)2
 * @author  Sivaramakrishnan Chandrasekaran <bscrk29@gmail.com>
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2009, The Regents of the University of California
 * All Rights Reserve
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 *
 * Change History:
 * 		Rev Date        	Author              Description of Change
 * @endverbatim
 */
///////////////////////////////////////////////////////////////////////////////

#ifndef _LIBSYS_H
#define _LIBSYS_H

#include <time.h>
#include <cstdio>

#ifdef DEBUG
#define ERROR(fmt, args...)						\
	printf("[error] %s() " fmt, __FUNCTION__, ##args);

#define INFO(fmt, args...)						\
	printf("[information] %s() " fmt, __FUNCTION__, ##args);
#else
#define ERROR(fmt, args...)
#define INFO(fmt, args...)
#endif

/**
 * @brief The global variable for current radio mode defined by STAT_XXX,
 * 		  where XXX = {TX, RX, ERR}
 */
extern int sys_status;

#define STAT_TX				1
#define STAT_RX				0
#define STAT_ERR			-1

/**
 * @brief Starts the function for Transmit.
 */
extern "C" void *tx_task_start(void*);

/**
 * @brief Starts the function for Receive.
 */
extern "C" void *rx_task_start(void *);

/**
 * @brief System init function. Not used for anything now.
 */
int sys_init(void);


/**
 * @brief System exit function. Not used for anything now.
 */
void sys_exit(int);

#endif
