//-------------------------------------------------------------------------
/**
 * @file 	vocoder.h
 * @brief	Vocoder Class Definition
 *
 * @author	Cal(it)2
 * @author  Anthony Nwokafor <aanwokafor@ucsd.edu>
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 *
 * Change History:
 * 		Rev Date        	Author              Description of Change
 * @endverbatim
 */
//-------------------------------------------------------------------------
#ifndef _VOCODER_H
#define _VOCODER_H

#include <pthread.h>
#include <termios.h>
#include <netinet/in.h>
#include "p25/config.h"
#include "controller.h"

class Vocoder
{
private:

	/** @brief	Pointer to Controller object that created instance of this class. */
	Controller*		mController;

	/** @brief	Handle for Vocoder thread */
	pthread_t		vcThread;

	
	/** @brief	Pointer to string that contains the socket port */
	char*			vc_sport;

	/** @brief	String that holds RX filename */
	char*			vc_rxfile;

	/** @brief	Handle to socket for VC55 data communication */
	int				vc_sock;

	
	/** @brief */
	int			counter;

	
	/** @brief	Bit FIFO buffer that holds VC55 voice codewords waiting to be transmitted. */
	struct	bit_fifo 	*vc_bf;

	/** @brief	Bit FIFO buffer that holds received VC55 voice codewords waiting to be played back. */
	struct	bit_fifo 	*pb_bf;

	/** @brief */
	int			vc_bf_level;

	/** @brief */
	int			vc_pb_bf_level;



	/** @brief */
	pthread_mutex_t		vc_vc_bf_m;

	/** @brief */
	pthread_mutex_t		vc_pb_bf_m;

	/** @brief */
	struct	sockaddr_in 	vc_p_addr;

	/** @brief */
	int 			fdSerial;

	/** @brief */
	struct termios 		port_settings;

	/** @brief */
	FILE* 			test_mode_file_tx;
	FILE* 			test_mode_file_rx;
	char*			fbuf;
	long			fbufsize;
	long			fbufpos;

	/** @brief */
	int		readfile(char *buf, int len);

	/** @brief */
	void	send_vc_data();

	/** @brief */
	void	initialize();

	/** @brief */
	void	shutdown();
	
public:

	/** @brief */
	Vocoder(Controller* _mController, char* _vc_sport, char* _vc_rxfile);

	/** @brief */
	int		receive_vc_data(char* buf);

	/** @brief */
	void	put_in_pb_buf(char* vcw, int len);

	/** @brief */
	void	run_vc();

	/** @brief */
	void	read_vocoder();

	/** @brief */
	void	join();

	/** @brief */
	int 	getvc(char* buf);

	/** @brief */
	void	putvc(char* buf, int len);

	/** @brief */
	static void* start_thread(void *obj);
};

#endif
