#!/bin/bash
# modem_loopback.sh:  USRP P25 Demo startup script (GPP1->DSP1->MODEM1->MODEM2->DSP2->GPP2 Loopback Test Mode)
#                   Authors:  Jeffrey Cuenco   (jcuenco@ucsd.edu)
#                             Anthony Nwokafor (aanwokafor@ucsd.edu)
#
#                   Script last modified 04/22/2011
#

RUN_DIR=run
MPATH=../..
JAVA_GUI=../$MPATH/java_audio/p25_client/dist/p25_client.jar
RADIO_GPP=$MPATH/bin/packetizer
RADIO_DSP=$MPATH/bin/baseband
RADIO_FPGA=$MPATH/bin/modem

VC55_SERIAL_PORT=/dev/ttyS0

# Filetest mode = 1, Live mode (VC55) = 2
GPP_MODE=1

# Radio 1 settings
VC_INPUT1=$VC55_SERIAL_PORT
VC_OUTPUT1=/dev/null
if [ $GPP_MODE == "1" ]
then
    VC_INPUT1=testfile1_tx.dat
    VC_OUTPUT1=testfile1_rx.dat
fi

# Radio 2 settings
VC_INPUT2=/dev/null
VC_OUTPUT2=/dev/null
if [ $GPP_MODE == "1" ]
then
    VC_INPUT2=testfile2_tx.dat
    VC_OUTPUT2=testfile2_rx.dat
fi


killall xterm 
killall gdb
killall `basename $JAVA_GUI`
killall `basename $RADIO_GPP` `basename $RADIO_DSP` `basename $RADIO_FPGA`

DSP0_UHD0_SOCK=dsp0_uhd0_sock
UHD0_DSP0_SOCK=uhd0_dsp0_sock
DSP1_UHD1_SOCK=dsp1_uhd1_sock
UHD1_DSP1_SOCK=uhd1_dsp1_sock

UHD0_UHD1_FIFO=UHD0_UHD1_FIFO
UHD1_UHD0_FIFO=UHD1_UHD0_FIFO

RX_FILTER_COEF=rxfir_coef.bin
TX_FILTER_COEF=txfir_coef.bin

if [ ! -d $RUN_DIR ]
then
	mkdir -p $RUN_DIR
fi

cd $RUN_DIR


if [ ! -p $UHD0_UHD1_FIFO ] 
then
	mkfifo $UHD0_UHD1_FIFO
fi

if [ ! -p $UHD1_UHD0_FIFO ] 
then
	mkfifo $UHD1_UHD0_FIFO
fi


if [ ! -e $TX_FILTER_COEF ]
then
        cp $MPATH/../$TX_FILTER_COEF .
fi

if [ ! -e $RX_FILTER_COEF ]
then
        cp $MPATH/../$RX_FILTER_COEF .
fi

echo -n "Starting radio1 ..."
#xterm -hold -e "$RADIO_GPP $VC_INPUT1 $VC_OUTPUT1 8002 dsp0_gpp0_sock gpp0_dsp0_sock c1sock $GPP_MODE > radio1_log" &
xterm -hold -e "$RADIO_GPP $VC_INPUT1 $VC_OUTPUT1 8002 dsp0_gpp0_sock gpp0_dsp0_sock c1sock $GPP_MODE" &
xterm -hold -e "$RADIO_DSP gpp0_dsp0_sock dsp0_gpp0_sock $DSP0_UHD0_SOCK $UHD0_DSP0_SOCK dsp_cntrl_sock" &
xterm -hold -e "$RADIO_FPGA --rx_osock=$UHD0_DSP0_SOCK --tx_isock=$DSP0_UHD0_SOCK --rc_sock=dsp_cntrl_sock --fc_sock=c1sock --tx_lfifo=$UHD0_UHD1_FIFO --rx_lfifo=$UHD1_UHD0_FIFO" &
java -jar $JAVA_GUI &

sleep 3

echo -n "Starting radio2 ..."
#xterm -hold -e "$RADIO_GPP $VC_INPUT2 $VC_OUTPUT2 8004 dsp1_gpp1_sock gpp1_dsp1_sock c2sock $GPP_MODE > radio2_log" &
xterm -hold -e "$RADIO_GPP $VC_INPUT2 $VC_OUTPUT2 8004 dsp1_gpp1_sock gpp1_dsp1_sock c2sock $GPP_MODE" &
xterm -hold -e "$RADIO_DSP gpp1_dsp1_sock dsp1_gpp1_sock $DSP1_UHD1_SOCK $UHD1_DSP1_SOCK dsp_ctrl_sock" &
xterm -hold -e "$RADIO_FPGA --rx_osock=$UHD1_DSP1_SOCK --tx_isock=$DSP1_UHD1_SOCK --rc_sock=dsp_ctrl_sock --fc_sock=c2sock --tx_lfifo=$UHD1_UHD0_FIFO --rx_lfifo=$UHD0_UHD1_FIFO" &
java -jar $JAVA_GUI &

