/****************************************************************************
Copyright 2007 Virginia Polytechnic Institute and State University
This file is part of the OSSIE __COMP_NAME__.
OSSIE __COMP_NAME__ is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
"(at your option) any later version
OSSIE __COMP_NAME__ is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with OSSIE __COMP_NAME__; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

/**
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 */

#include <string>
#include <iostream>
#include "EncryptionDevice.h"

#include "p25/config.h"
#include "p25/controller.h"
#include "p25/format/packets.h"
#include "p25/util/logger.h"
#include "p25/util/ticktock.h"

//#define FRAME_LOOPBACK_RX
//#define FRAME_LOOPBACK_TX
//#define VCW_LOOPBACK

#define RX_DEV	0x00
#define TX_DEV	0x01

extern "C" {
#include "p25/format/byte_fifo.h"
#include "p25_cryptography.h"
}


int framesize = 88;
int echanbits = 88;

//-------------------------------------------------------------------
/**
 * @details Encryption class constructor for OSSIE implementation
 *
 * @author
 */
//-------------------------------------------------------------------
EncryptionDevice_i::EncryptionDevice_i(const char *uuid, omni_condition *condition) :
    Resource_impl(uuid), component_running(condition)
{
    enc_in = new standardInterfaces_i::realChar_p("encrypt_in");
    enc_out = new standardInterfaces_i::realChar_u("encrypt_out");
    tx_in = new standardInterfaces_i::realChar_p("tx_in");
    tx_out = new standardInterfaces_i::realChar_u("tx_out");
    rx_in = new standardInterfaces_i::realChar_p("rx_in");
    rx_out = new standardInterfaces_i::realChar_u("rx_out");

    mController = new Controller(0, NULL, NULL, 2);
    mController->set_radio_activity(STANDBY_ACTIVITY);

    radio_activity_switch = false;
//    start();
}

//-------------------------------------------------------------------
/**
 * @details Encryption class destructor for OSSIE implementation
 *
 * @author
 */
//-------------------------------------------------------------------
EncryptionDevice_i::~EncryptionDevice_i(void)
{
    delete enc_in;
    delete enc_out;
    delete tx_in;
    delete tx_out;
    delete rx_in;
    delete rx_out;

	delete mController;

}

//-------------------------------------------------------------------
/**
 * @details Static function for omni thread
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::Run( void * data )
{
    ((EncryptionDevice_i*)data)->ProcessData();
}

//-------------------------------------------------------------------
/**
 * @details Static function for omni thread
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::vcw_Run( void * data )
{
    ((EncryptionDevice_i*)data)->vcw_ProcessData();
}

//-------------------------------------------------------------------
/**
 * @details Static function for omni thread
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::rx_frame_Run( void * data )
{
    ((EncryptionDevice_i*)data)->rx_frame_ProcessData();
}

//-------------------------------------------------------------------
/**
 * @details Static function for omni thread
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::tx_frame_Run( void * data )
{
    ((EncryptionDevice_i*)data)->tx_frame_ProcessData();
}

//-------------------------------------------------------------------
/**
 * @details Static function for omni thread
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::crypt_Run( void * data )
{
	((EncryptionDevice_i*)data)->crypt_ProcessData();
}

//-------------------------------------------------------------------
/**
 * @details Static function for omni thread
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::vcw_out_Run( void * data )
{
    ((EncryptionDevice_i*)data)->vcw_out_ProcessData();
}

//-------------------------------------------------------------------
/**
 * @details Gets packetizer data from cryptography buffer for further processing
 *			(MI, LC, LSD, etc).
 *
 * @see bf_crypto_pktizer.
 * @author Anthony Nwokafor <aanwokafor@ucsd.edu>
 */
//-------------------------------------------------------------------
int EncryptionDevice_i::push_crypto_pktizer(char *buf, int len)
{
	int ret;
	// @todo: Add mutexes

//	INFO("Entered push_crypto_pktizer [Req = %d]\n", len);

	// If space in crypto -> pktizer FIFO is less than the requested len, wait
	// for the pktizer process to free up more space
	while ((bf_size(bf_crypto_pktizer) < len) && !bf_crypto_pktizer_accept_less) {
		sem_wait(&sem_crypto_pktizer_notempty);
	}

	// Put buf in crypto->pktizer FIFO
	ret = bf_get_n(bf_crypto_pktizer, buf, len);
	if (ret != len)
		DBG("Requested %d, Got %d\n",len,ret);

	if (bf_crypto_pktizer_accept_less)
			bf_crypto_pktizer_accept_less = 0;

	// Inform pktizer process of new data
	// @todo: might want to do this only when the fifo size was 0 prior to
	// inserting len bytes
	sem_post(&sem_crypto_pktizer_empty);

	return ret;
}

//-------------------------------------------------------------------
/**
 * @details Gets vocoder data from cryptography buffer for further processing
 *			(VCWs).
 *
 * @see bf_crypto_vocoder
 * @author Anthony Nwokafor <aanwokafor@ucsd.edu>
 */
//-------------------------------------------------------------------
int EncryptionDevice_i::push_crypto_vocoder(char *buf, int len)
{
	// @todo: Add mutexes
	int ret;

//	DBG("Entered push_crypto_vocoder [Req = %d]\n", len);

	// If space in crypto -> pktizer FIFO is less than the requested len, wait
	// for the pktizer process to free up more space
	while ((bf_size(bf_crypto_vocoder) < len) && !bf_crypto_vocoder_accept_less) {
		sem_wait(&sem_crypto_vocoder_notempty);
	}

	// Put buf in crypto->pktizer FIFO
	ret = bf_get_n(bf_crypto_vocoder, buf, len);
	if (ret != len)
		INFO("Requested %d, Got %d\n",len,ret);

	if (bf_crypto_vocoder_accept_less)
				bf_crypto_vocoder_accept_less = 0;

	// Inform pktizer process of new data
	// @todo: might want to do this only when the fifo size was 0 prior to
	// inserting len bytes
	sem_post(&sem_crypto_vocoder_empty);

//	DBG("Leaving push_crypto_vocoder [Got = %d]\n", ret);

	return ret;
}

//-------------------------------------------------------------------
/**
 * @details Get port data
 *
 * @author
 */
//-------------------------------------------------------------------
CORBA::Object_ptr EncryptionDevice_i::getPort( const char* portName ) throw (
    CORBA::SystemException, CF::PortSupplier::UnknownPort)
{
    DEBUG(3, EncryptionDevice, "getPort() invoked with " << portName)

    CORBA::Object_var p;

    p = enc_in->getPort(portName);

    if (!CORBA::is_nil(p))
        return p._retn();

    p = enc_out->getPort(portName);

    if (!CORBA::is_nil(p))
        return p._retn();

    p = tx_in->getPort(portName);

    if (!CORBA::is_nil(p))
        return p._retn();

    p = tx_out->getPort(portName);

    if (!CORBA::is_nil(p))
        return p._retn();

    p = rx_in->getPort(portName);

    if (!CORBA::is_nil(p))
        return p._retn();

    p = rx_out->getPort(portName);

    if (!CORBA::is_nil(p))
        return p._retn();

    /*exception*/
    throw CF::PortSupplier::UnknownPort();
}

//-------------------------------------------------------------------
/**
 * @details Starts threads for Encryption component
 *
 * @author
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::start() throw (CORBA::SystemException,
    CF::Resource::StartError)
{
    DEBUG(3, EncryptionDevice, "start() invoked");

    // Instantiate Controller
    mController->start();
    crypto_init();

    omni_thread::stacksize(2*1024*1024);

	omni_mutex_lock  l(processing_mutex);
	if( false == thread_started )
	{
		thread_started = true;
		// Create the thread for the writer's processing function
		processing_thread = new omni_thread(Run, (void *) this);

		// Start the thread containing the writer's processing function
		processing_thread->start();

		INFO("Created main encryption thread\n");
	}

	omni_mutex_lock  ll(vcw_processing_mutex);
	if( false == vcw_thread_started )
	{
		vcw_thread_started = true;
		// Create the thread for the writer's processing function
		vcw_processing_thread = new omni_thread(vcw_Run, (void *) this);

		// Start the thread containing the writer's processing function
		vcw_processing_thread->start();

		INFO("Created vcw encryption thread\n");
	}

	omni_mutex_lock  lll(rx_frame_processing_mutex);
	if( false == rx_frame_thread_started )
	{
		rx_frame_thread_started = true;
		// Create the thread for the writer's processing function
		rx_frame_processing_thread = new omni_thread(rx_frame_Run, (void *) this);

		// Start the thread containing the writer's processing function
		rx_frame_processing_thread->start();

		INFO("Created frame related encryption thread for Rx mode\n");
	}

	omni_mutex_lock  llll(tx_frame_processing_mutex);
	if( false == tx_frame_thread_started )
	{
		tx_frame_thread_started = true;
		// Create the thread for the writer's processing function
		tx_frame_processing_thread = new omni_thread(tx_frame_Run, (void *) this);

		// Start the thread containing the writer's processing function
		tx_frame_processing_thread->start();

		INFO("Created frame related encryption thread for Tx mode\n");
	}

	omni_mutex_lock  llllll(crypt_processing_mutex);
	if( false == crypt_thread_started )
	{
		crypt_thread_started = true;
		// Create the thread for the writer's processing function
		crypt_processing_thread = new omni_thread(crypt_Run, (void *) this);

		// Start the thread containing the writer's processing function
		crypt_processing_thread->start();
	}

	omni_mutex_lock  vcwll(vcw_out_processing_mutex);
	if( false == vcw_out_thread_started )
	{
		vcw_out_thread_started = true;
		// Create the thread for the writer's processing function
		vcw_out_processing_thread = new omni_thread(vcw_out_Run, (void *) this);

		// Start the thread containing the writer's processing function
		vcw_out_processing_thread->start();
	}


}

//-------------------------------------------------------------------
/**
 * @details Stops all threads.
 *
 * @author
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::stop() throw (CORBA::SystemException, CF::Resource::StopError)
{
    DEBUG(3, EncryptionDevice, "stop() invoked")

	omni_mutex_lock l(processing_mutex);
	thread_started = false;

	omni_mutex_lock ll(vcw_processing_mutex);
	vcw_thread_started = false;

	omni_mutex_lock lll(rx_frame_processing_mutex);
	rx_frame_thread_started = false;
	
	omni_mutex_lock llll(tx_frame_processing_mutex);
	tx_frame_thread_started = false;

	omni_mutex_lock llllll(crypt_processing_mutex);
	crypt_thread_started = false;

	omni_mutex_lock vcwll(vcw_out_processing_mutex);
	vcw_out_thread_started = false;
}

//-------------------------------------------------------------------
/**
 * @details
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::releaseObject() throw (CORBA::SystemException,
    CF::LifeCycle::ReleaseError)
{
    DEBUG(3, EncryptionDevice, "releaseObject() invoked")

    component_running->signal();
}

//-------------------------------------------------------------------
/**
 * @details Initialize Encryption component
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::initialize() throw (CF::LifeCycle::InitializeError,
    CORBA::SystemException)
{
    DEBUG(3, EncryptionDevice, "initialize() invoked")
}

//-------------------------------------------------------------------
/**
 * @details
 *
 * @author
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::query( CF::Properties & configProperties ) throw (CORBA::SystemException, CF::UnknownProperties)
{
	if( configProperties.length() == 0 )
	{
		configProperties.length( propertySet.length() );
		for( int i = 0; i < propertySet.length(); i++ )
		{
			configProperties[i].id = CORBA::string_dup( propertySet[i].id );
			configProperties[i].value = propertySet[i].value;
		}
		return;
	} else {
		for( int i = 0; i < configProperties.length(); i++ ) {
			for( int j = 0; j < propertySet.length(); j++ ) {
				if( strcmp(configProperties[i].id, propertySet[j].id) == 0 ) {
					configProperties[i].value = propertySet[j].value;
				}
			}
		}
	} // end if-else
}

//-------------------------------------------------------------------
/**
 * @details
 *
 * @author
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::configure(const CF::Properties& props)
throw (CORBA::SystemException,
    CF::PropertySet::InvalidConfiguration,
    CF::PropertySet::PartialConfiguration)
{
    DEBUG(3, EncryptionDevice, "configure() invoked")

    static int init = 0;
    if( init == 0 ) {
        if( props.length() == 0 ) {
            std::cout << "configure called with invalid props.length - " << props.length() << std::endl;
            return;
        }
        propertySet.length(props.length());
        for( int j=0; j < props.length(); j++ ) {
            propertySet[j].id = CORBA::string_dup(props[j].id);
            propertySet[j].value = props[j].value;
        }
        init = 1;
    }

    std::cout << "props length : " << props.length() << std::endl;

    for ( int i = 0; i <props.length(); i++)
    {
        std::cout << "Property id : " << props[i].id << std::endl;

    }
}

//-------------------------------------------------------------------
/**
 * @details Handles the process of retrieving packetizer data from the cryptography buffer.  
 *			In RX mode, first the encrypted link control
 * 			then two LSDs are extracted from the buffer in a loop for as long as
 * 			the radio remains in RX mode.
 * 			In TX mode, an encrypted MI is generated first for packetizing with the HDU.
 * 			Next, the link control is extracted, followed by one LSD, followed by the MI
 * 			for LDU2, etc. As per the encryption schedule.
 *  @see TIA/EIA-102.AAA-A
 *  @author Anthony Nwokafor <aanwokafor@ucsd.edu>
 *  @author Justyn Bell		 <jwbell@ucsd.edu>
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::ProcessData()
{
    DEBUG(3, EncryptionDevice, "ProcessData() invoked")

    PortTypes::CharSequence_var buffer_var;

    CryptoPacket pkt;

    while(continue_processing())
    {
    	int lsd_count;
    	int ret;
    	char buf[8];

    	// Don't start processing until the semaphores are posted by receiving a start
    	//	from both the packetizer component and vocoder component
    	while(pktizer_crypto_finished || vocoder_crypto_finished) {
    		DBG("Waiting for Enc device to switch states\n");
    		sem_wait(&sem_pktizer_ProcessData);
    		sem_wait(&sem_vocoder_ProcessData);
    	}

    	DBG("Enc device switched states\n");
    	bf_crypto_pktizer_accept_less = 0;

   		if (mController->get_radio_activity() == RX_ACTIVITY) {
   			lsd_count = 0;

   			DBG("Changing mode to Rx\n");
   			while (mController->get_radio_activity() == RX_ACTIVITY) {
   				if (lsd_count%2 == 0) {
					ret = push_crypto_pktizer(buf, LC_SZ);
					if (ret != LC_SZ)
						break;

					if (!pktizer_crypto_finished) {
						sendbuf_frame_data(RX_DEV, CRYPTO_PKT_LC, buf, LC_SZ);
					}
   				}

   				ret = push_crypto_pktizer(buf, LSD_SZ);
				if (ret != LSD_SZ)
					break;

				if (!pktizer_crypto_finished) {
					sendbuf_frame_data(RX_DEV, CRYPTO_PKT_LSD, buf, LSD_SZ);
				}

   				lsd_count++;
   				if (lsd_count == 2) lsd_count = 0;
   			}
   		} else if (mController->get_radio_activity() == TX_ACTIVITY) {
   			lsd_count = 0;

   			DBG("Changing mode to Tx\n");

   			ret = push_crypto_pktizer(buf, MI_SZ);
			if (ret != MI_SZ)
				break;

			if (!pktizer_crypto_finished) {
				DBG("pushing MI\n");
				sendbuf_frame_data(TX_DEV, CRYPTO_PKT_MI_HDU, buf, MI_SZ);
			}

   			while (mController->get_radio_activity() == TX_ACTIVITY)
   			{
				if (lsd_count%2 == 0) {
					ret = push_crypto_pktizer(buf, LC_SZ);
					if (ret != LC_SZ)
						break;

					if (!pktizer_crypto_finished) {
						DBG("pushing LC\n");
						sendbuf_frame_data(TX_DEV, CRYPTO_PKT_LC, buf, LC_SZ);
					}
   				} else {
   					ret = push_crypto_pktizer(buf, MI_SZ);
					if (ret != MI_SZ)
						break;

					if (!pktizer_crypto_finished) {
						DBG("pushing MI\n");
						sendbuf_frame_data(TX_DEV, CRYPTO_PKT_MI_LDU, buf, MI_SZ);
					}
   				}

				ret = push_crypto_pktizer(buf, LSD_SZ);
				if (ret != LSD_SZ)
					break;

				if (!pktizer_crypto_finished) {
					DBG("pushing LSD\n");
					sendbuf_frame_data(TX_DEV, CRYPTO_PKT_LSD, buf, LSD_SZ);
				}
   				lsd_count++;
   				if (lsd_count == 2) lsd_count = 0;
   			}
   		}
    }
}

//-------------------------------------------------------------------
/**
 * @details
 */
//-------------------------------------------------------------------
bool EncryptionDevice_i::continue_processing()
{
	omni_mutex_lock l(processing_mutex);
	return thread_started;
}

//-------------------------------------------------------------------
/**
 * @details Reveives data from RX component. When a START control is received from the RX
 * 			component, the Encryption device goes into the RX_ACTIVITY state,
 * 			and both the input and output FIFO cryptography loops are started
 * 			for sending and receiving data from the actual cryptography process.
 * 			When a STOP control message is received from the RX component, the input and
 * 			output FIFO cryptography loops are notified that encryption is finished, the
 * 			cryptography loops stop sending additional data to the RX component,
 * 			notifies the voice code word thread that a stop has been requested, and waits
 * 			for the voice code word thread to acknowledge; then finally resets
 * 			the cryptography buffers, puts the Encryption device into
 * 			STANDBY_ACTIVITY, and sends an acknowledgement message back to the
 * 			RX component notifying it that all buffers have been reset, and the radio
 * 			is waiting in a standby mode.
 *
 * 	@see vcw_ProcessData()
 * 	@see crypt_ProcessData()
 * 	@see ProcessData()
 *
 * 	@author Anthony Nwokafor <aanwokafor@ucsd.edu>
 * 	@author Justyn Bell		 <jwbell@ucsd.edu>
 *
 *
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::rx_frame_ProcessData()
{
	int ret;

    DEBUG(3, EncryptionDevice, "rx_frame_ProcessData() invoked")

    PortTypes::CharSequence *rx_data_in(NULL);

    PortTypes::CharSequence_var buffer_var;

    CORBA::UShort rx_data_in_length;

    CryptoPacket *pkt;
    CryptoPacket ctrlPkt;
    
    while(rx_frame_continue_processing())
    {
		rx_in->getData(rx_data_in);
		rx_data_in_length = rx_data_in->length();

		pkt = (CryptoPacket *)(rx_data_in->get_buffer());
		DBG("Got payload of size %d\n", rx_data_in_length);
		DBG("pkt.hdr = 0x%04X, pkt.len = %d\n", pkt->hdr, pkt->len);

		switch (pkt->hdr) {
		case CRYPTO_PKT_CNTRL_START:
			DBG("Received CRYPTO_PKT_CNTRL_START message from RX Device!\n");
			if (mController->get_radio_activity() != RX_ACTIVITY) {
				pktizer_crypto_finished = false;
				// Posts in crypt_ProcessData()
				sem_post(&sem_pktizer_crypt_processData);
				// Posts in ProcessData()
				sem_post(&sem_pktizer_ProcessData);

				mController->set_radio_activity(RX_ACTIVITY);
				INFO("Switching Encryption Device to RX_ACTIVITY\n");
				// Posts in crypt_ProcessData()
				sem_post(&sem_crypt_ProcessData);
			}
			break;
		case CRYPTO_PKT_CNTRL_STOP:
			DBG("Received CRYPTO_PKT_CNTRL_STOP message from RX Device!\n");
			pktizer_crypto_finished = true;
			// Posts in vcw_ProcessData()
			sem_post(&sem_pktizer_crypto_finished);

			DBG("Waiting for Vocoder_crypto_finished...\n");
			sem_wait(&sem_vocoder_crypto_finished); //sched_yield();

			DBG("Done waiting for Vocoder_crypto_finished\n");
			INFO("Switching Encryption Device to STANDBY_ACTIVITY\n");
			mController->set_radio_activity(STANDBY_ACTIVITY);

			crypto_reset();

			// Send RX device ACK message
			ctrlPkt.hdr = CRYPTO_PKT_CNTRL_ACK;
			ctrlPkt.len = 0;
			buffer_var = new PortTypes::CharSequence(sizeof(ctrlPkt), sizeof(ctrlPkt), (CORBA::Char *)&ctrlPkt, 0);
			rx_out->pushPacket(buffer_var);
			break;
		case CRYPTO_PKT_LC:
		case CRYPTO_PKT_LSD:
		case CRYPTO_PKT_MI_HDU:
		case CRYPTO_PKT_MI_LDU:

			// Place payload data into FIFO buffer
			ret = bf_put_n(bf_pktizer_crypto, pkt->data, pkt->len);
			sem_post(&sem_pktizer_crypto_notempty);
			if (ret != pkt->len)
				DBG("Tried to put %d, put %d",pkt->len,ret);

			DBG("Just added %d bits to the FIFO in Rx mode\n", pkt->len);
			break;
		default:
			ERROR("Did not get a correct CryptoPacket from RX device!\n");
			break;
		}
	rx_in->bufferEmptied();
	}
}

//-------------------------------------------------------------------
/**
 * @details
 */
//-------------------------------------------------------------------
bool EncryptionDevice_i::rx_frame_continue_processing()
{
	omni_mutex_lock lll(rx_frame_processing_mutex);
	return rx_frame_thread_started;
}

//-------------------------------------------------------------------
/**
 * @details Reveives data from TX component. When a START control is received from the TX
 * 			component, the Encryption device goes into the TX_ACTIVITY state,
 * 			and both the input and output FIFO cryptography loops are started
 * 			for sending and receiving data from the actual cryptography process.
 * 			When a STOP control message is received from the TX component, the input and
 * 			output FIFO cryptography loops are notified that encryption is finished, the
 * 			cryptography loops stop sending additional data to the TX component,
 * 			notifies the voice code word thread that a stop has been requested, and waits
 * 			for the voice code word thread to acknowledge; then finally resets
 * 			the cryptography buffers, puts the Encryption device into
 * 			STANDBY_ACTIVITY, and sends an acknowledgement message back to the
 * 			TX component notifying it that all buffers have been reset, and the radio
 * 			is waiting in a standby mode.
 *
 *	@see vcw_ProcessData()
 * 	@see crypt_ProcessData()
 * 	@see ProcessData()
 *
 * 	@author Anthony Nwokafor <aanwokafor@ucsd.edu>
 * 	@author Justyn Bell		 <jwbell@ucsd.edu>
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::tx_frame_ProcessData()
{
	int ret;

    DEBUG(3, EncryptionDevice, "rx_frame_ProcessData() invoked")

    PortTypes::CharSequence *tx_data_in(NULL);
    CORBA::UShort tx_data_in_length;

    PortTypes::CharSequence_var buffer_var;

    CryptoPacket *pkt;
    CryptoPacket ctrlPkt;

    while(tx_frame_continue_processing())
    {
		tx_in->getData(tx_data_in);
		tx_data_in_length = tx_data_in->length();
		pkt = (CryptoPacket *)(tx_data_in->get_buffer());

		if (tx_data_in_length != sizeof(CryptoPacket)) {
			ERROR("tx_data_in_length = %d not sizeof(CryptoPacket)\n", tx_data_in_length);
			continue;
		}

		switch (pkt->hdr) {
		case CRYPTO_PKT_CNTRL_START:
			if (mController->get_radio_activity() != TX_ACTIVITY) {
				DBG("Received START message from TX Device!\n");

				pktizer_crypto_finished = false;
				mController->set_radio_activity(TX_ACTIVITY);
				INFO("Switching Encryption Device to TX_ACTIVITY\n");

				sem_post(&sem_pktizer_crypt_processData);
				sem_post(&sem_pktizer_ProcessData);
				sem_post(&sem_crypt_ProcessData);
			}
			break;
		case CRYPTO_PKT_CNTRL_STOP:
			DBG("Received CRYPTOC_STOP message from TX Device!\n");
			pktizer_crypto_finished = true;

			// Wait until Encryption device sees that pktizer_crypto_finished
			// was set and changes state.
			DBG("Waiting on vocoder_crypto_finished...\n");
			sem_wait(&sem_vocoder_crypto_finished);

			INFO("Switching Encryption Device to STANDBY_ACTIVITY\n");
			mController->set_radio_activity(STANDBY_ACTIVITY);
			crypto_reset();

			// Send ack message back to TX component
			ctrlPkt.hdr = CRYPTO_PKT_CNTRL_ACK;
			ctrlPkt.len = 0;
			buffer_var = new PortTypes::CharSequence(sizeof(ctrlPkt), sizeof(ctrlPkt), (CORBA::Char *)&ctrlPkt, 0);
			tx_out->pushPacket(buffer_var);

			break;
		case CRYPTO_PKT_LC:
		case CRYPTO_PKT_LSD:
			// Place payload data into FIFO buffer
			ret = bf_put_n(bf_pktizer_crypto, pkt->data, pkt->len);
			sem_post(&sem_pktizer_crypto_notempty);
			if (ret != pkt->len)
				DBG("Tried to put %d, put %d",pkt->len,ret);

			DBG("Just added %d bits to the FIFO in Tx mode\n", pkt->len);
			break;
		}
	tx_in->bufferEmptied();
    }
}

//-------------------------------------------------------------------
/**
 * @brief
 */
//-------------------------------------------------------------------
bool EncryptionDevice_i::tx_frame_continue_processing()
{
	omni_mutex_lock llll(tx_frame_processing_mutex);
	return tx_frame_thread_started;
}

//-------------------------------------------------------------------
/**
 * @details Receives data from the Vocoder component. Since the implementation of
 * 			the encryption device operates on voice code words and packetizer data
 * 			(MI, LSD, LC) separately, when a START control message is received from
 * 			the vocoder component, this thread is the second "key" to starting the
 * 			main cryptography threads (along with tx_frame_ProcessData()/
 * 			rx_frame_ProcessData() threads).
 * 			When a STOP control message is received, the thread waits for the packetizer
 * 			processing threads to receive a STOP control message as well, then resets the
 * 			cryptography buffers.
 * 			When a DATA control message is received, it simply puts the data into
 * 			the bf_vocoder_crypto buffer for encryption/decryption.
 *
 * 	@see tx_frame_ProcessData()
 * 	@see rx_frame_ProcessData()
 * 	@see crypt_ProcessData()
 * 	@see ProcessData()
 *
 * 	@author Anthony Nwokafor <aanwokafor@ucsd.edu>
 * 	@author Justyn Bell 	 <jwbell@ucsd.edu>
 *
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::vcw_ProcessData()
{
    DEBUG(3, EncryptionDevice, "vcw_ProcessData() invoked")

    PortTypes::CharSequence *enc_data_in(NULL);
    CORBA::UShort enc_data_in_length;

    CryptVocPacket *cv_pkt;

    while(vcw_continue_processing())
    {
	DBG("Waiting for data from Vocoder Device\n");

	enc_in->getData(enc_data_in);
	enc_data_in_length = enc_data_in->length();

	cv_pkt = (CryptVocPacket *)(enc_data_in->get_buffer());

	if (enc_data_in_length != sizeof(CryptVocPacket))
		ERROR("enc_data_in_length != sizeof(CryptVocPacket)");

		switch (cv_pkt->hdr) {
		case CRYPTO_PKT_CNTRL_START:
			DBG("Received CRYPTOC_START message from Vocoder Device!\n");
			vocoder_crypto_finished = false;
			sem_post(&sem_vocoder_crypt_processData);
			sem_post(&sem_vocoder_ProcessData);
			break;
		case CRYPTO_PKT_CNTRL_STOP:
			DBG("Received CRYPTOC_STOP message from Vocoder Device!\n");
			vocoder_crypto_finished = true;
			sem_post(&sem_vocoder_crypto_finished);

			while(!pktizer_crypto_finished)
				sem_wait(&sem_pktizer_crypto_finished); //sched_yield();

			crypto_reset();
			break;
		case CRYPTO_PKT_CNTRL_DATA:
			// Place payload data into FIFO buffer
			bf_put_n(bf_vocoder_crypto, cv_pkt->data, cv_pkt->len);
			sem_post(&sem_vocoder_crypto_notempty);
			DBG("Put %d bytes into vcw FIFO\n", cv_pkt->len);
			break;
		default:
			ERROR("Unrecognized pkt header\n");
			break;
		}
	enc_in->bufferEmptied();
    }
}

//-------------------------------------------------------------------
/**
 * @details
 */
//-------------------------------------------------------------------
bool EncryptionDevice_i::vcw_continue_processing()
{
	omni_mutex_lock ll(vcw_processing_mutex);
	return vcw_thread_started;
}

//-------------------------------------------------------------------
/**
 * @details Gets VCWs from cryptography component, and pushes them out to the Vocoder Device.
 *
 * @author Justyn Bell <jwbell@ucsd.edu>
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::vcw_out_ProcessData()
{
	PortTypes::CharSequence_var buffer_var;
	int ret;
	char out_vcw[11];

	while(vcw_out_continue_processing())
	{
		// Push Voice Code Words
		ret = push_crypto_vocoder(out_vcw, sizeof(out_vcw));
		if (ret != sizeof(out_vcw))
			continue;

		buffer_var = new PortTypes::CharSequence(sizeof(out_vcw), sizeof(out_vcw), (CORBA::Char *)out_vcw, 0);
		enc_out->pushPacket(buffer_var);
	}
}

//-------------------------------------------------------------------
/**
 * @details 
 */
//-------------------------------------------------------------------
bool EncryptionDevice_i::vcw_out_continue_processing()
{
	omni_mutex_lock vcwll(vcw_out_processing_mutex);
	return vcw_out_thread_started;
}

//-------------------------------------------------------------------
/**
 * @details Main encryption thread. Responsible for getting first MI,
 *			initializing the LFSR with either the received MI (for RX)
 *			or with the [currently] hard-coded MI (for TX),
 *			and initializing the state of the encryption device.
 *			It encrypts the LDUs as per the encryption schedule, with 
 *			special attention paid to certain output words: those that
 *			contain part packetizer data and VCW data.  Otherwise 
 *			(as is the case with most of the LDUs) we simply read in an
 *			entire word from the Overo Vocoder component, encrypt, and
 *			send back to the Overo Vocoder Component.
 *
 * @see TIA/EIA/102.AAAA-A Table 5-5 and Table 5-6
 * @author Anthony Nwokafor <aanwokafor@ucsd.edu> 
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::crypt_ProcessData()
{
	int ret;

	CRYPTO_PROCESS_STATE pState; // Track crypto process state
	P25_DES_SCHED_STATE dState; // Track P25 DES sched state
	P25_DES_SCHED_STATE tmp_dState;

	char key[BLK_SZ] = {0x1A,0x2A,0x3B,0x4A,0x5B,0x6B,0x7A,0x8A};
	char mi[8] = {0x12,0x34,0x56,0x78,0x90,0xAB,0xCD,0xEF};

	char public_mi[BLK_SZ]; // packed public mi
	char buf[BLK_SZ];

	int offset;

	// Set initial crypto state to INIT
	pState = CRYPTO_P_STATE_INIT;

	// Crypto process should run as long as radio is on
	while (crypt_continue_processing()) {
		while (mController->get_radio_activity() == STANDBY_ACTIVITY) {
			sem_wait(&sem_crypt_ProcessData);
		}

		switch (pState) {
		case CRYPTO_P_STATE_INIT:
			DBG("Entered CRYPTO_P_STATE_INIT\n");

			memcpy(public_mi, mi, sizeof(mi));

			// Initialize the LFSR with mi
			init_lfsr(public_mi);

			// Load key
			init_dState(&dState, key, mi);

			offset = 0;

			while(pktizer_crypto_finished || vocoder_crypto_finished) {
				DBG("Waiting for Enc device to switch states\n");
				sem_wait(&sem_pktizer_crypt_processData);
				sem_wait(&sem_vocoder_crypt_processData);
			}
			DBG("Enc device switched states\n");

			radio_activity_switch = false;
			crypto_reset_done = 0;

			bf_pktizer_crypto_accept_less = 0;
			bf_vocoder_crypto_accept_less = 0;

			bf_crypto_pktizer_accept_less = 0;
			bf_crypto_vocoder_accept_less = 0;

			if (mController->get_radio_activity() == TX_ACTIVITY)
				ret = send_crypto_pktizer(public_mi, sizeof(public_mi));

			// Set crypto state to IV (Waiting for first IV which is the MI)
			pState = CRYPTO_P_STATE_IV;

			break;
		case CRYPTO_P_STATE_IV:
			DBG("Entered CRYPTO_P_STATE_IV\n");

			if (mController->get_radio_activity() == RX_ACTIVITY) {
				// Request public_mi from rx process. This is a blocking call
				//unsigned char *tmp_mi = (unsigned char *)public_mi;
				ret = recv_pktizer_crypto(public_mi, sizeof(public_mi));

				if (ret != sizeof(public_mi) || radio_activity_switch) {
					pState = CRYPTO_P_STATE_INIT;
					radio_activity_switch = false;
					break;
				}

				// Initialize the LFSR with public mi
				init_lfsr(public_mi);
			}

			memcpy(dState.mi, public_mi, sizeof(dState.mi));

			// Reset des sched state
			dState.word_count = 1;
			dState.done = 0;

			// Iterate encryption once in OFB mode to generate private mi
			// dState will contain the private_mi (key) after one round of
			// of ofb encryption.
			xcrypt(&dState, public_mi);

			pState = CRYPTO_P_STATE_LDU;
			break;
		case CRYPTO_P_STATE_LDU:
			DBG("Entered CRYPTO_P_STATE_LDU\n");
			switch (dState.word_count) {
			case 2: // 2nd output word
				DBG("CRYPTO_P_STATE_LDU: 2\n");
				offset = 3;

				// 3 reserved octets
				memset(buf,0,offset);

				// Request first 5 octets of LC from rx process
				ret = recv_pktizer_crypto(buf+offset, BLK_SZ-offset);
				if ((ret != BLK_SZ-offset) || radio_activity_switch){
					pState = CRYPTO_P_STATE_INIT;
					radio_activity_switch = false;
					break;
				}
				xcrypt(&dState, buf);

				// send first 5 octets of xcrypted LC to rx process
				ret = send_crypto_pktizer(buf+offset, BLK_SZ-offset);

				// prepare buf for next iteration of xcrypt algorithm
				// by requesting last 3 bytes of LC from rx process
				ret = recv_pktizer_crypto(buf, offset);
				if ((ret != offset) || radio_activity_switch){
					pState = CRYPTO_P_STATE_INIT;
					radio_activity_switch = false;
					break;
				}

				break;
			case 3: // 3rd output word
				DBG("CRYPTO_P_STATE_LDU: 3\n");
				offset = 3;

				// Get the next imbe bytes from the vocoder to crypto
				// fifo that completes 8 bytes
				ret = recv_vocoder_crypto(buf+offset, BLK_SZ-offset);
				if ((ret != BLK_SZ-offset) || radio_activity_switch){
					pState = CRYPTO_P_STATE_INIT;
					radio_activity_switch = false;
					break;
				}
				xcrypt(&dState, buf);

				// Send remaining 3 bytes of LC data to rx_process
				ret = send_crypto_pktizer(buf,offset);

				// Send first few bytes of IMBE deciphered txt to vocoder
				ret = send_crypto_vocoder(buf+offset, BLK_SZ-offset);

				break;
			case 14: // 14th output word
				DBG("CRYPTO_P_STATE_LDU: 14\n");
				memcpy(&tmp_dState,&dState,sizeof(dState));

				// Get the next 3 imbe bytes from the vocoder to crypto
				// fifo
				offset = 3;
				memset(buf,0,offset);

				// Request LSD octets from rx process (2)
				ret = recv_pktizer_crypto(buf+offset, LSD_SZ);
				offset += LSD_SZ;

				// Get the next 3 imbe bytes from the vocoder to crypto
				// fifo
				memset(buf+offset,0,BLK_SZ-offset);
				xcrypt(&tmp_dState, buf);

				offset -= LSD_SZ;
				// Send next 2 bytes of LSD data to rx process
				ret = send_crypto_pktizer(buf+offset,LSD_SZ);

				// Get the next 3 imbe bytes from the vocoder to crypto
				// fifo
				offset = 3;
				ret = recv_vocoder_crypto(buf,offset);

				// Request LSD octets from rx process (2)
				//recv_pktizer_crypto(buf+offset, LSD_SZ);
				memset(buf+offset,0,LSD_SZ); /*+++++ hack ++++*/
				offset += LSD_SZ;

				// Get the next 3 imbe bytes from the vocoder to crypto
				// fifo
				ret = recv_vocoder_crypto(buf+offset, BLK_SZ-offset);
				xcrypt(&dState, buf);

				// send first 3 octets of xcrypted txt to vocoder process
				offset = 3;
				ret = send_crypto_vocoder(buf, offset);

				offset += LSD_SZ;

				// send last 3 octets of xcrypted txt to vocoder process
				ret = send_crypto_vocoder(buf+offset, BLK_SZ-offset);

				// Since we're near end of LDU1, generate new public MI
				// and send to pktizer process if in transmit mode else
				// compare with received mi before decrypting LDU1
				lfsr_new_mi(public_mi);
				if (mController->get_radio_activity() == TX_ACTIVITY)
					ret = send_crypto_pktizer(public_mi,sizeof(public_mi));

				break;
			case 27: // 27th output word
				DBG("CRYPTO_P_STATE_LDU: 27\n");
				memcpy(&tmp_dState,&dState,sizeof(dState));

				// Get the LSD octets from rx_process (2)
				ret = recv_pktizer_crypto(buf, LSD_SZ);
				offset = LSD_SZ;

				// Get the next 6 imbe bytes from the vocoder to crypto
				// fifo
				memset(buf+offset, 0, BLK_SZ-offset);
				xcrypt(&tmp_dState, buf);

				// Send first 2 bytes of LSD data to rx process
				ret = send_crypto_pktizer(buf,LSD_SZ);

				// Get the LSD octets from rx_process (2)
				memset(buf, 0, LSD_SZ); /*+++++ hack ++++*/
				offset = LSD_SZ;

				// Get the next 6 imbe bytes from the vocoder to crypto
				// fifo
				ret = recv_vocoder_crypto(buf+offset, BLK_SZ-offset);
				xcrypt(&dState, buf);

				// send last 6 octets of xcrypted txt to vocoder process
				ret = send_crypto_vocoder(buf+offset, BLK_SZ-offset);

				break;
			case 28: // 28th output word
				DBG("CRYPTO_P_STATE_LDU: 28\n");
				// Request last 5 octets of IMBE from vocoder process
				offset = 5;
				ret = recv_vocoder_crypto(buf, offset);

				// 3 reserved octets
				memset(buf+offset,0, BLK_SZ-offset);
				xcrypt(&dState, buf);

				// send first 5 octets of xcrypted IMBE to vocoder process
				ret = send_crypto_vocoder(buf, offset);

				// Return to waiting for IV (new public mi)
				pState = CRYPTO_P_STATE_IV;

				break;
			default:
				DBG("CRYPTO_P_STATE_LDU: default\n");
				// Get the next imbe bytes from the vocoder to crypto
				// fifo that completes 8 bytes
				ret = recv_vocoder_crypto(buf, BLK_SZ);

				xcrypt(&dState, buf);

				// Send first few bytes of IMBE deciphered txt to vocoder
				ret = send_crypto_vocoder(buf, BLK_SZ);

				break;
			}
			break;
			case CRYPTO_P_STATE_DONE:
				break;
			default:
				// If we're done or see an unknown state then reset the crypto
				// process state to IV (Waiting for first IV which is the MI)
				pState = CRYPTO_P_STATE_INIT;
				break;
		}
	}
}

//-------------------------------------------------------------------
/**
 * @details
 */
//-------------------------------------------------------------------
bool EncryptionDevice_i::crypt_continue_processing()
{
	omni_mutex_lock llllll(crypt_processing_mutex);
	return crypt_thread_started;
}

//-------------------------------------------------------------------
/**
 * @details Simple function to send data to packetizer components (TX/RX components).
 *
 * @param dev	Data is sent to either the TX or RX component based on the dev parameter.
 * @param hdr	Data is appended with an appropriate header determined by the hdr parameter.
 * @param data	The actual data.
 * @param len	The length of the data in bytes.
 *
 * @author Justyn Bell <jwbell@ucsd.edu>
 */
//-------------------------------------------------------------------
void EncryptionDevice_i::sendbuf_frame_data(short dev, short hdr, char *data, int len)
{
	//DBG("Sending %d bytes of data to encryption device", len);
	CryptoPacket c_pkt;
	int CPSize = sizeof(CryptoPacket);

	c_pkt.hdr = hdr;
	c_pkt.len = len;

	memcpy((char *)&c_pkt.data, data, len);
	DBG("Sending cntrl 0x%04X, len %d\n", c_pkt.hdr, c_pkt.len);

	PortTypes::CharSequence_var buffer_var;
	buffer_var = new PortTypes::CharSequence(CPSize, CPSize, (CORBA::Char *)&c_pkt, 0);

	if 		(dev == RX_DEV)		rx_out->pushPacket(buffer_var);
	else if (dev == TX_DEV)		tx_out->pushPacket(buffer_var);

	else ERROR("Not a valid device choice!\n");
}

