/**
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 */

#ifndef _p25_CRYPTOGRAPHY_H
#define _P25_CRYPTOGRAPHY_H

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <openssl/des.h>
#include <semaphore.h>
#include <pthread.h>

#include "p25/format/byte_fifo.h"

#define BLK_SZ  8 // In bytes
#define IMBE_FRAME_SZ  11 // In bytes
#define LC_SZ  	8 // In bytes
#define LSD_SZ 	2 // In bytes
#define MI_SZ	8 // In bytes


typedef struct p25_des_sched_state {
	short word_count;
	short done;
	int mi_idx;
	short kid;
	DES_cblock mi;
	DES_key_schedule sched;
} P25_DES_SCHED_STATE;


typedef enum crypto_states {
	CRYPTO_P_STATE_INIT,
	CRYPTO_P_STATE_IV,
	CRYPTO_P_STATE_LDU,
	CRYPTO_P_STATE_DONE
} CRYPTO_PROCESS_STATE;

struct byte_fifo *bf_pktizer_crypto; ///< pktizer -> crypto FIFO
struct byte_fifo *bf_crypto_pktizer; ///< pktizer <- crypto FIFO
struct byte_fifo *bf_vocoder_crypto; ///< vocoder -> crypto FIFO
struct byte_fifo *bf_crypto_vocoder; ///< vocoder <- crypto FIFO

/** @brief producer <-> consumer semaphore pair for pktizer -> crypto FIFO. */
sem_t	sem_pktizer_crypto_empty;
/** @brief producer <-> consumer semaphore pair for pktizer -> crypto FIFO. */
sem_t	sem_pktizer_crypto_notempty;

/** @brief producer <-> consumer semaphore pair for pktizer <- crypto FIFO. */
sem_t	sem_crypto_pktizer_empty;
/** @brief producer <-> consumer semaphore pair for pktizer <- crypto FIFO. */
sem_t	sem_crypto_pktizer_notempty;

/** @brief producer <-> consumer semaphore pair for vocoder -> crypto FIFO. */
sem_t	sem_vocoder_crypto_empty;
/** @brief producer <-> consumer semaphore pair for vocoder -> crypto FIFO. */
sem_t	sem_vocoder_crypto_notempty;

/** @brief producer <-> consumer semaphore pair for vocoder <- crypto FIFO. */
sem_t	sem_crypto_vocoder_empty;
/** @brief producer <-> consumer semaphore pair for vocoder <- crypto FIFO. */
sem_t	sem_crypto_vocoder_notempty;


sem_t	sem_pktizer_ProcessData;
sem_t	sem_vocoder_ProcessData;

sem_t	sem_pktizer_crypt_processData;
sem_t	sem_vocoder_crypt_processData;

sem_t 	sem_crypt_ProcessData;

sem_t	sem_pktizer_crypto_finished;
sem_t	sem_vocoder_crypto_finished;

/** @brief */
pthread_mutex_t		mtx_crypto_reset;

char crypto_reset_done;

char bf_pktizer_crypto_accept_less;
char bf_crypto_pktizer_accept_less;
char bf_vocoder_crypto_accept_less;
char bf_crypto_vocoder_accept_less;

char pktizer_crypto_finished;
char vocoder_crypto_finished;

void dump(const char *, unsigned char *, int);
int send_crypto_pktizer(char *, int);
int recv_pktizer_crypto(char *, int);
int send_crypto_vocoder(char *, int);
int recv_vocoder_crypto(char *, int);
void xcrypt(P25_DES_SCHED_STATE *, char *);
void reset_dState(P25_DES_SCHED_STATE *);
void init_dState(P25_DES_SCHED_STATE *, char *, char *);
void init_lfsr(char *);
void lfsr_new_mi(char *); 
int crypto_init();
void crypto_reset();



#endif

