/****************************************************************************
Copyright 2007 Virginia Polytechnic Institute and State University
This file is part of the OSSIE __COMP_NAME__.
OSSIE __COMP_NAME__ is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
"(at your option) any later version
OSSIE __COMP_NAME__ is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with OSSIE __COMP_NAME__; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

/**
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 */

#ifndef MODEMDEVICE_IMPL_H
#define MODEMDEVICE_IMPL_H

#include <stdlib.h>
#include "ossie/cf.h"
#include "ossie/PortTypes.h"
#include "ossie/Resource_impl.h"
#include "ossie/debug.h"


#include "standardinterfaces/realChar.h"
#include "standardinterfaces/realChar_u.h"
#include "standardinterfaces/realChar_p.h"

/** \brief
 *
 *
 */
class ModemDevice_i : public virtual Resource_impl
{
  public:
    /** @brief Initializing constructor. */
    ModemDevice_i(const char *uuid, omni_condition *sem);

    /** @brief Destructor. */
    ~ModemDevice_i(void);

    /** @brief Static function for omni thread. */
    static void Run( void * data );

    /** @brief Static function for omni thread. */
    static void bs_Run( void * data );

    /** @brief Static function for omni thread. */
    static void ui_Run( void * data );

    ///
    void start() throw (CF::Resource::StartError, CORBA::SystemException);

    ///
    void stop() throw (CF::Resource::StopError, CORBA::SystemException);

    ///
    CORBA::Object_ptr getPort( const char* portName )
        throw (CF::PortSupplier::UnknownPort, CORBA::SystemException);

    ///
    void releaseObject() throw (CF::LifeCycle::ReleaseError, CORBA::SystemException);

    ///
    void initialize() throw (CF::LifeCycle::InitializeError, CORBA::SystemException);

	/** @brief Query properties from .prf.xml. */
	void query(CF::Properties&)
		throw (CORBA::SystemException,
				CF::UnknownProperties);

    /** @brief Configures properties read from .prf.xml. */
    void configure(const CF::Properties&)
        throw (CORBA::SystemException,
            CF::PropertySet::InvalidConfiguration,
            CF::PropertySet::PartialConfiguration);


  private:
    /** @brief Disallow default constructor. */
    ModemDevice_i();

    /** @brief Disallow copy constructor. */
    ModemDevice_i(ModemDevice_i&);

    /** @brief Main signal processing method. */
    void ProcessData();
	bool continue_processing();
	volatile bool thread_started;
	omni_mutex processing_mutex;

    /** @brief Bit stream signal processing method. */
    void bs_ProcessData();
	bool bs_continue_processing();
	volatile bool bs_thread_started;
	omni_mutex bs_processing_mutex;

    /** @brief UI signal processing method. */
    void ui_ProcessData();
	bool ui_continue_processing();
	volatile bool ui_thread_started;
	omni_mutex ui_processing_mutex;
   
    omni_condition *component_running;  ///< for component shutdown.
    omni_thread *processing_thread;     ///< for component writer function.
    omni_thread *bs_processing_thread;  ///< for component writer function.
    omni_thread *ui_processing_thread;	///< for component writer function.

    
    // list components provides and uses ports
    standardInterfaces_i::realChar_p *dataIn_0;		///< Incoming data from TX device to be transmitted.
    standardInterfaces_i::realChar_u *dataOut_0;	///< Outgoing data from a transmission to be sent to RX component.
    standardInterfaces_i::realChar_p *ui_dataIn;	///< Incoming data from UI device.
    standardInterfaces_i::realChar_u *ui_dataOut;	///< Outgoing data to UI device.
    
};
#endif
