/**
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 */

#include "host_gpp_intf.h"
#include "cntrl_types.h"
#include "p25/comm/tcplib.h"
#include "p25/util/logger.h"

/**
 * @details
 */
HostGPPInterface::HostGPPInterface(char *_ip, int _port) :
	ip(_ip), port(_port)
{
	host_connected = 0;
}

/**
 * @details
 */
HostGPPInterface::~HostGPPInterface(void)
{
	/* Disconnect from GPP if connection still active */
	gpp_disconnect();
}

/**
 * @details
 */
int HostGPPInterface::gpp_connect()
{
	if (!host_connected) {
		// Start TCP connection to GPP server
		INFO("Init TCP connection to %s:%d...", ip, port);
		connfd = start_tcp_client(ip, port);
		if (connfd != -1) {
			INFO("done.\n");
			host_connected = 1;
		}
		else {
			ERROR("fail!\n");
			host_connected = 0;
			return -1;
		}
	}
	else {
		INFO("Host already connected to GPP!\n");
	}

	return 0;
}

/**
 * @details
 */
int HostGPPInterface::gpp_disconnect()
{
	if (host_connected) {
		INFO("Stopping TCP client connection\n");
		stop_tcp_client(connfd);
		host_connected = 0;
	}

	return 0;
}



/**
 * @details
 */
enum host_ctrl_t HostGPPInterface::read_ctrl_pkt()
{
	int res;
	enum host_ctrl_t ctrl;

	if (host_connected) {
		res = read_tcp_server_stream(connfd, (char *)&ctrl, sizeof(int));
		if (res < 0) {
				ERROR("ERROR, reading from host ");
				host_connected = 0;
				return H_SOCK_ERROR;
		}
		else if (res == 0) {
				INFO("GPP disconnected.\n");
				host_connected = 0;
				return H_SOCK_ERROR;
		}
		else {
				if (ctrl == H_SOCK_ERROR) {
						INFO("Got H_SOCK_ERROR that should be HOST_UNKNOWN\n");
						ctrl = HOST_UNKNOWN;
				}
		}
	}
	else {
		ctrl = H_SOCK_ERROR;
	}

	return ctrl;
}

/**
 * @details
 */
int HostGPPInterface::read_data_payload(char *buf, int len)
{
	if (len == 0) return 0;
	int res = 0;

	if (host_connected) {
		res = read_tcp_server_stream(connfd, buf, len);
		if (res < 0) {
			ERROR("ERROR, reading from GPP ");
			host_connected = 0;
			return -1;
		}
		else if (res == 0) {
			INFO("GPP disconnected.\n");
			host_connected = 0;
			return -1;
		}
	}

	return res;
}

/**
 * @details
 */
int HostGPPInterface::write_data_payload(char *buf, int len)
{
	if (len == 0) return 0;
	int res = 0;

	if (host_connected) {
		res = send_tcp_client_data(connfd, buf, len);
		if (res < 0) {
			ERROR("ERROR, writing to GPP ");
			host_connected = 0;
			return -1;
		}
		else if (res == 0) {
			INFO("GPP disconnected.\n");
			host_connected = 0;
			return -1;
		}
	}

	return res;
}
