/**
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 */

#ifndef _HOST_GPP_INTF_H
#define _HOST_GPP_INTF_H

#include "cntrl_types.h"

/*
 * @brief Host to GPP Interface Class
 */
class HostGPPInterface {
	public:
		/** @brief Class Constructor. */
		HostGPPInterface(char *ip, int port);

		/** @brief Class Destructor. */
		~HostGPPInterface(void);

		/** @brief Open connection from Host to GPP. */
		int gpp_connect();

		/** @brief Close connection from Host to GPP. */
		int gpp_disconnect();

		/** @brief Reads control packet from GPP. */
		enum host_ctrl_t read_ctrl_pkt();

		/** @brief Reads data payload from GPP. */
		int read_data_payload(char *buf, int len);

		/** @brief Writes data payload to GPP. */
		int write_data_payload(char *buf, int len);

	private:
		/** @brief Host connection status variable. */
		int host_connected;

		/** @brief Disallow default constructor. */
		HostGPPInterface();

		/** @brief Socket file descriptor. */
		int connfd;

		/** @brief IP address pointer. */
		char *ip;

		/** @brief Port variable. */
		int port;
};

#endif
