/****************************************************************************
Copyright 2007 Virginia Polytechnic Institute and State University
This file is part of the OSSIE __COMP_NAME__.
OSSIE __COMP_NAME__ is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
"(at your option) any later version
OSSIE __COMP_NAME__ is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with OSSIE __COMP_NAME__; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

/**
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 */

#ifndef USERINTERFACEDEVICE_IMPL_H
#define USERINTERFACEDEVICE_IMPL_H

#include <stdlib.h>
#include <semaphore.h>

#include "ossie/cf.h"
#include "ossie/PortTypes.h"
#include "ossie/Resource_impl.h"
#include "ossie/debug.h"

#include "standardinterfaces/realChar.h"
#include "standardinterfaces/realChar_u.h"
#include "standardinterfaces/realChar_p.h"

/* user includes */
#include "ui.h"
#include "p25/format/packets.h"

/** \brief
 *
 *
 */
class UserInterfaceDevice_i : public virtual Resource_impl
{
  public:
    /** @brief Initializing constructor */
    UserInterfaceDevice_i(const char *uuid, omni_condition *sem);

    /** @brief Destructor */
    ~UserInterfaceDevice_i(void);

    /** @brief Static function for omni thread */
    static void Run( void * data );

    /** @brief Static function for scanning thread */
    static void scanning_Run( void * data );

    /** @brief */
    void start() throw (CF::Resource::StartError, CORBA::SystemException);

    /** @brief */
    void stop() throw (CF::Resource::StopError, CORBA::SystemException);

    /** @brief */
    CORBA::Object_ptr getPort( const char* portName )
        throw (CF::PortSupplier::UnknownPort, CORBA::SystemException);

    /** @brief */
    void releaseObject() throw (CF::LifeCycle::ReleaseError, CORBA::SystemException);

    /** @brief */
    void initialize() throw (CF::LifeCycle::InitializeError, CORBA::SystemException);

	/** @brief Query properties from .prf.xml */
	void query(CF::Properties&)
		throw (CORBA::SystemException,
				CF::UnknownProperties);

    /** @brief Configures properties read from .prf.xml */
    void configure(const CF::Properties&)
        throw (CORBA::SystemException,
            CF::PropertySet::InvalidConfiguration,
            CF::PropertySet::PartialConfiguration);


  private:
    /** @brief Disallow default constructor */
    UserInterfaceDevice_i();

    /** @brief Disallow copy constructor */
    UserInterfaceDevice_i(UserInterfaceDevice_i&);

    /** @brief Main signal processing method */
    void ProcessData();
	bool continue_processing();
	volatile bool thread_started;
	omni_mutex processing_mutex;

    /** @brief Main signal processing method */
    void scanning_ProcessData();
	bool scanning_continue_processing();
	volatile bool scanning_thread_started;
	omni_mutex scanning_processing_mutex;
   
    omni_condition *component_running;  ///< for component shutdown
    omni_thread *processing_thread;     ///< for component writer function
    omni_thread *scanning_thread;		///< for component scanning thread
    	
    CORBA::Short simple_0_value;

    
    /** @brief List of components provides and uses ports */
	standardInterfaces_i::realChar_p *rx_data_in;
	standardInterfaces_i::realChar_p *tx_data_in;
	standardInterfaces_i::realChar_u *tx_data_out;
	standardInterfaces_i::realChar_u *rx_data_out;
	standardInterfaces_i::realChar_u *modem_data_out;
	standardInterfaces_i::realChar_p *modem_data_in;


	/* user variables and prototypes */
	/** @brief Connection port for GUI */
	int 			uiPort;
	UI 				*_ui;


	/** @brief Receiver state variables */
	bool rx_running;
	bool scan_running;

	/** @brief Receiver state semaphores */
	sem_t receiver_pkt_stop;
	sem_t sem_rx_running;

	/** @brief UI processing thread */
	static void run_ui(void *arg);
	void tcp_task(void);
	void setPttMode(UInterface ui, int mode);
	bool ui_continue_processing();
	volatile bool ui_thread_started;
	omni_mutex ui_mutex;
	omni_thread *ui_thread;

};
#endif
