/**
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 */

#ifndef _UI_H
#define _UI_H

#include "p25/config.h"

/**
 * @brief User Interface class definitions. */
class UI
{
public:
	
	/**
 	 * @brief Constructor for User Interface class.
	 */
	UI(void);
	
	/**
	 * @brief Destructor for User Interface class.
	 */
	~UI(void);
	
	//	void *run_ui(void *);

	/**
	 * @brief Gets data off established TCP connection from GUI.
	 */
	int receive_ui_data(char *, int);
	
	/**
	 * @brief Sends data on established TCP connection to GUI.
	 */
	void send_ui_data(char *, int);

	/**
	 * @brief Update scan struct with data received from GUI.
	 */
	void scan_ui_update(SUInterface s_ui_cntrl);

	/**
	 * @brief Sends message to GUI indicating that a call has been found, and scanning should stop.
	 */
	void stop_gui_scan();

	/**
	 * @brief Updates GUI wtih received P25 variables.
	 */
	void r_ui_update(UInterface r_ui);

	/**
	 * @brief Gets list of frequencies to scan.
	 */
	short get_freqs(double *buf);
	
	/**
	 * @brief Starts a TCP socket, waits for a connection from the GUI.
	 */
	void ui_initialize(int ui_port);

	/**
	 * @brief Updates User Interface with received P25 variables.
	 */
	void ui_update(UInterface ui_cntrl);	

	/**
	 * @brief Terminates connection to GUI.
	 */
	void ui_shutdown();	
	
	/** 
	 * @brief Returns User Interface struct.
	 */
	UInterface get_ui(void);

	/**
	 * @brief Returns Scan struct.
	 */
	SUInterface get_s_ui(void);
	
	/**
	 * @brief Check to see if GUI is connected.
	 */
	int get_ui_connected(void);
	
private:

	/**
	 * @brief User Interface struct.
	 */
	UInterface      ui;

	/**
	 * @brief Scanning struct.
	 */
	SUInterface		s_ui;

	/** @brief GUI TCP connection. */
	int             ui_connFd;
	/** @brief GUI TCP socket. */
	int             ui_sock;
	/** @brief Connection state variable. */
	int             ui_connected;	
	
	RadioControl    rc;		// extern
};

#endif
