#ifndef _P25_FORMAT_BYTE_FIFO_H
#define _P25_FORMAT_BYTE_FIFO_H

#define BF_SIZE_DFT		1024
#define BF_SIZE_MAX		BF_SIZE_DFT*5

/**
 * @brief Byte FIFO struct
 */
struct byte_fifo { //DSP verison used unsigned int instead of int
	unsigned int	byte_size;  ///< Size of byte fifo in bytes
	unsigned int	head_index; ///< Head Index
	unsigned int	tail_index; ///< Tail Index
	char	*bytes; ///< Contents of byte fifo
};

/** @brief Alloc byte fifo, size in bytes, the actual size may be larger. */
struct byte_fifo *bf_alloc(int size);
/** @brief Free byte fifo */
void bf_free(struct byte_fifo *bf);
/** @brief Reset byte fifo */
void bf_reset(struct byte_fifo *bf);
/** @brief Check if byte fifo is empty */
int bf_empty(struct byte_fifo *bf);
/** @brief Check if byte fifo is full */
int bf_full(struct byte_fifo *bf);
/** @brief Get number of bytes in byte fifo */
int bf_size(struct byte_fifo *bf);
/** @brief Get available space in byte fifo */
int bf_space(struct byte_fifo *bf);

/* These access functions are dangerous. */
/** @brief Byte fifo access function for putting bytes into fifo */
void __bf_put(struct byte_fifo *bf, char b);
/** @brief Byte fifo access function for getting bytes from fifo */
char __bf_get(struct byte_fifo *bf);

/* Get n bytes at a time, start r/w from [0] */
/** @brief Put bytes into byte fifo */
int bf_put_n(struct byte_fifo *bf, char *bytes, int n);
/** @brief Get bytes from byte fifo */
int bf_get_n(struct byte_fifo *bf, char *bytes, int n);

/** @brief Dumps contents of byte fifo */
void bf_dump(struct byte_fifo *bf);

#endif

