/*
 * packets.h
 *
 *  Created on: Sep 15, 2011
 *      Author: sdr-overo
 */

#ifndef _P25_FORMAT_PACKETS_H_
#define _P25_FORMAT_PACKETS_H_

#include "p25/config.h"

/** @brief Block size */
#define BLK_SZ						8
/** @brief Size in bytes of data output from vocoder */
#define ECHANBYTES					11

/** @brief Packet header for Link Control */
#define CRYPTO_PKT_LC				0x00DD
/** @brief Packet header for Low Speed Data */
#define CRYPTO_PKT_LSD				0x00CC
/** @brief Packet header for HDU Message Indicator */
#define CRYPTO_PKT_MI_HDU			0x00BB
/** @brief Packet header for LDU Message Indicator */
#define CRYPTO_PKT_MI_LDU			0x00EE
/** @brief Packet header for Cryptography Start */
#define CRYPTO_PKT_CNTRL_START 		0x00AA
/** @brief Packet header for Cryptography Stop */
#define CRYPTO_PKT_CNTRL_STOP		0x00FF
/** @brief Packet header for Cryptography ACK message */
#define CRYPTO_PKT_CNTRL_ACK		0x11FF
/** @brief Packet header for Cryptography Data */
#define CRYPTO_PKT_CNTRL_DATA		0x22FF

/** @brief Packet header for Vocoder Start */
#define VOCODER_PKT_CNTRL_START		0x10AA
/** @brief Packet header for Vocoder Data */
#define VOCODER_PKT_CNTRL_DATA		0x10DD
/** @brief Packet header for Vocoder Stop */
#define VOCODER_PKT_CNTRL_STOP		0x10BB
/** @brief Packet header for Vocoder ACK message */
#define VOCODER_PKT_CNTRL_ACK		0x10CC

/** @brief Packet header for Modem buffer flush */
#define BS_OUTPUT_FLUSH				0x0900
/** @brief Packet header for Modem input buffer flush */
#define BS_INPUT_FLUSH				0x0909
/** @brief Packet header for Modem frequency change */
#define FREQ_CHANGE					0x0901

/** @brief Packet header for Tx/Rx Start */
#define START						0x0001
/** @brief Packet header for Tx/Rx Stop */
#define STOP						0x0002
/** @brief Packet header for Tx/Rx ACK message */
#define ACK							0x0007
/** @brief Packet header for UI update from Rx device */
#define UPDATE 						0x0005
/** @brief Packet header for flush request from Rx device */
#define FLUSH						0x0055

/** @brief Packet header for UI Scan Halt */
#define SCAN_HALT					0x0003
/** @brief Packet header for UI Scan Continue */
#define SCAN_CONTINUE				0x0004

/** @brief */
#define BLOB_LEN					60
/** @brief */
#define BLOB_LEN8					BLOB_LEN*8
/** @brief */
#define FIFO_LEN8					BLOB_LEN8*2

/** @brief */
#define BS_PKT_DATA					0x00DD
/** @brief */
#define BS_PKT_LAST					0x00EE

/**
 * @brief Bit Stream packet format.
 */
struct bs_packet {
	short hdr;           ///< Bit Stream packet header
	short blen;          ///< Bit Stream packet length
	char data[BLOB_LEN]; ///< Bit Stream packet data
};

/**
 * @brief Cryptography<-->Tx/Rx component packet format struct.
 */
typedef struct {
	short hdr;           ///< Cryptography-Tx/Rx packet header
	short len;           ///< Cryptography-Tx/Rx packet length
	char data[BLK_SZ];   ///< Cryptography-Tx/Rx packet data
} CryptoPacket;

/**
 * @brief Vocoder<-->Tx/Rx component packet format struct.
 */
typedef struct {
	short hdr;            ///< Vocoder-Tx/Rx packet header
	short len;            ///< Vocoder-Tx/Rx packet length
	char data[144*9];     ///< Vocoder-Tx/Rx packet data (9 voice code words x 144 bits/vcw)
} VocoderPacket;

/**
 * @brief Vocoder<-->Cryptography packet format struct.
 */
typedef struct {
	short hdr;             ///< Vocoder-Cryptography packet header
	short len;             ///< Vocoder-Cryptography packet length
	char data[ECHANBYTES]; ///< Vocoder-Cryptography packet data
} CryptVocPacket;

/**
 * @brief UserInterface<-->Tx/Rx packet format struct.
 */
typedef struct {
	short hdr;             ///< UI-Tx/Rx packet header
	UInterface ui;         ///< UI-Tx/Rx UserInterface struct
} UIControlPacket;

/**
 * @brief UserInterface<-->Modem packet format struct.
 */
typedef struct {
	short hdr;             ///< UI-Modem packet header
	double freq;           ///< UI-Modem frequency change (when applicable)
} ModemPacket;


#endif /* _P25_FORMAT_PACKETS_H_ */
