/******************************************************************************
     Name:  golay.c
   Author:  Anthony Nwokafor
     Date:  6/24/09
  History:  6/24/09  New
This module contains three functions:
	golay_24_12_encode ( word )    encodes (24,12,8) extended  Golay code
    golay_18_6_encode ( word )    encodes (18, 6,8) shortened Golay code
    golay_24_12_decode ( word )    decodes (24,12,8) extended  Golay code
    golay_18_6_decode ( word )    decodes (18, 6,8) shortened Golay code
The encode functions return a long int value that is 18 or 24 bits long
for the respective function.
The decode functions return an int value that is 6 or 12 bits long
for the respective function.
******************************************************************************/
#include <stdio.h>

#include "p25/ecc/golay.h"

long syndrome(long cw)
{
	int i,j;
	long tmp;
	long syn = 0;
	long bit_msk = 0x800000;
	
	for (i=0;i<K;i++) {
		bit_msk = 0x800000;
		tmp=cw&G[i];
		for (j=1;j<=24;j++) {
			syn^=(tmp&bit_msk) >> (24-j);
			bit_msk>>=1;
		}
		if (i+1 < K) { syn<<=1; }
	}
	
	return syn;
}

int wt(long syn) {
	int i;
	int weight = 0;
	
	for (i=0;i < K;i++) {
		if (syn & (1 << i)) {
			weight++;
		}
	}
	//printf("wt(%ld) = %d\n",syn,weight);
	return weight;
}

int sum(long bits) {
	int i;
	int s = 0;
	
	s = bits & 1;
	for (i=1;i<K;i++) {
		s ^= bits & (1 << i);
	}
	return s;
}

long golay_18_6_encode(int code_word_in)
{
    return ( golay_24_12_encode ( code_word_in & 077 ) & 0777777 );
}

long golay_24_12_encode(int code_word_in)
{
    long code_word_out = 0;
    long temp_word;
    int i;
    for(i=0;i<K;i++)
    {
        temp_word = code_word_in & code_word_mask[i];
        if(temp_word >= 1)
            code_word_out = code_word_out ^ G[i];
    }
    return(code_word_out);
}

int golay_24_12_decode(long *rc)
{
	int i;
	long tmp;
	long syn = 0;
	long synb = 0;
	long err = 0;
	int err_corrected = 0;
    long rcw = *rc;

	//printf("received = %ld\n",rcw);
	
	syn = syndrome(rcw);
	//printf("syn = %ld\n",syn);
	
	if ( wt(syn) <= 3 ) {
		err = syn << K;
		//printf("err <= 3, err = %ld\n",err);
		err_corrected = 1;
	} else {
		for (i=0;i < P;i++) {
			if (wt(syn ^ BT[i]) <= 2) {
				err = (((syn ^ BT[i]) << K) & 0xFFF000)  | (1 << (P-i-1));
				//printf("err > 3, found @ %d, err = %ld\n",i,err);
				err_corrected = 1;
				break;
			}
		}
		if (!err) {
			for (i=0;i<K;i++) {
				tmp = syn&BT[i];
				synb |= sum(tmp);
				synb = synb << 1;
			}
			if (wt(synb) <= 3) {
				err = (synb & 0x000FFF);
				err_corrected = 1;
			} else {
				for (i=0;i < P;i++) {
					if (wt(synb ^ B[i]) <= 20 ) {
						err = (1 << i) | ((synb ^ B[i])  & 0x000FFF) ;
						err_corrected = 1;
						break;
					}
				}
				if (!err) {
					err_corrected = 0;
				}
			}
		}
	}
	
	if (err_corrected) {
		rcw = (rcw ^ err) >> (N-K);
        *rc = rcw;
        return 0;
	} else {
        rcw = rcw >> (N-K);
        *rc = rcw;
		return 1;
	}
}

int golay_18_6_decode(long *rcw)
{
    int res;
    long rc = *rcw & 0777777;

    res = golay_24_12_decode ( &rc );
    *rcw = rc & 077;

    return res;
}


