#include <stdio.h>
#include <string.h>

#include "p25/config.h"
#include "p25/format/packets.h"
#include "p25/util/logger.h"
#include "p25/util/sys.h"

extern "C" {
    #include "p25/format/bit_fifo.h"
}

/* FIXME: optimize bit stream */

/* debug dumps
#ifdef DEBUG_DUMP
#define TX_DBG_FILE	"bs_tx_log.txt"
#define RX_DBG_FILE	"bs_rx_log.txt"
static FILE *tx_dbg_file;
static FILE *rx_dbg_file;
#endif
*/

static struct bit_fifo *bf_in;
static struct bit_fifo *bf_out;

/* debug dumps */
#ifdef DEBUG_DUMP
FILE *tx_fm_file, *rx_fm_file;
#endif


/**
 * @details 
 * 
 * @param bits	Array of bits, each bit is represented by a byte
 * @param n		Number of bits to read
 *
 * @return 		The number of bits read or -1 if error
 */
int bs_input(char *bits, unsigned int n)
{
	int ret;

	struct bs_packet pkt;
	bool received_tdu = false;

	if (n > BLOB_LEN8) {
		ERROR("bs_input n too large %d\n", n);
		n = BLOB_LEN8;
	}

	while ((unsigned int)bf_size(bf_in) < n) {
		ret = read_dsp(&pkt, sizeof(struct bs_packet));
		if (ret < 0) {
			ERROR("read_dsp failed\n");
			return -1;
		}

		if (pkt.hdr == BS_PKT_LAST) {
			INFO("Got TDU\n");
			received_tdu = true;
		}

		ret = bf_import(bf_in, pkt.data, pkt.blen);
		if (ret != pkt.blen) {
			ERROR("bf_import error\n");
			return -1;
		}

#ifdef DEBUG_DUMP
		int cnt = fwrite(pkt.data, 1, pkt.blen/8, rx_fm_file);
		if (cnt != pkt.blen/8)
			ERROR("Error writing to rx_fm_file!\n");
#endif

		if (received_tdu) {
			n = bf_size(bf_in);
			received_tdu = false;
		}
	}

	ret = bf_get_n(bf_in, bits, n);
	if ((unsigned int)ret != n) {
		ERROR("bf get error\n");
		return -1;
	}

/* debug dumps
#ifdef DEBUG_DUMP
	for (i = 0; i < n; i++)
		fprintf(rx_dbg_file, "%d\n", bits[i]);
#endif
*/

	return ret;
}

/**
 * @details
 * 
 * @param bits	Array of bits, each bit is represented by a byte
 * @param n		Number of bits to write
 *
 * @return 		The number of bits written or -1 if error
 */
int bs_output(char *bits, unsigned int n)
{
	int ret;

	struct bs_packet pkt;

	if (n > BLOB_LEN8) {
		ERROR("bs_output n too large %d\n", n);
		n = BLOB_LEN8;
	}

	ret = bf_put_n(bf_out, bits, n);
	if ((unsigned int)ret != n) {
		ERROR("bf put error\n");
		return -1;
	}

	while (bf_size(bf_out) >= BLOB_LEN8) {
		ret = bf_export(bf_out, pkt.data, BLOB_LEN8);
		if (ret != BLOB_LEN8) {
			ERROR("bf export error\n");
			return -1;
		}
		pkt.blen = BLOB_LEN8;

		ret = write_dsp(&pkt, sizeof(struct bs_packet));

		if (ret < 0) {
			ERROR("write dsp error\n");
			return -1;
		}

/* debug dumps */
#ifdef DEBUG_DUMP
		int cnt = fwrite(pkt.data, 1, pkt.blen/8, tx_fm_file);
		if (cnt != pkt.blen/8)
			ERROR("Error writing to tx_fm_file!\n");
#endif

	}
	/*
	for (i = 0; i < n; i++)
		fprintf(tx_dbg_file, "%d\n", bits[i]);
	*/
	return n;
}

/**
 * @details
 */
void bs_output_flush(void)
{
	int ret;
	struct bs_packet pkt;

	memset(pkt.data, 0, sizeof(pkt.data));
	pkt.blen = bf_size(bf_out);
	ASSERT(pkt.blen < BLOB_LEN8, "bf size is %d\n", pkt.blen);

	bf_export(bf_out, pkt.data, pkt.blen);

	ret = write_dsp(&pkt, sizeof(struct bs_packet));

	if (ret < 0)
		ERROR("write dsp error\n");

/* debug dumps */
#ifdef DEBUG_DUMP
	int cnt = fwrite(pkt.data, 1, pkt.blen/8, tx_fm_file);
	if (cnt != pkt.blen/8)
		ERROR("Error writing to tx_fm_file!\n");
#endif

}

/** 
 * @details
 *
 * @return 0 if success, -1 if failed
 */
int bs_init(void)
{
/* debug dumps
#ifdef DEBUG_DUMP
	tx_dbg_file = fopen(TX_DBG_FILE, "w+");
	rx_dbg_file = fopen(RX_DBG_FILE, "w+");
#endif
*/

	bf_in = bf_alloc(FIFO_LEN8);
	bf_out = bf_alloc(FIFO_LEN8);
	if (!bf_in || !bf_out) {
		ERROR("bf_alloc failed\n");
		return -1;
	}

/* debug dumps */
#ifdef DEBUG_DUMP
	tx_fm_file = fopen("tx_bit_dump.dat", "wb");
	if (tx_fm_file == NULL)
		ERROR("Error opening up tx_bit_dump.dat!\n");

	rx_fm_file = fopen("rx_bit_dump.dat", "wb");
	if (rx_fm_file == NULL)
		ERROR("Error opening up rx_bit_dump.dat!\n");
#endif

	return 0;
}

/**
 * @details
 */
void bs_exit(void)
{
	bf_free(bf_in);
	bf_free(bf_out);

/* debug dumps
#ifdef DEBUG_DUMP
	fclose(tx_dbg_file);
	fclose(rx_dbg_file);
#endif
*/
}
