#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "p25/comm/ipcomm.h"
#include "p25/util/logger.h"
#include "p25/util/sys.h"

/** @brief Channel for communication from modem to GPP */
P25::Comm::IPComm* modem2gpp_chan;
/** @brief Channel for communication from GPP to modem */
P25::Comm::IPComm* gpp2modem_chan;
/** @brief channel for communication from GPP to UHD drivers */
P25::Comm::IPComm* gpp2uhd_chan;

// tick-tock timer macros
#include <sys/time.h>
#include <iostream>
#define TICK() struct timeval tp1, tp2;						\
{															\
	gettimeofday(&tp1, NULL);								\
}

#define _TOCK(fmt, thresh)	{								\
	gettimeofday(&tp2, NULL);								\
	if (((tp2.tv_usec - tp1.tv_usec) / 1e3) > thresh) 		\
		std::cout << fmt << " time:  " << 					\
			(tp2.tv_usec - tp1.tv_usec) / 1e3 << " ms."     \
				<< std::endl;								\
}

#define TOCK(fmt)		_TOCK(fmt, 0)


typedef struct {
	unsigned char signal[8];
	double cfreq;
} control_pkt;

/**
 * @details
 *
 * @param fc  Carrier_frequency
 *
 * @return 0 if success, negative if error
 */
int set_fm_fc(double fc)
{
	int res;
	control_pkt ctrl_pkt;

	ctrl_pkt.signal[0] = 0xFC;
	ctrl_pkt.signal[1] = 0;
	ctrl_pkt.signal[2] = 0;
	ctrl_pkt.signal[3] = 0;
	ctrl_pkt.signal[4] = 0;
	ctrl_pkt.signal[5] = 0;
	ctrl_pkt.signal[6] = 0;
	ctrl_pkt.signal[7] = 0;
	ctrl_pkt.cfreq = fc;

	res = gpp2uhd_chan->send((char *)&ctrl_pkt, sizeof(ctrl_pkt));

	int errsv = errno;
	if (res != sizeof(ctrl_pkt)) {
		ERROR("Control Channel Error:  %d, %s\n", res, strerror(errsv));
		return -1;
	}

	return 0;
}

int dump_fpga(int select, int rate, char* name)
{
	return 0;
}

/** 
 * @details 
 *
 * @param	data read data buffer
 * @param	len  length to read, the length must be a multiple of 64
 *
 * @return 0 if success, negative if error
 */
int read_dsp(void *data, int len)
{
	int res;
	res = modem2gpp_chan->receive((char*)data, len);
//	INFO("Received %d bytes from Baseband\n",res);
	if (res != len) {
		ERROR("%d of %d bytes read\n",res,len);
		return -1;
	}

	return res;
}

/**
 * 
 * @param	data data buffer to write
 * @param	len  length to write, the length must be a multiple of 64
 *
 * @return 0 if success, negative if error
 */
int write_dsp(void* data, int len)
{
//	TICK()

	int res;
	res = gpp2modem_chan->send((char*)data, len);
	if (res != len) {
		ERROR("%d of %d bytes sent\n",res,len);
		return -1;
	}

//	TOCK("write_dsp")

	return res;
}

/*
 * @brief
 * @details	open DSP and FPGA devices and setup data communication channels.
 * @param	tx_spath	Transmitter communication socket path
 * @param	rx_spath	Receiver communication socket path
 * @return 0 if success, negative if error
 */
int sys_init(const char* tx_spath, const char* rx_spath, const char* fc_spath)
{
	gpp2modem_chan = new P25::Comm::IPComm(tx_spath);
	modem2gpp_chan = new P25::Comm::IPComm(rx_spath);
	gpp2uhd_chan = new P25::Comm::IPComm(fc_spath);

	gpp2modem_chan->start_connection(P25::Comm::IPComm::TXMode);
	modem2gpp_chan->start_connection(P25::Comm::IPComm::RXMode);
	gpp2uhd_chan->start_connection(P25::Comm::IPComm::TXMode);

	//set_fm_fc(671088640); // 15M
	//set_fm_fc(894784853); // 20M
	//set_fm_fc(883600043); // 19.75M

	return 0;
}

/*
 * @details
 */
void sys_exit(void)
{
	free(gpp2modem_chan);
	free(modem2gpp_chan);
	free(gpp2uhd_chan);

}
