/********************************************************************************
 *
 *	P25 Java Voice/Data Program Interface
 *	
 *	This program reads audio samples from the serial port, and allows the
 *	sending of text messages to the TCP net devices for transmission to the P25 
 *	radio components.
 *
 *	Copyright �2009, The Regents of the University of California
 *	All Rights Reserved 
 *
 *	This software is open for modification.  See details in the file 
 *	COPYRIGHT.doc in the root directory of the project.
 *
********************************************************************************/

/*
 * P25_Client_JFrame.java
 *
 * Created on August 6, 2009, 15:20 PM
 */


package p25_client;

import javax.swing.*;
import java.util.*;
import java.util.concurrent.*;
import java.awt.event.*;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.TargetDataLine;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.LineUnavailableException;
import java.net.*;
import java.lang.Integer;
import java.io.*;

/**
 *
 * @author  Jeffrey Cuenco
 */


public class P25_Client_JFrame extends javax.swing.JFrame {
    
    private byte i_buf[] = new byte[0];
    private byte o_buf[] = new byte[640];
    private ByteArrayOutputStream byte_out = new ByteArrayOutputStream ();
    private DataOutputStream data_out = new DataOutputStream (byte_out);
    private ByteArrayOutputStream byte_tmp = new ByteArrayOutputStream ();
    private DataOutputStream buf_tmp = new DataOutputStream (byte_tmp);
    private byte i_databuf[] = new byte[0];
    private Socket MyClient;
    private DatagramSocket NIClient;
    private Socket MyDataClient;
    private static AudioFormat audioFormat;
    private static AudioFormat audioFormat2;
    private static DataLine.Info td_info;
    private static DataLine.Info sd_info;
    private DataInputStream inputStream;
    private DataOutputStream outputStream;
    private DataInputStream dataInputStream;
    private DataOutputStream dataOutputStream;
    private AudioCaptureThread captureThread;
    private AudioPlaybackThread playbackThread;
    private DataReceiveThread dataReceiveThread;
    private P25FeedbackThread p25FeedbackThread;
    private static Thread timerThread;
    private static long bytesReceivedCount;
    private static long bytesSentCount;
    private static long dataBytesReceivedCount;
    private static long dataBytesSentCount;
    private Boolean resetTimer;
    private Boolean haltTimer;
    String AlgIDLastValue = "0x80";
    String KeyIDLastValue = "0x0000";
    String NACLastValue = "0x000";
    String MFIDLastValue = "0x0";
    String MI1LastValue = "0x000000000000000000";
    String TalkGroupIDLastValue = "0x0001";
    String SourceIDLastValue = "0x000001";
    String DestIDLastValue = "0x000002";
    
    /** Creates new form P25_Client_JFrame */
    public P25_Client_JFrame() {
        initComponents();
        MyClient = null;
        bytesReceivedCount = 0;
        bytesSentCount = 0;
        dataBytesReceivedCount = 0;
        dataBytesSentCount = 0;
        resetTimer = false;
        haltTimer = true;
    }
    
    class TimerThread extends Thread {    
        private long startTime;
        private long currTime;
        private long deltaTime;
        private String time;
        
	TimerThread() {
	}
	TimerThread(String threadName) {
		super(threadName); // Initialize thread.
		start();
	}

        public void resetTime() {
            startTime = System.currentTimeMillis();
            currTime = System.currentTimeMillis();
            deltaTime = 0;
        }
        
        public void getRunningTime() {
            long seconds = deltaTime / 1000;
            if (seconds > 0) {
                String sec;
                sec = "" + (seconds % 60);
                if (seconds % 60 < 10) {
                    sec = "0" + (seconds % 60);
                }
                String min;
                if (seconds > 60) {
                    min = ""+ (seconds / 60 % 60);
                    if ((seconds / 60 % 60) < 10) {
                        min = "0" + (seconds / 60 % 60);
                    }
                }
                else {
                    min = "00";
                }
                String hours;
                if (seconds / 60 > 60) {
                    hours = "" + (seconds / 60 / 60);
                    if((seconds / 60 / 60) < 10) {
                        hours = "0" + (seconds / 60 / 60);
                    }

                }
                else {
                    hours = "00";
                }

                time = "" + hours + ":" + min + ":" + sec;
            }
        }        
        
	public void run() {
            startTime = System.currentTimeMillis();
            while (true) 
            {
                if (haltTimer) {
                    try {
                        Thread.sleep(1000);
                    }
                    catch (InterruptedException e) {

                    }
                    resetTime();
                    continue;
                }
                
                currTime = System.currentTimeMillis();
                deltaTime = currTime - startTime;

                getRunningTime();
                ConnectionTime.setText(time);

                try {
                    Thread.sleep(1000);
                }
                catch (InterruptedException e) {

                }

                if (resetTimer) {
                    resetTime();
                    resetTimer = false;
                }
            }          
	}
    }    

    /** A new instance of this thread runs when capture is enabled
     * (talk mode). Talk mode can only be enabled when the socket is
     * connected. */
    class AudioCaptureThread extends Thread {
	Boolean running = true;
	TargetDataLine targetDataLine;
	
	AudioCaptureThread() {
	}
	AudioCaptureThread(String threadName) {
		super(threadName); // Initialize thread.
		start();
	}
	
	public void run() {
            int count;
            short sh;

            FM3TR_SetAudioCaptureRate();
            
	    try {
                targetDataLine = 
		    (TargetDataLine) AudioSystem.getLine(td_info);
                targetDataLine.open(audioFormat);
	    }
	    catch (LineUnavailableException e) {
		StatusMessageBox.append(
                    "Error: unable to get a recording line");
		return;
	    }
      
            targetDataLine.start();
            
            while (running) 
            {
                count = 0;
                count = targetDataLine.read(o_buf, 0, 640);
                
                if (count < 640)
                    StatusMessageBox.append(
			"Error: captured less than 640 bytes of audio");
               
		try {
                    data_out.flush();                    
                    byte_out.reset();

		    sh = (short)(4 + count);
                    data_out.writeShort(sh);
                    
		    if (P25_Talk_Button.isSelected())
			data_out.writeShort(0);
		    else {
			sh = (short)1;
                        data_out.writeShort(sh);
		    }
                    data_out.write(o_buf, 0, 640);

		    outputStream.write(byte_out.toByteArray(), 0, 644);
             //       System.out.println(System.currentTimeMillis());
		    bytesSentCount += count;
		    BytesSentLabel.setText(Long.toString(bytesSentCount));
		}
		catch (IOException e) {
		    //StatusMessageBox.append("Error: " + e + "\n");
		}
	    }          
            
            // send an EOM header just in case it wasn't sent on the last
            // voice packet
            try {
                sh = (short)(4 + 0);
                outputStream.writeShort(sh);

                sh = (short)1;
                outputStream.writeShort(sh);
            }
            catch (IOException e)
            {                
            }

            targetDataLine.stop();
	    targetDataLine.close();
	}
    }

    /** 
	 * This thread is running all the time 
	 */
    class AudioPlaybackThread extends Thread {
        Boolean running = true;
	SourceDataLine sourceDataLine;

        AudioPlaybackThread() {
        }
        AudioPlaybackThread(String threadName) {
                super(threadName); // Initialize thread.
                start();
        }

	public void run() {
            int count, length, flag;

            FM3TR_SetAudioPlaybackRate();
            
	    try {
                sourceDataLine = 
                        (SourceDataLine) AudioSystem.getLine(sd_info);
                sourceDataLine.open(audioFormat2);
	    }
	    catch (LineUnavailableException e) {
		StatusMessageBox.append(
			"Error: unable to get a playback line");
		return;
	    }          

            sourceDataLine.start();
            
            while (running) 
            {
		count = 0;
                if (P25_Voice_IsConnected() && inputStream != null) {
                    try {
                        length = inputStream.readShort();
                        flag = inputStream.readShort();

                        if (length-4 > 0) {
                            i_buf = new byte[length-4];
                            inputStream.readFully(i_buf, 0, length-4);
                            count = length - 4;

                            if (count > 0) {
                                if (count != 640)
                                System.out.println("count = " + count);

                                bytesReceivedCount += count;
                                BytesReceivedLabel.setText(
                                    Long.toString(bytesReceivedCount));
                                
                                sourceDataLine.write(i_buf, 0, count);
                            }
                        }
                        
                        if (flag == 1) {
                            StatusMessageBox.append("Voice EOM received\n");
                            sourceDataLine.drain();
                            sourceDataLine.stop();
                            sourceDataLine.close();
                            
                            try {
                                sourceDataLine = 
                                  (SourceDataLine) AudioSystem.getLine(sd_info);
                                sourceDataLine.open(audioFormat2);
                            }
                            catch (LineUnavailableException e) {
                                StatusMessageBox.append(
                                        "Error: unable to get a playback line");
                                return;
                            }                                      
                            sourceDataLine.start();
                        }
                    }
                    catch (IOException e) {
                        StatusMessageBox.append("Error: " + e + "\n");
                    }
                }
	    }
            sourceDataLine.stop();
            sourceDataLine.close();
	}
    }
   
    /** 
	 * This thread is instantiated per connection
	 */
    class DataReceiveThread extends Thread {
        Boolean running = true;
        
        DataReceiveThread() {
        }
        DataReceiveThread(String threadName) {
                super(threadName); // Initialize thread.
                start();
        }
        public void run() {
            int count, length, flag, src;
            String message = new String();
            
            while (running) 
            {
                src = 0;
                count = 0;
                message = "";

                try {
                    length = dataInputStream.readShort();
                    flag = dataInputStream.readShort();
                    
                    if (length - 4 > 0) {
                        i_databuf = new byte[length-4];
                        dataInputStream.readFully(i_databuf,0,length-4);
                        count = length-4;

                        src = i_databuf[0];
                        
                        // network byte order (big endian)
                        short len = (short)((i_databuf[3] & 0xFF) | 
                                ((i_databuf[2] & 0xFF) << 8));

                        for (int i = 4; i < len + 4; i++)
                            message += (char) i_databuf[i];
                    }
                    
                    if (flag == 1) {
                        StatusMessageBox.append("Data EOM received\n");
                    }                           
                }
                catch (IOException e) {
                    //StatusMessageBox.append("Error: " + e + "\n");
                }

                if (count > 0) {
                    dataBytesReceivedCount += (count+4);
                    DataBytesReceivedLabel.setText(
                        Long.toString(dataBytesReceivedCount));                    

                    TextMsgRcvBox.append("Radio " + src + ":  " + message 
                            + "\n");   
                    TextMsgRcvBox.setCaretPosition(
                            TextMsgRcvBox.getText().length());
                }         
            }
        }
    }    


    /** 
	 * This thread is running all the time 
	 */
    class P25FeedbackThread extends Thread {
        Boolean running = true;

        P25FeedbackThread() {
        }
        P25FeedbackThread(String threadName) {
                super(threadName); // Initialize thread.
                start();
        }

        /**
         * Convert the byte array to an int starting from the given offset.
         *
         * @param b The byte array
         * @param offset The array offset
         * @return The integer
         */
        public int byteArrayToInt(byte[] b, int offset) {
            int value = 0;
            for (int i = 0; i < 4; i++) {
                int shift = (4 - 1 - i) * 8;
                value += (b[i + offset] & 0x000000FF) << shift;
            }
            return value;
        }

		/** Receives data from UI component.  Uses received radio information such as 
		 * 		MI to update GUI elements.  The thread can receive two structs from
		 *		the UI component: a radio information struct appended with the
		 *		"p25_control_header" (0A and A0) or a scan struct appended with the
		 *		"p25_scan_header" (0B and B0).  The scan struct is received when
		 *		scanning is finished in the UI component, and used to update the scan button.
		 * 
		 * @author Justyn Bell <jwbell@ucsd.edu>
		 * @author Jeffrey Cuenco
		 */
        public void run() {
            int temp;
            double cfreq;
            int nac, tgid, keyid;
            int mfid, algid, ebit, ptt, tfmode;
            long uid, dstid;
            byte tmp[] = new byte[4];
            byte mi[] = new byte[9];
            byte tfpath[] = new byte[255];

            while (running)
            {
                if (P25_Voice_IsConnected() && inputStream != null) {
                    try {

                        inputStream.read(tmp,0,2);
                        if ((tmp[0] == (byte)0x0A) && (tmp[1] == (byte)0xA0)) {

    /*
                            temp = inputStream.readByte();
                            if (temp != (byte)0x0A)
                                continue;

                            temp = inputStream.readByte();
                            if (temp != (byte)0xA0)
                                continue;
    */
                            // Start receiving fields and printing them out in
                            // the textbox:
                            cfreq = inputStream.readDouble();
                            nac = inputStream.readShort() & 0xffff;
                            tgid = inputStream.readShort() & 0xffff;
                            keyid = inputStream.readShort() & 0xffff;
                            mfid = inputStream.readByte() & 0xff;
                            algid = inputStream.readByte() & 0xff;
                            ebit = inputStream.readByte() & 0xff;

                            //tmp[0] = (byte)0;
                            inputStream.read(tmp,1,3);
                            uid = byteArrayToInt(tmp, 0) & 0xffffffffL;

                            tmp[0] = (byte)0;
                            inputStream.read(tmp,1,3);
                            dstid = byteArrayToInt(tmp, 0) & 0xffffffffL;


                            inputStream.read(mi,0,9);

                            inputStream.read(tfpath,0,255);
                            tfmode = inputStream.readByte() & 0xff;

                            // If we're RX'ing data and we're in File mode.
                            if (tfmode == 0xF5) {
                                StatusMessageBox.append("Writing data to file: " +
                                        TestModeDirRx.getText() + "/" +
                                        tfpath.toString());
                            }

                            ptt = inputStream.readByte() & 0xff;

                            // If PTT state is in TX mode and RX mode PTT signal is
                            // received, then change state to RX mode.
                            Boolean PTTmodeTX = P25_Talk_Button.isSelected();
                            if (PTTmodeTX && (ptt == 0x00))
                                P25_Talk_Button.setSelected(false);
    /*
                            TextMsgRcvBox.append("NAC: " + nac + ", TGID: "
                                    + tgid + ", KeyID: " + keyid + ", MFID: "
                                    + mfid + ", ALGID: " + algid + ", EmBit: "
                                    + ebit + ", UnitID: " + uid + ", DestID: "
                                    + dstid + "\n");
                            TextMsgRcvBox.setCaretPosition(
                                TextMsgRcvBox.getText().length());
    */
                            StatusMessageBox.append("Maybe an error " + cfreq + "\n");
                            RFFreqLabel.setText(Double.toString(cfreq/1e6) + "MHz");

                            rALGID.setText("0x" +
                                    String.format("%02X", algid));
                            rDestID.setText("0x" +
                                    String.format("%06X", (int)dstid));

                            if (ebit == (byte)1)
                                rEmBit.setText("Yes");
                            else
                                rEmBit.setText("No");

                            rKeyID.setText("0x" +
                                    String.format("%04X", keyid));

                            rMFID.setText("0x" +
                                    String.format("%1X", mfid));

                            rNAC.setText("0x" +
                                    String.format("%03X", nac));

                            rTGID.setText("0x" +
                                    String.format("%04X", tgid));

                            rUnitID.setText("0x" +
                                    String.format("%06X", (int)uid));

                        } if ((tmp[0] == (byte)0x0B) && (tmp[1] == (byte)0xB0)) {
                            inputStream.read(tmp,0,1);
                            if (tmp[0] == 0x00) {
                                Scan_Button.setSelected(false);
                            } else
                                StatusMessageBox.append("Got weird state\n");
                        } else continue;
                    }
                    catch (Exception e) {
                        StatusMessageBox.append("Error: " + e + "\n");
                    }
                }
            }
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanel1 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        TextMsgRcvBox = new javax.swing.JTextArea();
        TextMsgSendBox = new javax.swing.JTextField();
        jLabel7 = new javax.swing.JLabel();
        jPanel4 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        P25_Radio_IP = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        P25_Radio_Port = new javax.swing.JTextField();
        jLabel6 = new javax.swing.JLabel();
        P25_RF_IP = new javax.swing.JTextField();
        jLabel29 = new javax.swing.JLabel();
        P25_RF_Port = new javax.swing.JTextField();
        jLabel28 = new javax.swing.JLabel();
        RFFreq = new javax.swing.JTextField();
        P25_Connect_Button = new javax.swing.JToggleButton();
        jPanel7 = new javax.swing.JPanel();
        P25_Talk_Button = new javax.swing.JToggleButton();
        jLabel31 = new javax.swing.JLabel();
        ToggleSelection = new javax.swing.JToggleButton();
        UseToggle = new java.awt.Checkbox();
        jPanel5 = new javax.swing.JPanel();
        jLabel18 = new javax.swing.JLabel();
        AlgID = new javax.swing.JTextField();
        KeyID = new javax.swing.JTextField();
        jLabel17 = new javax.swing.JLabel();
        jLabel20 = new javax.swing.JLabel();
        MFID = new javax.swing.JTextField();
        MI = new javax.swing.JTextField();
        jLabel16 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        TalkGroupID = new javax.swing.JTextField();
        RX_Enc_Button = new java.awt.Checkbox();
        jPanel10 = new javax.swing.JPanel();
        jLabel21 = new javax.swing.JLabel();
        UnitID = new javax.swing.JTextField();
        jLabel22 = new javax.swing.JLabel();
        DestID = new javax.swing.JTextField();
        jLabel19 = new javax.swing.JLabel();
        NAC = new javax.swing.JTextField();
        EmergencyBit = new java.awt.Checkbox();
        NoNACs = new java.awt.Checkbox();
        Scan_Button = new javax.swing.JToggleButton();
        jPanel11 = new javax.swing.JPanel();
        ConnectionTime = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        PTTMode = new javax.swing.JLabel();
        jLabel12 = new javax.swing.JLabel();
        RFFreqLabel = new javax.swing.JLabel();
        jPanel8 = new javax.swing.JPanel();
        jLabel13 = new javax.swing.JLabel();
        jLabel14 = new javax.swing.JLabel();
        jLabel15 = new javax.swing.JLabel();
        rNAC = new javax.swing.JLabel();
        rTGID = new javax.swing.JLabel();
        rKeyID = new javax.swing.JLabel();
        jLabel23 = new javax.swing.JLabel();
        rMFID = new javax.swing.JLabel();
        rALGID = new javax.swing.JLabel();
        rEmBit = new javax.swing.JLabel();
        jLabel24 = new javax.swing.JLabel();
        jLabel25 = new javax.swing.JLabel();
        jLabel26 = new javax.swing.JLabel();
        rDestID = new javax.swing.JLabel();
        rUnitID = new javax.swing.JLabel();
        jLabel27 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jPanel6 = new javax.swing.JPanel();
        StatusMessageBoxClearButton = new java.awt.Button();
        jScrollPane2 = new javax.swing.JScrollPane();
        StatusMessageBox = new javax.swing.JTextArea();
        jPanel9 = new javax.swing.JPanel();
        DataBytesReceivedLabel = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        BytesReceivedLabel = new javax.swing.JLabel();
        BytesSentLabel = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel11 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        DataBytesSentLabel = new javax.swing.JLabel();
        jPanel12 = new javax.swing.JPanel();
        TestModeCheckBox = new javax.swing.JCheckBox();
        TestModeFile = new javax.swing.JTextField();
        TestModeChooseFileBtn = new javax.swing.JButton();
        jLabel30 = new javax.swing.JLabel();
        jLabel32 = new javax.swing.JLabel();
        TestModeDirRx = new javax.swing.JTextField();
        TestModeChooseDirRxBtn = new javax.swing.JButton();
        jLabel33 = new javax.swing.JLabel();
        jScrollPane3 = new javax.swing.JScrollPane();
        ScanningFreqs = new javax.swing.JTextArea();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        jMenu2 = new javax.swing.JMenu();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("P25 Client (Apr. 11, 2011)");
        addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                formKeyPressed(evt);
            }
            public void keyReleased(java.awt.event.KeyEvent evt) {
                formKeyReleased(evt);
            }
        });

        jTabbedPane1.setMaximumSize(new java.awt.Dimension(743, 616));

        jPanel1.setMaximumSize(new java.awt.Dimension(722, 570));
        jPanel1.setMinimumSize(new java.awt.Dimension(722, 570));

        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder("Text Messaging"));
        jPanel3.setEnabled(false);
        jPanel3.setMaximumSize(new java.awt.Dimension(722, 230));
        jPanel3.setMinimumSize(new java.awt.Dimension(722, 230));
        jPanel3.setPreferredSize(new java.awt.Dimension(722, 230));

        TextMsgRcvBox.setColumns(20);
        TextMsgRcvBox.setEditable(false);
        TextMsgRcvBox.setLineWrap(true);
        TextMsgRcvBox.setRows(5);
        TextMsgRcvBox.setEnabled(false);
        TextMsgRcvBox.setMaximumSize(new java.awt.Dimension(240, 80));
        TextMsgRcvBox.setMinimumSize(new java.awt.Dimension(240, 80));
        jScrollPane1.setViewportView(TextMsgRcvBox);

        TextMsgSendBox.setText("Type your message here, <Enter> to send.");
        TextMsgSendBox.setEnabled(false);
        TextMsgSendBox.setMaximumSize(new java.awt.Dimension(670, 28));
        TextMsgSendBox.setMinimumSize(new java.awt.Dimension(670, 28));
        TextMsgSendBox.setPreferredSize(new java.awt.Dimension(670, 28));
        TextMsgSendBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                TextMsgSendBoxActionPerformed(evt);
            }
        });
        TextMsgSendBox.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                TextMsgSendBoxFocusGained(evt);
            }
            public void focusLost(java.awt.event.FocusEvent evt) {
                TextMsgSendBoxFocusLost(evt);
            }
        });

        jLabel7.setText("Must be in Listen mode to send/receive text messages (piggybacked over Low-Speed Data fields)");

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 689, Short.MAX_VALUE)
                    .addComponent(jLabel7)
                    .addComponent(TextMsgSendBox, javax.swing.GroupLayout.DEFAULT_SIZE, 689, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addComponent(jLabel7)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 122, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(TextMsgSendBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        jPanel4.setBorder(javax.swing.BorderFactory.createTitledBorder("Connection Settings"));
        jPanel4.setMaximumSize(new java.awt.Dimension(416, 140));
        jPanel4.setMinimumSize(new java.awt.Dimension(416, 140));
        jPanel4.setName(""); // NOI18N
        jPanel4.setPreferredSize(new java.awt.Dimension(416, 140));

        jLabel1.setLabelFor(P25_Radio_IP);
        jLabel1.setText("Radio IP:");
        jLabel1.setMaximumSize(new java.awt.Dimension(64, 16));
        jLabel1.setMinimumSize(new java.awt.Dimension(64, 16));
        jLabel1.setPreferredSize(new java.awt.Dimension(64, 16));

        P25_Radio_IP.setText("127.0.0.1");
        P25_Radio_IP.setMaximumSize(new java.awt.Dimension(157, 28));
        P25_Radio_IP.setMinimumSize(new java.awt.Dimension(157, 28));
        P25_Radio_IP.setPreferredSize(new java.awt.Dimension(157, 28));

        jLabel3.setText("Port:");

        P25_Radio_Port.setText("8002");
        P25_Radio_Port.setMaximumSize(new java.awt.Dimension(78, 28));
        P25_Radio_Port.setMinimumSize(new java.awt.Dimension(78, 28));
        P25_Radio_Port.setPreferredSize(new java.awt.Dimension(78, 28));
        P25_Radio_Port.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                P25_Radio_PortActionPerformed(evt);
            }
        });

        jLabel6.setText("NI RF IP:");
        jLabel6.setEnabled(false);
        jLabel6.setMaximumSize(new java.awt.Dimension(64, 16));
        jLabel6.setMinimumSize(new java.awt.Dimension(64, 16));
        jLabel6.setPreferredSize(new java.awt.Dimension(64, 16));

        P25_RF_IP.setText("10.10.10.155");
        P25_RF_IP.setEnabled(false);
        P25_RF_IP.setMaximumSize(new java.awt.Dimension(157, 28));
        P25_RF_IP.setMinimumSize(new java.awt.Dimension(157, 28));
        P25_RF_IP.setPreferredSize(new java.awt.Dimension(157, 28));

        jLabel29.setText("Port:");
        jLabel29.setEnabled(false);

        P25_RF_Port.setText("9001");
        P25_RF_Port.setEnabled(false);
        P25_RF_Port.setMaximumSize(new java.awt.Dimension(78, 28));
        P25_RF_Port.setMinimumSize(new java.awt.Dimension(78, 28));
        P25_RF_Port.setPreferredSize(new java.awt.Dimension(78, 28));
        P25_RF_Port.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                P25_RF_PortActionPerformed(evt);
            }
        });

        jLabel28.setText("RF Freq:");
        jLabel28.setMaximumSize(new java.awt.Dimension(64, 16));
        jLabel28.setMinimumSize(new java.awt.Dimension(64, 16));
        jLabel28.setPreferredSize(new java.awt.Dimension(64, 16));

        RFFreq.setText("139.75MHz");
        RFFreq.setMaximumSize(new java.awt.Dimension(157, 28));
        RFFreq.setMinimumSize(new java.awt.Dimension(157, 28));
        RFFreq.setPreferredSize(new java.awt.Dimension(157, 28));
        RFFreq.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                RFFreqActionPerformed(evt);
            }
        });

        P25_Connect_Button.setLabel("Connect");
        P25_Connect_Button.setMaximumSize(new java.awt.Dimension(128, 29));
        P25_Connect_Button.setMinimumSize(new java.awt.Dimension(128, 29));
        P25_Connect_Button.setPreferredSize(new java.awt.Dimension(128, 29));
        P25_Connect_Button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                P25_Connect_ButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel6, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel28, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(8, 8, 8)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(P25_RF_IP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(P25_Radio_IP, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(RFFreq, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel4Layout.createSequentialGroup()
                        .addComponent(jLabel3)
                        .addGap(18, 18, 18)
                        .addComponent(P25_Radio_Port, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(jPanel4Layout.createSequentialGroup()
                        .addComponent(jLabel29)
                        .addGap(18, 18, 18)
                        .addComponent(P25_RF_Port, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(P25_Connect_Button, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(P25_Radio_IP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel3)
                    .addComponent(P25_Radio_Port, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel6, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel29)
                    .addComponent(P25_RF_Port, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(P25_RF_IP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(7, 7, 7)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel28, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(RFFreq, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(P25_Connect_Button, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        P25_Connect_Button.getAccessibleContext().setAccessibleName("");

        jPanel7.setBorder(javax.swing.BorderFactory.createTitledBorder("Push To Talk Control"));
        jPanel7.setMaximumSize(new java.awt.Dimension(306, 114));
        jPanel7.setMinimumSize(new java.awt.Dimension(306, 114));
        jPanel7.setPreferredSize(new java.awt.Dimension(306, 114));

        P25_Talk_Button.setText("Push To Talk");
        P25_Talk_Button.setToolTipText("Toggles Talk/Listen modes");
        P25_Talk_Button.setEnabled(false);
        P25_Talk_Button.setMaximumSize(new java.awt.Dimension(260, 23));
        P25_Talk_Button.setMinimumSize(new java.awt.Dimension(260, 23));
        P25_Talk_Button.setPreferredSize(new java.awt.Dimension(260, 23));
        P25_Talk_Button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                P25_Talk_ButtonActionPerformed(evt);
            }
        });

        jLabel31.setFont(new java.awt.Font("Lucida Grande", 0, 8));
        jLabel31.setText("<html>Check 'Use Toggle' button to simulate a PTT button depress with<br>the SPACEBAR when PTT is highlighted.</html>");
        jLabel31.setMaximumSize(new java.awt.Dimension(253, 20));
        jLabel31.setMinimumSize(new java.awt.Dimension(253, 20));
        jLabel31.setPreferredSize(new java.awt.Dimension(253, 20));

        ToggleSelection.setText("PTT");
        ToggleSelection.setToolTipText("Choose Toggle Button");
        ToggleSelection.setEnabled(false);
        ToggleSelection.setMaximumSize(new java.awt.Dimension(140, 23));
        ToggleSelection.setMinimumSize(new java.awt.Dimension(140, 23));
        ToggleSelection.setPreferredSize(new java.awt.Dimension(140, 23));
        ToggleSelection.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                ToggleSelectionKeyPressed(evt);
            }
            public void keyReleased(java.awt.event.KeyEvent evt) {
                ToggleSelectionKeyReleased(evt);
            }
        });

        UseToggle.setEnabled(false);
        UseToggle.setLabel("Use Toggle:");
        UseToggle.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                UseToggleItemStateChanged(evt);
            }
        });

        javax.swing.GroupLayout jPanel7Layout = new javax.swing.GroupLayout(jPanel7);
        jPanel7.setLayout(jPanel7Layout);
        jPanel7Layout.setHorizontalGroup(
            jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel7Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel31, javax.swing.GroupLayout.DEFAULT_SIZE, 272, Short.MAX_VALUE)
                    .addComponent(P25_Talk_Button, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(jPanel7Layout.createSequentialGroup()
                        .addComponent(UseToggle, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(19, 19, 19)
                        .addComponent(ToggleSelection, javax.swing.GroupLayout.PREFERRED_SIZE, 141, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        jPanel7Layout.setVerticalGroup(
            jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel7Layout.createSequentialGroup()
                .addComponent(P25_Talk_Button, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(7, 7, 7)
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(UseToggle, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(ToggleSelection, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel31, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        ToggleSelection.getAccessibleContext().setAccessibleName("");

        jPanel5.setBorder(javax.swing.BorderFactory.createTitledBorder("P25 Header Settings"));
        jPanel5.setMaximumSize(new java.awt.Dimension(306, 140));
        jPanel5.setMinimumSize(new java.awt.Dimension(306, 140));
        jPanel5.setPreferredSize(new java.awt.Dimension(306, 140));

        jLabel18.setText("ALGID:");
        jLabel18.setToolTipText("Algorithm ID (if encryption enabled)");

        AlgID.setText("0x80");
        AlgID.setMaximumSize(new java.awt.Dimension(70, 28));
        AlgID.setMinimumSize(new java.awt.Dimension(70, 28));
        AlgID.setPreferredSize(new java.awt.Dimension(70, 28));
        AlgID.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                AlgIDFocusLost(evt);
            }
        });

        KeyID.setText("0x0000");
        KeyID.setMaximumSize(new java.awt.Dimension(70, 28));
        KeyID.setMinimumSize(new java.awt.Dimension(70, 28));
        KeyID.setPreferredSize(new java.awt.Dimension(70, 28));
        KeyID.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                KeyIDFocusLost(evt);
            }
        });

        jLabel17.setText("KEYID:");
        jLabel17.setToolTipText("Key ID (if encryption enabled)");

        jLabel20.setText("MFID:");
        jLabel20.setToolTipText("Manufacturer ID");
        jLabel20.setMaximumSize(new java.awt.Dimension(43, 16));
        jLabel20.setMinimumSize(new java.awt.Dimension(43, 16));
        jLabel20.setPreferredSize(new java.awt.Dimension(43, 16));

        MFID.setText("0x0");
        MFID.setMaximumSize(new java.awt.Dimension(70, 28));
        MFID.setMinimumSize(new java.awt.Dimension(70, 28));
        MFID.setPreferredSize(new java.awt.Dimension(70, 28));
        MFID.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                MFIDFocusLost(evt);
            }
        });

        MI.setText("0x000000000000000000");
        MI.setMaximumSize(new java.awt.Dimension(203, 28));
        MI.setMinimumSize(new java.awt.Dimension(203, 28));
        MI.setPreferredSize(new java.awt.Dimension(205, 28));
        MI.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                MIFocusLost(evt);
            }
        });

        jLabel16.setText("MI:");
        jLabel16.setToolTipText("Message Indicator (000000000 if no encryption)");
        jLabel16.setMaximumSize(new java.awt.Dimension(43, 16));
        jLabel16.setMinimumSize(new java.awt.Dimension(43, 16));
        jLabel16.setPreferredSize(new java.awt.Dimension(43, 16));

        jLabel9.setText("TGID:");
        jLabel9.setToolTipText("Talk Group ID");
        jLabel9.setMaximumSize(new java.awt.Dimension(41, 16));
        jLabel9.setMinimumSize(new java.awt.Dimension(41, 16));
        jLabel9.setPreferredSize(new java.awt.Dimension(41, 16));

        TalkGroupID.setText("0x0001");
        TalkGroupID.setMaximumSize(new java.awt.Dimension(70, 28));
        TalkGroupID.setMinimumSize(new java.awt.Dimension(70, 28));
        TalkGroupID.setPreferredSize(new java.awt.Dimension(70, 28));
        TalkGroupID.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                TalkGroupIDFocusLost(evt);
            }
        });

        RX_Enc_Button.setFont(new java.awt.Font("Dialog", 0, 10));
        RX_Enc_Button.setLabel("Force ALGID for incoming calls");

        javax.swing.GroupLayout jPanel5Layout = new javax.swing.GroupLayout(jPanel5);
        jPanel5.setLayout(jPanel5Layout);
        jPanel5Layout.setHorizontalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel5Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel20, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addComponent(jLabel16, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(jLabel18)))
                .addGap(12, 12, 12)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(RX_Enc_Button, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(jPanel5Layout.createSequentialGroup()
                        .addComponent(MFID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(12, 12, 12)
                        .addComponent(jLabel9, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(12, 12, 12)
                        .addComponent(TalkGroupID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(MI, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(jPanel5Layout.createSequentialGroup()
                        .addComponent(AlgID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(12, 12, 12)
                        .addComponent(jLabel17)
                        .addGap(12, 12, 12)
                        .addComponent(KeyID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(18, Short.MAX_VALUE))
        );
        jPanel5Layout.setVerticalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel5Layout.createSequentialGroup()
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel20, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(MFID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel9, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(TalkGroupID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel16, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(MI, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel18)
                    .addComponent(AlgID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel17)
                    .addComponent(KeyID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(RX_Enc_Button, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        jPanel10.setBorder(javax.swing.BorderFactory.createTitledBorder("Link Control Settings"));
        jPanel10.setDoubleBuffered(false);
        jPanel10.setMaximumSize(new java.awt.Dimension(416, 114));
        jPanel10.setMinimumSize(new java.awt.Dimension(416, 114));
        jPanel10.setPreferredSize(new java.awt.Dimension(416, 114));

        jLabel21.setText("SRCID:");
        jLabel21.setToolTipText("Source ID / Unit ID");
        jLabel21.setMaximumSize(new java.awt.Dimension(64, 16));
        jLabel21.setMinimumSize(new java.awt.Dimension(64, 16));
        jLabel21.setPreferredSize(new java.awt.Dimension(64, 16));

        UnitID.setText("0x000001");
        UnitID.setMaximumSize(new java.awt.Dimension(86, 28));
        UnitID.setMinimumSize(new java.awt.Dimension(86, 28));
        UnitID.setPreferredSize(new java.awt.Dimension(86, 28));
        UnitID.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                UnitIDFocusLost(evt);
            }
        });

        jLabel22.setText("DSTID:");
        jLabel22.setToolTipText("Destination ID");
        jLabel22.setMaximumSize(new java.awt.Dimension(64, 16));
        jLabel22.setMinimumSize(new java.awt.Dimension(64, 16));
        jLabel22.setPreferredSize(new java.awt.Dimension(64, 16));

        DestID.setText("0x000002");
        DestID.setPreferredSize(new java.awt.Dimension(86, 28));
        DestID.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                DestIDFocusLost(evt);
            }
        });

        jLabel19.setText("NAC:");
        jLabel19.setToolTipText("Network Access Code");

        NAC.setText("0x293");
        NAC.setMaximumSize(new java.awt.Dimension(67, 28));
        NAC.setMinimumSize(new java.awt.Dimension(67, 28));
        NAC.setPreferredSize(new java.awt.Dimension(67, 28));
        NAC.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                NACFocusLost(evt);
            }
        });

        EmergencyBit.setLabel("Emergency Bit");

        NoNACs.setLabel("No NACs");
        NoNACs.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                NoNACsItemStateChanged(evt);
            }
        });

        Scan_Button.setEnabled(false);
        Scan_Button.setLabel("Scan");
        Scan_Button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                Scan_ButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel10Layout = new javax.swing.GroupLayout(jPanel10);
        jPanel10.setLayout(jPanel10Layout);
        jPanel10Layout.setHorizontalGroup(
            jPanel10Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel10Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel10Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel21, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel22, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel10Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(UnitID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(DestID, javax.swing.GroupLayout.PREFERRED_SIZE, 86, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel10Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel10Layout.createSequentialGroup()
                        .addComponent(jLabel19)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(NAC, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(EmergencyBit, javax.swing.GroupLayout.PREFERRED_SIZE, 124, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel10Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(Scan_Button)
                    .addComponent(NoNACs, javax.swing.GroupLayout.PREFERRED_SIZE, 69, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(17, Short.MAX_VALUE))
        );
        jPanel10Layout.setVerticalGroup(
            jPanel10Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel10Layout.createSequentialGroup()
                .addGroup(jPanel10Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel10Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel21, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(UnitID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(jPanel10Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel19)
                        .addComponent(NAC, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(NoNACs, javax.swing.GroupLayout.PREFERRED_SIZE, 28, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel10Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(Scan_Button)
                    .addGroup(jPanel10Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addComponent(EmergencyBit, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(DestID, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(jLabel22, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(23, Short.MAX_VALUE))
        );

        jPanel11.setBorder(javax.swing.BorderFactory.createTitledBorder("Radio Status"));
        jPanel11.setMaximumSize(new java.awt.Dimension(300, 86));
        jPanel11.setMinimumSize(new java.awt.Dimension(300, 86));
        jPanel11.setPreferredSize(new java.awt.Dimension(300, 86));

        ConnectionTime.setText("00:00:00");
        ConnectionTime.setMaximumSize(new java.awt.Dimension(100, 16));
        ConnectionTime.setMinimumSize(new java.awt.Dimension(100, 16));
        ConnectionTime.setPreferredSize(new java.awt.Dimension(100, 16));

        jLabel2.setText("PTT Mode:");
        jLabel2.setToolTipText("Sets Push-To-Talk modes.  Start to connect to radio, and then Listen for RX operation, and Talk for TX operation.");
        jLabel2.setMaximumSize(new java.awt.Dimension(111, 16));
        jLabel2.setMinimumSize(new java.awt.Dimension(111, 16));
        jLabel2.setPreferredSize(new java.awt.Dimension(111, 16));

        jLabel8.setText("Connection Time:");

        PTTMode.setText("Listen Mode");
        PTTMode.setToolTipText("Sets Push-To-Talk modes.  Start to connect to radio, and then Listen for RX operation, and Talk for TX operation.");
        PTTMode.setMaximumSize(new java.awt.Dimension(100, 16));
        PTTMode.setMinimumSize(new java.awt.Dimension(100, 16));
        PTTMode.setPreferredSize(new java.awt.Dimension(100, 16));

        jLabel12.setText("RF Freq:");
        jLabel12.setToolTipText("Sets Push-To-Talk modes.  Start to connect to radio, and then Listen for RX operation, and Talk for TX operation.");
        jLabel12.setMaximumSize(new java.awt.Dimension(111, 16));
        jLabel12.setMinimumSize(new java.awt.Dimension(111, 16));
        jLabel12.setPreferredSize(new java.awt.Dimension(111, 16));

        RFFreqLabel.setText("139.75MHz");
        RFFreqLabel.setToolTipText("Sets Push-To-Talk modes.  Start to connect to radio, and then Listen for RX operation, and Talk for TX operation.");
        RFFreqLabel.setMaximumSize(new java.awt.Dimension(100, 16));
        RFFreqLabel.setMinimumSize(new java.awt.Dimension(100, 16));
        RFFreqLabel.setPreferredSize(new java.awt.Dimension(100, 16));

        javax.swing.GroupLayout jPanel11Layout = new javax.swing.GroupLayout(jPanel11);
        jPanel11.setLayout(jPanel11Layout);
        jPanel11Layout.setHorizontalGroup(
            jPanel11Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel11Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel11Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(jPanel11Layout.createSequentialGroup()
                        .addGroup(jPanel11Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel8)
                            .addComponent(jLabel12, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(36, 36, 36)
                        .addGroup(jPanel11Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(RFFreqLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(PTTMode, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(ConnectionTime, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addContainerGap())
        );
        jPanel11Layout.setVerticalGroup(
            jPanel11Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel11Layout.createSequentialGroup()
                .addGroup(jPanel11Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel8, javax.swing.GroupLayout.PREFERRED_SIZE, 16, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(ConnectionTime, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(5, 5, 5)
                .addGroup(jPanel11Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(PTTMode, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(5, 5, 5)
                .addGroup(jPanel11Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel12, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(RFFreqLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
        );

        jPanel8.setBorder(javax.swing.BorderFactory.createTitledBorder("Received Frame Fields"));
        jPanel8.setMaximumSize(new java.awt.Dimension(422, 86));
        jPanel8.setMinimumSize(new java.awt.Dimension(422, 86));
        jPanel8.setPreferredSize(new java.awt.Dimension(422, 86));

        jLabel13.setText("NAC:");
        jLabel13.setMaximumSize(new java.awt.Dimension(40, 16));
        jLabel13.setMinimumSize(new java.awt.Dimension(40, 16));
        jLabel13.setPreferredSize(new java.awt.Dimension(40, 16));

        jLabel14.setText("TGID:");
        jLabel14.setMaximumSize(new java.awt.Dimension(40, 16));
        jLabel14.setMinimumSize(new java.awt.Dimension(40, 16));
        jLabel14.setPreferredSize(new java.awt.Dimension(40, 16));

        jLabel15.setText("KeyID:");

        rNAC.setText("0x000");
        rNAC.setMaximumSize(new java.awt.Dimension(60, 16));
        rNAC.setMinimumSize(new java.awt.Dimension(60, 16));
        rNAC.setPreferredSize(new java.awt.Dimension(60, 16));

        rTGID.setText("0x0000");
        rTGID.setMaximumSize(new java.awt.Dimension(60, 16));
        rTGID.setMinimumSize(new java.awt.Dimension(60, 16));
        rTGID.setPreferredSize(new java.awt.Dimension(60, 16));

        rKeyID.setText("0x0000");
        rKeyID.setMaximumSize(new java.awt.Dimension(60, 16));
        rKeyID.setMinimumSize(new java.awt.Dimension(60, 16));
        rKeyID.setPreferredSize(new java.awt.Dimension(60, 16));

        jLabel23.setText("MFID:");
        jLabel23.setMaximumSize(new java.awt.Dimension(45, 16));
        jLabel23.setMinimumSize(new java.awt.Dimension(45, 16));
        jLabel23.setPreferredSize(new java.awt.Dimension(45, 16));

        rMFID.setText("0x0");
        rMFID.setMaximumSize(new java.awt.Dimension(45, 16));
        rMFID.setMinimumSize(new java.awt.Dimension(45, 16));
        rMFID.setPreferredSize(new java.awt.Dimension(45, 16));

        rALGID.setText("0x00");
        rALGID.setMaximumSize(new java.awt.Dimension(45, 16));
        rALGID.setMinimumSize(new java.awt.Dimension(45, 16));
        rALGID.setPreferredSize(new java.awt.Dimension(45, 16));

        rEmBit.setText("No");
        rEmBit.setMaximumSize(new java.awt.Dimension(45, 16));
        rEmBit.setMinimumSize(new java.awt.Dimension(45, 16));
        rEmBit.setPreferredSize(new java.awt.Dimension(45, 16));

        jLabel24.setText("EmBit:");
        jLabel24.setMaximumSize(new java.awt.Dimension(45, 16));
        jLabel24.setMinimumSize(new java.awt.Dimension(45, 16));
        jLabel24.setPreferredSize(new java.awt.Dimension(45, 16));

        jLabel25.setText("ALGID:");
        jLabel25.setMaximumSize(new java.awt.Dimension(45, 16));
        jLabel25.setMinimumSize(new java.awt.Dimension(45, 16));
        jLabel25.setPreferredSize(new java.awt.Dimension(45, 16));

        jLabel26.setText("UnitID:");
        jLabel26.setMaximumSize(new java.awt.Dimension(48, 16));
        jLabel26.setMinimumSize(new java.awt.Dimension(48, 16));
        jLabel26.setPreferredSize(new java.awt.Dimension(48, 16));

        rDestID.setText("0x000000");

        rUnitID.setText("0x000000");
        rUnitID.setMaximumSize(new java.awt.Dimension(70, 16));
        rUnitID.setMinimumSize(new java.awt.Dimension(70, 16));
        rUnitID.setPreferredSize(new java.awt.Dimension(70, 16));

        jLabel27.setText("DestID:");
        jLabel27.setMaximumSize(new java.awt.Dimension(48, 16));
        jLabel27.setMinimumSize(new java.awt.Dimension(48, 16));
        jLabel27.setPreferredSize(new java.awt.Dimension(48, 16));

        javax.swing.GroupLayout jPanel8Layout = new javax.swing.GroupLayout(jPanel8);
        jPanel8.setLayout(jPanel8Layout);
        jPanel8Layout.setHorizontalGroup(
            jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel8Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(jLabel13, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel14, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel15))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(rKeyID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(rTGID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(rNAC, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(jLabel23, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel24, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel25, javax.swing.GroupLayout.PREFERRED_SIZE, 54, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(rEmBit, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(rALGID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(rMFID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(jLabel26, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel27, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(rDestID)
                    .addComponent(rUnitID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        jPanel8Layout.setVerticalGroup(
            jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                .addGroup(jPanel8Layout.createSequentialGroup()
                    .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel13, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(rNAC, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGap(5, 5, 5)
                    .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel14, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(rTGID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGap(5, 5, 5)
                    .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel15)
                        .addComponent(rKeyID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addGroup(jPanel8Layout.createSequentialGroup()
                    .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel23, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(rMFID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGap(5, 5, 5)
                    .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel25, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(rALGID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGap(5, 5, 5)
                    .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel24, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(rEmBit, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addGroup(jPanel8Layout.createSequentialGroup()
                    .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel26, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(rUnitID, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGap(5, 5, 5)
                    .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel27, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(rDestID))))
        );

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(0, 0, 0)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGap(127, 127, 127)
                        .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jPanel11, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(0, 0, 0)
                        .addComponent(jPanel8, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jPanel10, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(0, 0, 0)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jPanel5, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jPanel7, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jPanel5, javax.swing.GroupLayout.PREFERRED_SIZE, 149, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel10, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jPanel7, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jPanel11, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jPanel8, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, 3, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(236, 236, 236))
        );

        jPanel7.getAccessibleContext().setAccessibleName("");

        jTabbedPane1.addTab("P25 Radio Control", jPanel1);

        jPanel2.setMaximumSize(new java.awt.Dimension(722, 570));
        jPanel2.setMinimumSize(new java.awt.Dimension(722, 570));
        jPanel2.setPreferredSize(new java.awt.Dimension(722, 570));

        jPanel6.setBorder(javax.swing.BorderFactory.createTitledBorder("Message Log"));
        jPanel6.setMaximumSize(new java.awt.Dimension(722, 490));
        jPanel6.setMinimumSize(new java.awt.Dimension(722, 490));
        jPanel6.setPreferredSize(new java.awt.Dimension(722, 490));

        StatusMessageBoxClearButton.setLabel("Clear");
        StatusMessageBoxClearButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                StatusMessageBoxClearButtonActionPerformed(evt);
            }
        });

        jScrollPane2.setPreferredSize(new java.awt.Dimension(688, 415));

        StatusMessageBox.setColumns(20);
        StatusMessageBox.setEditable(false);
        StatusMessageBox.setFont(new java.awt.Font("DejaVu Sans", 0, 10));
        StatusMessageBox.setLineWrap(true);
        StatusMessageBox.setRows(5);
        StatusMessageBox.setWrapStyleWord(true);
        StatusMessageBox.setBorder(javax.swing.BorderFactory.createTitledBorder(""));
        StatusMessageBox.setMaximumSize(new java.awt.Dimension(688, 32768));
        StatusMessageBox.setMinimumSize(new java.awt.Dimension(688, 415));
        StatusMessageBox.setPreferredSize(new java.awt.Dimension(688, 415));
        jScrollPane2.setViewportView(StatusMessageBox);

        javax.swing.GroupLayout jPanel6Layout = new javax.swing.GroupLayout(jPanel6);
        jPanel6.setLayout(jPanel6Layout);
        jPanel6Layout.setHorizontalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel6Layout.createSequentialGroup()
                .addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel6Layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 698, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(jPanel6Layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(StatusMessageBoxClearButton, javax.swing.GroupLayout.PREFERRED_SIZE, 143, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel6Layout.setVerticalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel6Layout.createSequentialGroup()
                .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 228, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(StatusMessageBoxClearButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(12, Short.MAX_VALUE))
        );

        StatusMessageBoxClearButton.getAccessibleContext().setAccessibleName("button1");

        jPanel9.setBorder(javax.swing.BorderFactory.createTitledBorder("Voice / Data Statistics (currently not used)"));
        jPanel9.setMaximumSize(new java.awt.Dimension(722, 80));
        jPanel9.setMinimumSize(new java.awt.Dimension(722, 80));
        jPanel9.setPreferredSize(new java.awt.Dimension(722, 80));

        DataBytesReceivedLabel.setText("0");

        jLabel5.setText("Voice Bytes Received:");

        BytesReceivedLabel.setText("0");

        BytesSentLabel.setText("0");

        jLabel4.setText("Voice Bytes Sent:");

        jLabel11.setText("Data Bytes Received:");

        jLabel10.setText("Data Bytes Sent:");

        DataBytesSentLabel.setText("0");

        javax.swing.GroupLayout jPanel9Layout = new javax.swing.GroupLayout(jPanel9);
        jPanel9.setLayout(jPanel9Layout);
        jPanel9Layout.setHorizontalGroup(
            jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel9Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel4)
                    .addComponent(jLabel5))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(BytesReceivedLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(BytesSentLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 114, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel10)
                    .addComponent(jLabel11))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(DataBytesSentLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(DataBytesReceivedLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 128, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(136, Short.MAX_VALUE))
        );
        jPanel9Layout.setVerticalGroup(
            jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel9Layout.createSequentialGroup()
                .addGap(5, 5, 5)
                .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                        .addGroup(jPanel9Layout.createSequentialGroup()
                            .addComponent(jLabel4)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addComponent(jLabel5))
                        .addGroup(jPanel9Layout.createSequentialGroup()
                            .addComponent(BytesSentLabel)
                            .addGap(24, 24, 24)))
                    .addGroup(jPanel9Layout.createSequentialGroup()
                        .addGap(24, 24, 24)
                        .addComponent(BytesReceivedLabel))
                    .addGroup(jPanel9Layout.createSequentialGroup()
                        .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel10)
                            .addComponent(DataBytesSentLabel))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel11)
                            .addComponent(DataBytesReceivedLabel))))
                .addContainerGap())
        );

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel9, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jPanel6, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addComponent(jPanel9, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel6, javax.swing.GroupLayout.PREFERRED_SIZE, 298, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(210, 210, 210))
        );

        jTabbedPane1.addTab("Status Messages", jPanel2);

        TestModeCheckBox.setText("Enable File Test Mode");
        TestModeCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                TestModeCheckBoxActionPerformed(evt);
            }
        });

        TestModeFile.setText("/tmp/file.dat");
        TestModeFile.setEnabled(false);
        TestModeFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                TestModeFileActionPerformed(evt);
            }
        });

        TestModeChooseFileBtn.setText("Choose File");
        TestModeChooseFileBtn.setEnabled(false);
        TestModeChooseFileBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                TestModeChooseFileBtnActionPerformed(evt);
            }
        });

        jLabel30.setText("File to Send:");

        jLabel32.setText("Directory for Received Files:");

        TestModeDirRx.setText("/tmp");
        TestModeDirRx.setEnabled(false);
        TestModeDirRx.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                TestModeDirRxActionPerformed(evt);
            }
        });

        TestModeChooseDirRxBtn.setText("Choose File");
        TestModeChooseDirRxBtn.setEnabled(false);
        TestModeChooseDirRxBtn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                TestModeChooseDirRxBtnActionPerformed(evt);
            }
        });

        jLabel33.setText("List of Frequencies to Scan:");

        ScanningFreqs.setColumns(20);
        ScanningFreqs.setRows(5);
        ScanningFreqs.setText("139.75MHz\n141.175MHz\n142.9250MHz\n149.4000MHz\n150.3875MHz");
        jScrollPane3.setViewportView(ScanningFreqs);

        javax.swing.GroupLayout jPanel12Layout = new javax.swing.GroupLayout(jPanel12);
        jPanel12.setLayout(jPanel12Layout);
        jPanel12Layout.setHorizontalGroup(
            jPanel12Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel12Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel12Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 291, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(TestModeCheckBox)
                    .addGroup(jPanel12Layout.createSequentialGroup()
                        .addComponent(TestModeFile, javax.swing.GroupLayout.PREFERRED_SIZE, 488, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(TestModeChooseFileBtn, javax.swing.GroupLayout.PREFERRED_SIZE, 115, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(jLabel30)
                    .addGroup(jPanel12Layout.createSequentialGroup()
                        .addComponent(TestModeDirRx, javax.swing.GroupLayout.PREFERRED_SIZE, 488, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(TestModeChooseDirRxBtn, javax.swing.GroupLayout.PREFERRED_SIZE, 115, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(jLabel32)
                    .addComponent(jLabel33))
                .addContainerGap(109, Short.MAX_VALUE))
        );
        jPanel12Layout.setVerticalGroup(
            jPanel12Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel12Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(TestModeCheckBox)
                .addGap(20, 20, 20)
                .addComponent(jLabel30)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel12Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(TestModeFile, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(TestModeChooseFileBtn))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabel32)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel12Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(TestModeDirRx, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(TestModeChooseDirRxBtn))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabel33)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 188, Short.MAX_VALUE)
                .addContainerGap())
        );

        jTabbedPane1.addTab("Data Sources", jPanel12);

        jMenu1.setText("File");
        jMenuBar1.add(jMenu1);

        jMenu2.setText("Edit");
        jMenuBar1.add(jMenu2);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addComponent(jTabbedPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 735, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jTabbedPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 415, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jTabbedPane1.getAccessibleContext().setAccessibleName("Status Messages");

        pack();
    }// </editor-fold>//GEN-END:initComponents
    
	/** 
	 * Used for FM3TR
	 */
    public void FM3TR_SetAudioCaptureRate() {
        /* PCM 16 kHz, 16 bit, mono, signed, big endian. */
 //       Integer sampleRate = new Integer(FM3TR_AudioCaptureRate.getText());
 //       audioFormat = new AudioFormat((float)sampleRate, 16, 1, true, true);

        /* Now, we are trying to get a TargetDataLine. The
        TargetDataLine is used later to read audio data from the capture device.
        If requesting the line was successful, we are opening
        it (important!).
        */
 //       td_info = new DataLine.Info(TargetDataLine.class, audioFormat);
    }

	/**
	 * Used for FM3TR
	 */    
    public void FM3TR_SetAudioPlaybackRate() {
        /* PCM 16 kHz, 16 bit, mono, signed, big endian. */
//        Integer sampleRate = new Integer(FM3TR_AudioPlaybackRate.getText());
//        audioFormat2 = new AudioFormat((float)sampleRate, 16, 1, true, true);

        /* Now, we are trying to get a SourceDataLine. The
        SourceDataLine is used later to write audio data to the playback device.
        If requesting the line was successful, we are opening
        it (important!).
        */
        sd_info = new DataLine.Info(SourceDataLine.class, audioFormat2);               
    }    
    
	/**
	 * Starts p25 related GUI threads.
	 */
    public void P25_StartThreads() {
//        playbackThread =
//            new AudioPlaybackThread();
//        playbackThread.start();

        p25FeedbackThread =
            new P25FeedbackThread();
        p25FeedbackThread.start();
    }
    
	/**
	 * Stop p25 related GUI threads.
	 */
    public void P25_StopThreads() {
//        playbackThread.running = false;
//        try {playbackThread.join();} catch (Exception e) {}
        p25FeedbackThread.running = false;
        try {p25FeedbackThread.join();} catch (Exception e) {}
    }
    
	/**
	 * Used for FM3TR
	 */
    public Boolean P25_ITM_Connect() {
        // Close any existing data socket connections as a precaution
        // before a new connection is established.
/*        try {
            MyDataClient = new Socket(P25_Radio_IP.getText(),
                   new Integer(FM3TR_Data_Port.getText()));
            try {
                dataInputStream = 
                   new DataInputStream(MyDataClient.getInputStream());
                dataOutputStream = 
                   new DataOutputStream(MyDataClient.getOutputStream());
            }
            catch (IOException e) {
                StatusMessageBox.append("Error: " + e + "\n");
                return false;
            }
        }
        catch (IOException e) {
            StatusMessageBox.append("Error: " + e + "\n");
            return false;
        }          
        
        // Start data receive thread
	dataReceiveThread = new DataReceiveThread();
	dataReceiveThread.start();   
*/
        return true;
    }
    
    public boolean P25_ITM_IsConnected() {
        return (MyDataClient != null && MyDataClient.isConnected());
    }    

    public void P25_ITM_Disconnect() {
        if (P25_ITM_IsConnected()) {
            try {
                MyDataClient.close();         
            }
            catch (IOException e) {
                StatusMessageBox.append("Error: " + e + "\n");
                return;
            }       
            
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }            
        }    
        
        dataReceiveThread.running = false;
        try {dataReceiveThread.join();} catch(Exception e) {}
    }

	/**
	 * Gets connection to OSSIE UI component via TCP connection.
	 *		Also starts timer and starts the relevant threads.
	 *
	 * @return Returns true if a connection is established, false if the UI
	 *		doesn't have a port already initialized for a connection.
	 */    
    public Boolean P25_Voice_Connect() {
        // Establishes the new voice socket connection
        try {
            StatusMessageBox.append("Connecting to host " +
                    P25_Radio_IP.getText() + " using TCP port " +
                    P25_Radio_Port.getText() + "... ");
            
            MyClient = new Socket(P25_Radio_IP.getText(),
                   new Integer(P25_Radio_Port.getText()));
            
            MyClient.setTcpNoDelay(true);

            StatusMessageBox.append("success!\n");

            try {
                StatusMessageBox.append("Establishing TCP data stream... ");

                inputStream = 
                    new DataInputStream(MyClient.getInputStream());                
                outputStream = 
                    new DataOutputStream(MyClient.getOutputStream());

                StatusMessageBox.append("success!\n");
            }
            catch (IOException e) {
                StatusMessageBox.append("Error: " + e + "\n");
                return false;
            }
            
            
            if (timerThread == null) {
                timerThread = new Thread(new TimerThread(), "TimerThread");   
                timerThread.start();
            }
            else
                if (!timerThread.isAlive()) {
                    timerThread = new Thread(new TimerThread(), "TimerThread");   
                    timerThread.start();
                }            
        }
        catch (IOException e) {
            StatusMessageBox.append("Error: " + e + "\n");
            return false;
        }            
        
        P25_StartThreads();
        
        // Playback rate can only be edited when disconnected from radio.
      //  FM3TR_AudioPlaybackRate.setEnabled(false);
        
        return true;
    }
    
	/**
	 * Checks to see if a connection is active.
	 */
    public boolean P25_Voice_IsConnected() {
        return (MyClient != null && MyClient.isConnected());
    }        

	/**
	 * Disconnects from OSSIE UI component.
	 */
    public void P25_Voice_Disconnect() {
        if (P25_Voice_IsConnected()) {
            try {
                MyClient.close();
            }
            catch (IOException e) {
                StatusMessageBox.append("Error: " + e + "\n");
                return;
            }
        }
        
        P25_StopThreads();
        
        // Allow editing of playback rate once disconnected from radio.
//        FM3TR_AudioPlaybackRate.setEnabled(true);
    }    

	/** Send programming fields + PTT on message to program radio data fields
	 * and update TX/RX status on the radio, simulating the press of a
	 * push-to-talk button on a handset.
	 * 
	 * Data fields are sent in-order based on the following receiver struct:
	 * 
	 * typedef struct {
	 *       double cfreq;
	 *       short 	nac;
 	 *       short 	tgid;
	 *       short	keyid;
	 *       char 	mfid;
	 *       char	algid;
	 *       char	ebit;
	 *       char	uid[3];
	 *       char	dstid[3];
	 *       char	mi[9];
	 *       char   tfpath[255];
	 *       char   tfmode;
	 *       char	ptt;
	 *   } P25Config;
	 * 
	 */

    public void P25_SendControlPacket(byte PTTBtnStatus) {

        String f_RF = RFFreq.getText();
        //String f_RF = RFFreqLabel.getText();
        f_RF = f_RF.replace("k", "e3");
        f_RF = f_RF.replace("M", "e6");
        f_RF = f_RF.replace("G", "e9");
        f_RF = f_RF.replace("Hz", "");
        f_RF = f_RF.replace(" ", "");
        //RFFreqLabel.setText(RFFreq.getText());
        // Set RF frequency on the NI box
//        String f_RF = RFFreq.getText();
//        f_RF = f_RF.replace("k", "E3");
//        f_RF = f_RF.replace("M", "E6");
//        f_RF = f_RF.replace("G", "E9");
//        byte[] b_pkt = ("P25_RF:CF " + f_RF).getBytes();
//
//        try {
//            StatusMessageBox.append("Connecting to RF frontend at address " +
//                    P25_RF_IP.getText() + " using UDP port " +
//                    P25_RF_Port.getText() + "... ");
//
//            DatagramPacket p = new DatagramPacket(b_pkt, 0, b_pkt.length,
//                    InetAddress.getByName(P25_RF_IP.getText()),
//                    Integer.decode(P25_RF_Port.getText()));
//
//            NIClient = new DatagramSocket(
//                    Integer.decode(P25_RF_Port.getText()));
//
//            StatusMessageBox.append("success!\n");
//
//            try {
//                StatusMessageBox.append("Sending RF frequency data... ");
//                NIClient.send(p);
//                StatusMessageBox.append("success!\n");
//            }
//            catch (IOException e) {
//                StatusMessageBox.append("Error: " + e + "\n");
//                //return;
//            }
//        }
//        catch (IOException e) {
//            StatusMessageBox.append("Error: " + e + "\n");
//            return;
//        }



        try {
            // Note:  Borrowing data_out and byte_out from currently unused
            // AudioCaptureThread; would need to create a different data_out
            // and byte_out instance in the event that AudioCaptureThread is
            // brought back due to the use of an internal audio codec in the
            // radio.
            data_out.flush();
            byte_out.reset();

            // Send header 0x0A + 0xA0
            data_out.writeByte(0x0A);
            data_out.writeByte(0xA0);

            // Carrier frequency (8 bytes)
            StatusMessageBox.append("Setting RF Frequency: " +
                    Double.toString(Double.parseDouble(f_RF)) + "\n");
            data_out.writeDouble(Double.parseDouble(f_RF));

            // Network Access Code (2 bytes)
            data_out.writeShort(Integer.decode(
                    NAC.getText()).shortValue());

            // Talk Group ID (2 bytes)
            data_out.writeShort(Integer.decode(
                    TalkGroupID.getText()).shortValue());

            // Key ID (2 bytes)
            data_out.writeShort(Integer.decode(
                    KeyID.getText()).shortValue());

            // Manufacturer ID (1 byte)
            data_out.writeByte(Integer.decode(
                    MFID.getText()).byteValue());

            // Algorithm ID (1 byte)
            data_out.writeByte(Integer.decode(
                    AlgID.getText()).byteValue());

            // Emergency bit (1 byte)
            if (EmergencyBit.getState())
                data_out.writeByte((byte)1);
            else
                data_out.writeByte((byte)0);

            // Source ID / Unit ID (3 bytes)
            buf_tmp.flush();
            byte_tmp.reset();
            buf_tmp.writeInt(Integer.decode(UnitID.getText()).intValue());
            data_out.write(byte_tmp.toByteArray(), 1, 3);

            // Destination ID (3 bytes)
            buf_tmp.flush();
            byte_tmp.reset();
            buf_tmp.writeInt(Integer.decode(DestID.getText()).intValue());
            data_out.write(byte_tmp.toByteArray(), 1, 3);

            // Message Indicator (9 bytes)
            buf_tmp.flush();
            byte_tmp.reset();

            try {
                for (int i = 2; i < 20; i+=2) 
                    buf_tmp.writeByte(Integer.decode(
                            "0x" + MI.getText(i,2)).byteValue());
            }
            catch (Exception e) {
                    StatusMessageBox.append("Error reading MI text field: "
                            + e + "\n");
            }
/*
            StatusMessageBox.append("MI: 0x");
            for (int i = 0; i < 9; i++)
                StatusMessageBox.append(
                        String.format("%02X", byte_tmp.toByteArray()[i]));
            StatusMessageBox.append("\n");
*/
            data_out.write(byte_tmp.toByteArray(), 0, 9);

            // Test File Path (used only if Test File Mode is set)
            byte[] fp = TestModeFile.getText().getBytes();
            int fplen = TestModeFile.getText().getBytes().length;
            if (fplen > 255) fplen = 255;
            data_out.write(fp, 0, fplen);
            for (int i = fplen; i < 255; i++) data_out.writeByte('\0');

            // Test File Mode (0x55=Use_VC55, 0xF5=Use_Test_File)
            if (TestModeCheckBox.isSelected())
                data_out.writeByte(0xF5);
            else
                data_out.writeByte(0x55);

            // PTT Status (0x01=TX, 0x00=RX) (1 byte)
            data_out.writeByte(PTTBtnStatus);

            // RX Encryption State (0 = Use AlgID of incoming call, 1 = Use GUI AlgID)
            if (RX_Enc_Button.getState())
                data_out.writeByte(0x01);
            else
                data_out.writeByte(0x00);
            
            StatusMessageBox.append("Sizeof byte_out = " + byte_out.size());

            outputStream.write(byte_out.toByteArray(), 0, byte_out.size());
        }
        catch (IOException e) {
            StatusMessageBox.append("Error: " + e + "\n");
        }
    }

	/**
	 * Sets the state of the GUI, and sends the programming fields and PTT on a message to
	 *		the OSSIE UI component.  If talk mode is active (TX activity), PTT is set to 1
	 *		and the OSSIE UI component goes into TX activity.  If talk mode is deactivated,
	 *		PTT is set to 0, and the OSSIE UI component goes into RX activity.
	 */
    private void P25_Talk_ButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_P25_Talk_ButtonActionPerformed
        // TODO add your handling code here:
        if (!P25_Voice_IsConnected())
            return;
	
        if (P25_Talk_Button.isSelected()) {   // Talk mode
            PTTMode.setText("Talk Mode");
            Scan_Button.setSelected(false);
            Scan_Button.setEnabled(false);

	    // Start audio capture thread
//            captureThread = new AudioCaptureThread();
//            captureThread.start();

            // Send programming & control packet with PTT status to TX
            P25_SendControlPacket((byte)0x01);


 //           TextMsgSendBox.setEnabled(false);
 //           TextMsgRcvBox.setEnabled(false);
        }
        else {  // Listen mode
  /*          if (captureThread != null) {
                captureThread.running = false;
                try {captureThread.join();} catch (Exception e) {}
            }
  */
            PTTMode.setText("Listen Mode");
            Scan_Button.setEnabled(true);
            Scan_Button.setSelected(false);

            // Send programming & control packet with PTT status to RX
            P25_SendControlPacket((byte)0x00);

//            TextMsgSendBox.setEnabled(true);
//            TextMsgRcvBox.setEnabled(true);
        }
}//GEN-LAST:event_P25_Talk_ButtonActionPerformed

	/**
	 * Resets GUI fields, buttons, counters, etc, and establishes a connection to the OSSIE
	 *	side UI component.
	 */
    private void P25_Connect_ButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_P25_Connect_ButtonActionPerformed
        // TODO add your handling code here:
        if (P25_Connect_Button.isSelected()) {
            bytesSentCount = 0;
            BytesSentLabel.setText(Long.toString(bytesSentCount));  

            bytesReceivedCount = 0;
            BytesReceivedLabel.setText(Long.toString(bytesReceivedCount));         
            
            dataBytesSentCount = 0;
            DataBytesSentLabel.setText(Long.toString(dataBytesSentCount));  

            dataBytesReceivedCount = 0;
            DataBytesReceivedLabel.setText(Long.toString(
                    dataBytesReceivedCount));                 

            if (!P25_Voice_Connect()) {
                P25_Connect_Button.setSelected(false);
                P25_Connect_Button.setText("Connect");               
                UseToggle.setState(false);
                UseToggleItemStateChanged(null);
                UseToggle.setEnabled(false);
                P25_Talk_Button.setEnabled(false);
                Scan_Button.setSelected(false);
                Scan_Button.setEnabled(false);
                return;
            }

            rALGID.setText("0x00");
            rDestID.setText("0x000000");
            rEmBit.setText("No");
            rKeyID.setText("0x0000");
            rMFID.setText("0x0");
            rNAC.setText("0x000");
            rTGID.setText("0x0000");
            rUnitID.setText("0x000000");
            RFFreqLabel.setText(RFFreq.getText());
            //RFFreqLabel.setText("139.75MHz");

//            if (P25_ITM_Connect()) {
//                TextMsgSendBox.setEnabled(true);
//                TextMsgRcvBox.setEnabled(true);
/*            }
            else {
                TextMsgSendBox.setEnabled(false);
                TextMsgRcvBox.setEnabled(false);                
            }
 */
            // Start in listen mode
            P25_Connect_Button.setText("Disconnect");
            UseToggle.setEnabled(true);
            P25_Talk_Button.setEnabled(true);
            P25_Talk_Button.setSelected(false);
            P25_Talk_ButtonActionPerformed(evt);
            
            haltTimer = false;
        }
        else {           
            haltTimer = true;
            
            // Close any existing data and voice socket connections, and stop
            // all audio capturing/playback
//            P25_ITM_Disconnect();
            P25_Voice_Disconnect();
            
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }            
            
            P25_Connect_Button.setText("Connect");          
            UseToggle.setState(false);
            UseToggleItemStateChanged(null);
            UseToggle.setEnabled(false);
            P25_Talk_Button.setEnabled(false);
            Scan_Button.setSelected(false);
            Scan_Button.setEnabled(false);
            
            TextMsgSendBox.setEnabled(false);
            TextMsgRcvBox.setEnabled(false);
        }
}//GEN-LAST:event_P25_Connect_ButtonActionPerformed

	/**
	 * The text message capabilities are not implemented in the P25 project in OSSIE.
	 *		This is left over from the SDR4000.
	 */
    private void TextMsgSendBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_TextMsgSendBoxActionPerformed
        // TODO add your handling code here:
        if (!evt.getActionCommand().equals("")) {
            TextMsgRcvBox.append("me:  " + evt.getActionCommand() +"\n");                   
            TextMsgRcvBox.setCaretPosition(TextMsgRcvBox.getText().length());

            byte databuf[] = evt.getActionCommand().getBytes();
            short databuf_len = (short)databuf.length;
            try {
                // write ITM header
                short sh = (short)(4 + 4 + databuf_len);
                dataOutputStream.writeShort(sh);
                dataOutputStream.writeShort(1);
                dataOutputStream.writeByte(0);              // src
                dataOutputStream.writeByte(Byte.parseByte(  // dest
                    TalkGroupID.getText()));
                dataOutputStream.writeShort(databuf_len);
                
                // write ITM data
                dataOutputStream.write(databuf);
                
                // update data bytes sent counter on GUI
                dataBytesSentCount += (4+4+databuf_len);
		DataBytesSentLabel.setText(Long.toString(dataBytesSentCount));
            }
            catch (IOException e) {
                StatusMessageBox.append("Error: " + e + "\n");
            }

            TextMsgSendBox.setText("");
            TextMsgSendBox.grabFocus();
        }
}//GEN-LAST:event_TextMsgSendBoxActionPerformed

	/**
	 * The text message capabilities are not implemented in the P25 project in OSSIE.
	 *		This is left over from the SDR4000.
	 */
    private void TextMsgSendBoxFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_TextMsgSendBoxFocusLost
        // TODO add your handling code here:
        if (TextMsgSendBox.getText().equals("")) {
            TextMsgSendBox.setText("Type your message here, <Enter> to send.");
        }
    }//GEN-LAST:event_TextMsgSendBoxFocusLost

	/**
	 * The text message capabilities are not implemented in the P25 project in OSSIE.
	 *		This is left over from the SDR4000.
	 */
    private void TextMsgSendBoxFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_TextMsgSendBoxFocusGained
        // TODO add your handling code here:
        if (TextMsgSendBox.getText().equals(
                "Type your message here, <Enter> to send.")) {
            TextMsgSendBox.setText("");
        }
    }//GEN-LAST:event_TextMsgSendBoxFocusGained

	/**
	 * For Key ID field.
	 */
    private void KeyIDFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_KeyIDFocusLost
        // TODO add your handling code here:
        try {
            KeyID.setText("0x" + String.format("%04X", Integer.decode(
                KeyID.getText())));
            KeyIDLastValue = KeyID.getText();
        }
        catch (NumberFormatException e) {
            KeyID.setText(KeyIDLastValue);
        }
    }//GEN-LAST:event_KeyIDFocusLost

	/**
	 * For Algorithm ID field.
	 */
    private void AlgIDFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_AlgIDFocusLost
        // TODO add your handling code here:
        try {
            AlgID.setText("0x" + String.format("%02X", Integer.decode(
                AlgID.getText())));
            AlgIDLastValue = AlgID.getText();
        }
        catch (NumberFormatException e) {
            AlgID.setText(AlgIDLastValue);
        }
    }//GEN-LAST:event_AlgIDFocusLost

	/**
	 * For Talk Group ID field.
	 */
    private void TalkGroupIDFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_TalkGroupIDFocusLost
        // TODO add your handling code here:
        try {
            TalkGroupID.setText("0x" + String.format("%04X", Integer.decode(
                TalkGroupID.getText())));
            TalkGroupIDLastValue = TalkGroupID.getText();
        }
        catch (NumberFormatException e) {
            TalkGroupID.setText(TalkGroupIDLastValue);
        }
    }//GEN-LAST:event_TalkGroupIDFocusLost

	/**
	 * For Message Indicator field.
	 */
    private void MIFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_MIFocusLost
       // TODO add your handling code here:
       try {
            if (MI.getText().length() == 20 && MI.getText(0,2).equals("0x")) {
                String newMI = "0x";
                for (int i = 2; i < 20; i+=2) {
                    // Restrict value of MI1 to 8 bits
                    String tmp = "0x" + MI.getText(i, 2);
                    if ((Integer.decode(tmp).intValue() >= 0) &&
                        (Integer.decode(tmp).intValue() <= 255))
                    {
                        newMI = newMI + String.format("%02X", Integer.decode(
                            tmp));
                    }
                    else {
                        MI.setText(MI1LastValue);
                        break;
                    }
                }
                MI.setText(newMI);
                MI1LastValue = MI.getText();
            }
            else
                MI.setText(MI1LastValue);
        }
        catch (Exception e) {
            MI.setText(MI1LastValue);
        }
}//GEN-LAST:event_MIFocusLost

	/**
	 * For Network Access Code field.
	 */
    private void NACFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_NACFocusLost
        // TODO add your handling code here:
        try {
            // Restrict value of NAC to 12 bits
            if ((Integer.decode(NAC.getText()).shortValue() >= 0) &&
                (Integer.decode(NAC.getText()).shortValue() <= 4095))
            {
                NAC.setText("0x" + String.format("%03X", Integer.decode(
                    NAC.getText())));
                NACLastValue = NAC.getText();
            }
            else
                NAC.setText(NACLastValue);
        }
        catch (NumberFormatException e) {
            NAC.setText(NACLastValue);
        }
    }//GEN-LAST:event_NACFocusLost

	/**
	 * For Manufacturer ID field.
	 */
    private void MFIDFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_MFIDFocusLost
        // TODO add your handling code here:
        try {
            // Restrict value of MFID to 4 bits
            if ((Integer.decode(MFID.getText()).shortValue() >= 0) &&
                (Integer.decode(MFID.getText()).shortValue() <= 15))
            {
                MFID.setText("0x" + Integer.toHexString(Integer.decode(
                    MFID.getText())).toUpperCase());
                MFIDLastValue = MFID.getText();
            }
            else
                MFID.setText(MFIDLastValue);
        }
        catch (NumberFormatException e) {
            MFID.setText(MFIDLastValue);
        }
    }//GEN-LAST:event_MFIDFocusLost

	/**
	 * For Unit ID field.
	 */
    private void UnitIDFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_UnitIDFocusLost
        // TODO add your handling code here:
        try {
            UnitID.setText("0x" +  String.format("%06X", Integer.decode(
                UnitID.getText())));
            SourceIDLastValue = UnitID.getText();
        }
        catch (NumberFormatException e) {
            UnitID.setText(SourceIDLastValue);
        }
    }//GEN-LAST:event_UnitIDFocusLost

	/**
	 * Clears status message box upon pressing the button.
	 */
    private void StatusMessageBoxClearButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_StatusMessageBoxClearButtonActionPerformed
        // TODO add your handling code here:
        StatusMessageBox.setText("");
    }//GEN-LAST:event_StatusMessageBoxClearButtonActionPerformed

	/**
	 * For Destination ID Field.
	 */
    private void DestIDFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_DestIDFocusLost
        // TODO add your handling code here:
        try {
            DestID.setText("0x" + String.format("%06X", Integer.decode(
                DestID.getText())));
            DestIDLastValue = DestID.getText();
        }
        catch (NumberFormatException e) {
            DestID.setText(DestIDLastValue);
        }
}//GEN-LAST:event_DestIDFocusLost

    private void P25_Radio_PortActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_P25_Radio_PortActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_P25_Radio_PortActionPerformed

    private void P25_RF_PortActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_P25_RF_PortActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_P25_RF_PortActionPerformed

	/**
	 * For No NAC check box.
	 */
    private void NoNACsItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_NoNACsItemStateChanged
        // TODO add your handling code here:
        if (NoNACs.getState()) {
            NACLastValue = NAC.getText();
            NAC.setText("0xF7E");
            NAC.setEnabled(false);
        }
        else {
            NAC.setText(NACLastValue);
            NAC.setEnabled(true);
        }
    }//GEN-LAST:event_NoNACsItemStateChanged

    private void formKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_formKeyPressed
        // TODO add your handling code here:
        if (UseToggle.getState()) {
            if (evt.getKeyCode() == KeyEvent.VK_SPACE &&
                    P25_Talk_Button.isSelected() == false) {
                P25_Talk_Button.setSelected(true);
                P25_Talk_ButtonActionPerformed(null);
            }
            UseToggle.setEnabled(false);
        }
    }//GEN-LAST:event_formKeyPressed

    private void formKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_formKeyReleased
        // TODO add your handling code here:
        if (UseToggle.getState()) {
            if (evt.getKeyCode() == KeyEvent.VK_SPACE &&
                    P25_Talk_Button.isSelected()) {
                P25_Talk_Button.setSelected(false);
                P25_Talk_ButtonActionPerformed(null);
            }
            UseToggle.setEnabled(true);
        }
    }//GEN-LAST:event_formKeyReleased

    private void UseToggleItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_UseToggleItemStateChanged
        // TODO add your handling code here:
        ToggleSelection.setEnabled(UseToggle.getState());
        P25_Talk_Button.setEnabled(!UseToggle.getState());
        if (ToggleSelection.isEnabled())
            ToggleSelection.grabFocus();
    }//GEN-LAST:event_UseToggleItemStateChanged

    private void ToggleSelectionKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_ToggleSelectionKeyPressed
        // TODO add your handling code here:
        ToggleSelection.setSelected(false);
        if (PTTMode.getText().equals("Listen Mode")) {
            P25_Talk_Button.setSelected(true);
            P25_Talk_ButtonActionPerformed(null);
        }
    }//GEN-LAST:event_ToggleSelectionKeyPressed

    private void ToggleSelectionKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_ToggleSelectionKeyReleased
        // TODO add your handling code here:
        ToggleSelection.setSelected(false);
        if (PTTMode.getText().equals("Talk Mode")) {
            P25_Talk_Button.setSelected(false);
            P25_Talk_ButtonActionPerformed(null);
        }
    }//GEN-LAST:event_ToggleSelectionKeyReleased

    private void RFFreqActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_RFFreqActionPerformed
        // TODO add your handling code here:
        String f_RF = RFFreq.getText();
        f_RF = f_RF.replace(" ", "");
        f_RF = f_RF.replace("Hz", "");        
        f_RF = f_RF.replace("k", "e3");
        f_RF = f_RF.replace("M", "e6");
        f_RF = f_RF.replace("G", "e9");
        double rffreq = Double.parseDouble(f_RF);
        //double iffreq = 15e6 + (Math.abs(rffreq - 15e6) % 5e6);
        //double rffefreq = rffreq + 25e6 - iffreq;

        //IFFreqLabel.setText(Double.toString(iffreq/1e6) + "MHz");
        //RFFreqLabel.setText(Double.toString(rffefreq/1e6) + "MHz");
        RFFreqLabel.setText(Double.toString(rffreq/1e6) + "MHz");
        StatusMessageBox.append("Something just happened, I can FEEL it " + "\n");
}//GEN-LAST:event_RFFreqActionPerformed

    private void TestModeCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_TestModeCheckBoxActionPerformed
        // TODO add your handling code here:
        TestModeFile.setEnabled(TestModeCheckBox.isSelected());
        TestModeDirRx.setEnabled(TestModeCheckBox.isSelected());
        TestModeChooseFileBtn.setEnabled(TestModeCheckBox.isSelected());
        TestModeChooseDirRxBtn.setEnabled(TestModeCheckBox.isSelected());
}//GEN-LAST:event_TestModeCheckBoxActionPerformed

    private void TestModeFileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_TestModeFileActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_TestModeFileActionPerformed

    private void TestModeChooseFileBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_TestModeChooseFileBtnActionPerformed
        // TODO add your handling code here:
        JFileChooser fileChooser = new JFileChooser();

        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

        if (fileChooser.showOpenDialog(this) == JFileChooser.CANCEL_OPTION)
            return;

        File fileName = fileChooser.getSelectedFile();

        if (fileName == null || fileName.getName().equals("")) {
            String e = "Invalid File Name";
            StatusMessageBox.append("Error: " + e + "\n");
        } else
            TestModeFile.setText(fileName.getAbsolutePath());

    }//GEN-LAST:event_TestModeChooseFileBtnActionPerformed

    private void TestModeDirRxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_TestModeDirRxActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_TestModeDirRxActionPerformed

    private void TestModeChooseDirRxBtnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_TestModeChooseDirRxBtnActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_TestModeChooseDirRxBtnActionPerformed

	/**
	 * When the scan button is pressed, the GUI sends a scan struct to the UI component
	 *  with a list of frequencies to scan, and also the number of frequencies to scan.
	 */
    private void Scan_ButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_Scan_ButtonActionPerformed
        // TODO add your handling code here:
        //if (!P25_Voice_IsConnected())
        //    return;

        if (Scan_Button.isSelected()) {
            P25_Send_Scanning_Packet((byte)1);
            
        } else {
            P25_Send_Scanning_Packet((byte)0);
            return;
        }
    }//GEN-LAST:event_Scan_ButtonActionPerformed

	/**
	 * Gathers list of frequencies from the text entry box, and sends a list of frequencies, and
	 *  number of frequnecies to scan to the OSSIE UI component.
	 * 
	 * @param The state of scanning (on/off)
	 * @author Justyn Bell <jwbell@ucsd.edu>
	 */
    public void P25_Send_Scanning_Packet(byte ScanState) {
    String all_freqs = ScanningFreqs.getText();

        int snum = ScanningFreqs.getLineCount();
        if (snum > 15) {
            StatusMessageBox.append("Error: No more than 15"
                    + " scanning frequencies allowed (for now).\n");
            Scan_Button.setEnabled(false);
            return;
        }

        double[] freqs;
        freqs = new double[15];

        try {
            for (int i=0; i < snum; i++) {
                int start = ScanningFreqs.getLineStartOffset(i);
                int stop = ScanningFreqs.getLineEndOffset(i);
                String freq = all_freqs.substring(start, stop);

                freq = freq.replace("k", "e3");
                freq = freq.replace("M", "e6");
                freq = freq.replace("G", "e9");
                freq = freq.replace("Hz", "");
                freq = freq.replace("hz", "");
                freq = freq.replace(" ", "");

                freqs[i] = Double.parseDouble(freq);
            }
        } catch (Exception ex) {
            StatusMessageBox.append("Error: " + ex + "\n");
        }

        if (snum < 15)
            for (int j=snum; j < 15; j++)
                freqs[j] = 0;


        for (int k=0; k < snum; k++)
            StatusMessageBox.append("Read scanning freq"
                    + Double.toString(freqs[k]) + "MHz\n");

        byte state = ScanState;
        try {
            data_out.flush();
            byte_out.reset();

            // Send scan struct header 0x0B + 0xB0
            data_out.write((byte)0x0B);
            data_out.write((byte)0xB0);

            StatusMessageBox.append("Sent header to UI\n");

            data_out.writeByte(state);
            data_out.writeShort(snum);

            for (int l=0; l < 15; l++)
                data_out.writeDouble(freqs[l]);

            outputStream.write(byte_out.toByteArray(), 0, byte_out.size());
            
        } catch (Exception ex) {
            StatusMessageBox.append("Error: " + ex + "\n");
        }

    }
    /**
     * @param ar jgs the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {          
                new P25_Client_JFrame().setVisible(true);
            }
        });
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField AlgID;
    private javax.swing.JLabel BytesReceivedLabel;
    private javax.swing.JLabel BytesSentLabel;
    private javax.swing.JLabel ConnectionTime;
    private javax.swing.JLabel DataBytesReceivedLabel;
    private javax.swing.JLabel DataBytesSentLabel;
    private javax.swing.JTextField DestID;
    private java.awt.Checkbox EmergencyBit;
    private javax.swing.JTextField KeyID;
    private javax.swing.JTextField MFID;
    private javax.swing.JTextField MI;
    private javax.swing.JTextField NAC;
    private java.awt.Checkbox NoNACs;
    private javax.swing.JToggleButton P25_Connect_Button;
    private javax.swing.JTextField P25_RF_IP;
    private javax.swing.JTextField P25_RF_Port;
    private javax.swing.JTextField P25_Radio_IP;
    private javax.swing.JTextField P25_Radio_Port;
    private javax.swing.JToggleButton P25_Talk_Button;
    private javax.swing.JLabel PTTMode;
    private javax.swing.JTextField RFFreq;
    private javax.swing.JLabel RFFreqLabel;
    private java.awt.Checkbox RX_Enc_Button;
    private javax.swing.JToggleButton Scan_Button;
    private javax.swing.JTextArea ScanningFreqs;
    private javax.swing.JTextArea StatusMessageBox;
    private java.awt.Button StatusMessageBoxClearButton;
    private javax.swing.JTextField TalkGroupID;
    private javax.swing.JCheckBox TestModeCheckBox;
    private javax.swing.JButton TestModeChooseDirRxBtn;
    private javax.swing.JButton TestModeChooseFileBtn;
    private javax.swing.JTextField TestModeDirRx;
    private javax.swing.JTextField TestModeFile;
    private javax.swing.JTextArea TextMsgRcvBox;
    private javax.swing.JTextField TextMsgSendBox;
    private javax.swing.JToggleButton ToggleSelection;
    private javax.swing.JTextField UnitID;
    private java.awt.Checkbox UseToggle;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel17;
    private javax.swing.JLabel jLabel18;
    private javax.swing.JLabel jLabel19;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel20;
    private javax.swing.JLabel jLabel21;
    private javax.swing.JLabel jLabel22;
    private javax.swing.JLabel jLabel23;
    private javax.swing.JLabel jLabel24;
    private javax.swing.JLabel jLabel25;
    private javax.swing.JLabel jLabel26;
    private javax.swing.JLabel jLabel27;
    private javax.swing.JLabel jLabel28;
    private javax.swing.JLabel jLabel29;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel30;
    private javax.swing.JLabel jLabel31;
    private javax.swing.JLabel jLabel32;
    private javax.swing.JLabel jLabel33;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenu jMenu2;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel10;
    private javax.swing.JPanel jPanel11;
    private javax.swing.JPanel jPanel12;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JPanel jPanel9;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JLabel rALGID;
    private javax.swing.JLabel rDestID;
    private javax.swing.JLabel rEmBit;
    private javax.swing.JLabel rKeyID;
    private javax.swing.JLabel rMFID;
    private javax.swing.JLabel rNAC;
    private javax.swing.JLabel rTGID;
    private javax.swing.JLabel rUnitID;
    // End of variables declaration//GEN-END:variables
    
}
