/**
 * ============================================================================
 * @file	ambe_intf.h
 * @brief	This file contains definitions for the processing functions for
 * 			the AMBE Interface library. This file contains modifications from
 * 			the test file provided by DVSI
 * @note	none.
 *
 * @author 	DVSI
 * @author	Cal(it)2
 * @author	Anthony Nwokafor <aanwokafor@ucsd.edu>
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 * @endverbatim
 * ============================================================================
 */

#ifndef AMBE_TEST_H
#define AMBE_TEST_H

/* ---------------------------- Application specific ----------------------- */
#include "common.h"
#include "apcop2.h"
#include "apcop2fec.h"
#include "pcargs.h"

#define MEASURE_STACK_SIZE
#define PRINT_TIMING
//#define OVERALL_MAXTIME


// all I/O is in 16-bit words
#define SIZEINT 	2
#define SIZEBIT		1
#define PACKED_BITS_SZ 256 /* bytes */

#define VAD_LEVEL_MIN 	-90
#define VAD_LEVEL 	0
#define VAD_LEVEL_MAX 	3
#define TONE_AMP 	-10
#define TONE_INDEX 	128
#define SD_BITS_MIN 	1
#define SD_BITS 	1
#define SD_BITS_MAX 	8

#define SPEECH_BUF_SZ 2*AMBE_WS_MAX

#define CHAN_BUF_SZ (7200/50)

void usage(char *name);

// set default options and read in command line options
void get_options(PC_OPTIONS *o, int argc, char *argv[]);

int getnextline(FILE *fp, char *line);

/**
 * ----------------------------------------------------------------------------
 * @func	process_inputfile
 * @brief	Processes test file input. Implemented in main.c
 * @note	For testing purposes only
 * ----------------------------------------------------------------------------
 */
int process_inputfile(char *inputfile);

/**
 * ----------------------------------------------------------------------------
 * @func	process_file
 * @brief	Processes test file input. Implemented in main.c
 * @note	For testing purposes only
 * ----------------------------------------------------------------------------
 */
int process_file(int argc, char *argv[]);

/**
 * ----------------------------------------------------------------------------
 * @func	process_test_input
 * @brief  	Processes test data to be sent to DSP using options specified in po
 * ----------------------------------------------------------------------------
 */
int process_test_input(PC_OPTIONS * o);

/**
 * ----------------------------------------------------------------------------
 * @func	process_test_output
 * @brief  	Processes test data received from DSP using options specified in po
 * ----------------------------------------------------------------------------
 */
int process_test_output(PC_OPTIONS * o);

/**
 * ----------------------------------------------------------------------------
 * @func	process_test_cryptography
 * @brief  	Simulates encryption/decryption processing by delaying and looping
 * 			back voice data.
 * ----------------------------------------------------------------------------
 */
int process_test_cryptography(PC_OPTIONS * o);

#endif //AMBE_TEST_H
