/**
 * ============================================================================
 * @file 	gpp_host_intf.h
 * @brief
 *
 * @author	Cal(it)2
 * @author  Jeffrey Cuenco <jcuenco@ucsd.edu>
 * @author  Justyn Bell <jwbell@ucsd.edu>
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 *
 * Change History:
 * 		Rev Date        	Author              Description of Change
 * @endverbatim
 * ============================================================================
 */

#ifndef _GPP_HOST_INTF_H
#define _GPP_HOST_INTF_H

/** @brief PCM raw data buffer size */
#define BUFF_LEN 	320

/** @brief IMBE-encoded buffer size */
#define E_BUFF_LEN 	288

#include "common.h"
#include "pcargs.h"


/**
 * @brief Host --> ARM GPP control packet headers.
 */
enum imbe_ctrl_t {
	IMBE_UNKNOWN,				///< Unknown packet header
	IMBE_DATA,					///< Data packet header from Host
	IMBE_ENCODE_START,			///< Encode start packet header
	IMBE_ENCR_ENCODE_START,		///< Encode start with encryption enabled packet header
	IMBE_ENCODE_STOP,			///< Encode stop packet header
	IMBE_DECODE_START,			///< Decode start packet header
	IMBE_DECR_DECODE_START,		///< Decode start with encryption enabled packet header
	IMBE_DECODE_STOP,			///< Decode stop packet header
	I_SOCK_DISCONNECT,			///< Socket disconnect control header
	I_SOCK_ERROR				///< Socket error control header
};

/**
 * @brief ARM GPP --> Host control packet headers.
 */
enum host_ctrl_t {
	HOST_UNKNOWN,				///< Unknown packet header
	HOST_STREAM_DATA,			///< Data packet header from GPP
	HOST_START_STREAM,			///< Host start stream packet header
	HOST_STOP_STREAM,			///< Host stop stream packet header
	H_SOCK_DISCONNECT,			///< Socket disconnect control header
	H_SOCK_ERROR				///< Socket error control header
};

/** @brief Closes socket connection. */
void gh_intf_shutdown(void);
/** @brief Main Host vocoder to ARM Loop thread. */
int gh_intf_loop(int port);
/** @brief Main ARM to host vocoder loop thread. */
int gh_intf_output(AMBE_OPTIONS * ao);
/** @brief Main encryption socket monitoring thread. */
int gh_intf_crypt_loop(int port);
/** @brief Main encryption loop thread. */
int gh_intf_cryptography(PC_OPTIONS * po);

#endif
