/**
 * ============================================================================
 * @file	logger.h
 * @brief	This file implements a timer
 * @note	none.
 *
 * @author	Cal(it)2
 * @author	Wenhua Zhao <whzhao@ucsd.edu>
 * @author	Anthony Nwokafor <aanwokafor@ucsd.edu>
 * @author	Jeff Cuenco <jcuenco@ucsd.edu>
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 * @endverbatim
 * ============================================================================
 */


#ifndef _P25_UTIL_LOGGER_H_
#define _P25_UTIL_LOGGER_H_

#include <stdio.h>
#include <stdlib.h>

#ifdef _DEBUG

static inline void dump(char *data, int len)
{
	int i;

	if (len == 0)
		return;

	printf("%02hhx ", data[0]); 
 	for (i = 1; i < len; i++) { 
 		if ((i % 8) == 0) 
 			printf(" "); 
 		if ((i % 16) == 0) 
 			printf("\n"); 
 		printf("%02hhx ", data[i]); 
 	} 
	printf("\n");
}

#define DBG(fmt, args...)	do {					\
	printf("[debug] %s() " fmt, __FUNCTION__, ##args);		\
} while (0)

#define DBGDATA(data, len)	do {					\
	printf("[debug] %s()\n",  __FUNCTION__);			\
	dump(data, len);						\
} while (0)

#else /* _DEBUG */

#define DBG(fmg, args...)
#define DBGDATA(data, len)
	
#endif

#define ASSERT(cond, fmt, args...) do {					\
	if (!(cond)) {							\
		printf("[assert] %s() " fmt, __FUNCTION__, ##args);	\
		exit(1);						\
	}								\
} while(0)

#define INFO(fmt, args...)						\
	printf("[info] %s() " fmt, __FUNCTION__, ##args);

#define ERROR(fmt, args...)						\
	printf("[error] %s() " fmt, __FUNCTION__, ##args);

#endif
