/******************************************************************************
 * DVSI CONFIDENTIAL PROPRIETARY
 ******************************************************************************
 * DVSI AMBE+2 Vocoder
 * (c) Copyright, Digital Voice Systems Inc., 2001, 2002
 * All Rights Reserved
 *
 * DVSI claims certain rights, including patent rights under
 * U.S. Patents #6,199,037 B1, #6,161,089, #5,870,045, #5,826,222,
 * #5,754,974, #5,715,365, #5,701,390, #5,649,050, #5,630,011,
 * #5,247,579, and #5,226,084, and under other U.S. and foreign
 * patents and patents pending, in the AMBE^(TM) speech coding
 * technology embodied in this software.  Any use of this software
 * or technology requires a separate written license from DVSI.
 * 	
 * AMBE is a registered trademark of Digital Voice Systems, Inc.
 * AMBE+ and AMBE+2 are trademarks of Digital Voice Systems, Inc.
 ******************************************************************************
 *****************************************************************************/

#define AMBE_ALLOW_NATIVE_TYPES

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <ctype.h>
#include "apcop2.h"
#include "pcargs.h"

/******************************************************************************
 * Open the input and output encoder/decoder files in the options structure.
 */

static void
open_files( PC_OPTIONS *a )
{
	if ( a->fpi_name && *a->fpi_name ) {
		if ( (a->fpi = fopen(a->fpi_name,"rb")) == (FILE *) 0 ) {
			fprintf( stderr, "Can't open input data file %s\n",
				 a->fpi_name );
			exit(1);
		}
	}
	
	if ( a->fpo_name && *a->fpo_name ) {
		if ( (a->fpo = fopen(a->fpo_name,"wb")) == (FILE *) 0 ) {
			fprintf( stderr, "Can't open output data file %s\n",
				 a->fpo_name );
			exit(1);
		}
	}


	
	if ( a->fpc_name && *a->fpc_name ) {
		if ( (a->fpc = fopen(a->fpc_name,"rb")) == (FILE *) 0 ) {
			fprintf( stderr, "Can't open compare data file %s\n",
				 a->fpc_name );
			exit(1);
		}
	}
}

/******************************************************************************
 * Open the input and output encoder/decoder files in the options structure.
 */

/*proto*/

void
ambe_close_files( PC_OPTIONS *a )
{

	if ( a->fpi_name && *a->fpi_name && (a->fpi != (FILE *) 0) )
		fclose( a->fpi );
	if ( a->fpo_name && *a->fpo_name && (a->fpo != (FILE *) 0) )
		fclose( a->fpo );
	if ( a->fpc_name && *a->fpc_name && (a->fpc != (FILE *) 0) )
		fclose( a->fpc );
}
/******************************************************************************
 */
/*proto*/

int
   valid_long( char    *s,
	       char **ptr,
	       int   base,
	       long    *i )
{
	char *es;
	long val = strtoul( s, &es, base );

	if ( es == s )
		return ( 0 );

	*i = val;

	if ( ptr )
		*ptr = es;

	return ( 1 );
}

/******************************************************************************
 */
/*proto*/

int
   valid_int( char    *s,
	      char **ptr,
	      int   base,
	      int     *i )
{
	long val;

	if ( !valid_long(s,ptr,base,&val) )
		return ( 0 );

	/*
	 * check the unsigned value on the positive side to catch negative values
	 * specified in hex, etc.
	 */
	if ( (val < INT_MIN) || (val > UINT_MAX) )
		return ( 0 );

	*i = (int) val;

	return ( 1 );
}

/******************************************************************************
 * Parse the arguments in argv and initialize the options structure 
 * appropriately.
 */

/*proto*/

int
ambe_read_args( PC_OPTIONS *a,
		int	    argc,
		char       *argv[])
{
	int i;

	/*
	 * set default options here
	 */


#define ARG_IS(str) !strcmp(str,argv[i])

#define CHK_ARG( chk_fn, result, errstr )			\
	if ( (i >= argc) || !chk_fn(argv[i],0,0,&(result)) ) {	\
		printf( errstr );				\
		goto return_error;				\
	}

	i = 1;
	while ( i < argc ) {

		if ( ARG_IS("-enc") ) {

			a->run_encoder = 1;
			a->run_decoder = 0;
      
		} else if ( ARG_IS("-dec") ) {

			a->run_encoder = 0;
			a->run_decoder = 1;

		} else if ( ARG_IS("-steal") ) {

			long bit_steal;

			i++;
			CHK_ARG( valid_long, bit_steal, "Invalid bit_steal.\n" );

			a->bit_steal = (short) bit_steal;
			

		}  else if ( ARG_IS("-fec") ) {
			
			int sd_dist;
			int hd_errs;

			i++;
			CHK_ARG( valid_int, sd_dist, "Invalid soft decision repeat distance.\n" )

			a->sd_dist = (short) sd_dist;

			i++;
			CHK_ARG( valid_int, hd_errs, "Invalid hard decision repeat distance.\n" )

			a->hd_errs = (short) hd_errs;

		} else if ( ARG_IS("-vad") ) {

			long lvl;

			i++;
			CHK_ARG( valid_long, lvl, "Invalid vad level.\n" );

			a->vad_level = (short) lvl;

		} else if ( ARG_IS("-nsoff") ) {

			a->nsprsn_enabled = 0;

		} else if ( ARG_IS("-toneoff") ) {

			a->tone_det_enabled = 0;

		} else if ( ARG_IS("-tone") ) {

			long amp;
			long idx;

			i++;
			CHK_ARG( valid_long, idx, "Invalid tone index.\n" );

			a->tone_index = (short) idx;

			i++;
			CHK_ARG( valid_long, amp,
				 "Invalid tone amplitude.\n" );

			a->tone_amp = (short) amp;

			a->tone_on    = 1;

		}  else if ( ARG_IS("-sdbits") ) {

			long sdbits;

			i++;
			CHK_ARG( valid_long, sdbits,
				 "Invalid soft decision size.\n" );

			a->sd_bits = (short) sdbits;

		} else if ( ARG_IS("-dtx") ) {

			a->dtx = 1;

		} else if ( ARG_IS("-pack") ) {

			a->pack = 1;
		
		} else if ( ARG_IS("-p25") ) {

			a->mode = APCO_MODE;
			a->fec_mode = APCO_MODE;

		} else if ( ARG_IS("-hr") ) {

			a->mode = APCO_HR_MODE;
			a->fec_mode = APCO_HR_MODE;		

		} else if ( ARG_IS("-fr") ) {

			a->mode = APCO_MODE;
			a->fec_mode = APCO_MODE;

		}  else if ( ARG_IS("-rep") ) {

			if ( a->mode == APCO_MODE ) {
				a->rep_mode = APCO_HR_MODE;
				a->fec_mode = APCO_HR_MODE;				
			} else {
				a->rep_mode = APCO_MODE;
				a->fec_mode = APCO_MODE;					
			}

		} else if ( ARG_IS("-cnirate") ) {

			int cnirate;

			i++;
			CHK_ARG( valid_int, cnirate,
				 "Invalid cnirate value.\n" );

			a->cnirate = (short) cnirate;

		}  else if ( ARG_IS("-vv") || ARG_IS("-info") ) {

			a->verbose += 2;

		} else if ( ARG_IS("-v") ) {

			a->verbose += 1;

		} else if ( ARG_IS("-nofec") ) {

			a->use_fec = 0;
			
		} else if ( ARG_IS("-feconly") ) {

			a->fec_only = 1;

		}  else if ( ARG_IS("-agc") ) {

			a->agc_on = 1;
			
		} else if ( ARG_IS("-cmp") ) {
			i++;
			a->fpc_name = argv[i];
			a->compare = COMPARE_ON;

		} else if ( ARG_IS("-cmpfail") ) {
			i++;
			a->fpc_name = argv[i];
			a->compare = COMPARE_FAIL;			

		} else {

			if ( argv[i][0] == '-' ) {
				goto return_error;

			} else if ( a->fpi_name == NULL ) {
				a->fpi_name = argv[i];

			} else if ( a->fpo_name == NULL ) {
				a->fpo_name = argv[i];

			} else
				goto return_error;
		}

		i++;
	}

	open_files( a );
  
	return ( 0 );

return_error:
	return ( -1 );
}

/*****************************************************************************/
