/******************************************************************************
 * DVSI CONFIDENTIAL PROPRIETARY
 ******************************************************************************
 * DVSI AMBE^(TM) Speech Coder
 * (c) Copyright, Digital Voice Systems Inc., 1997
 * All Rights Reserved
 *
 * DVSI claims certain rights, including patent rights under
 * U.S. Patents #5,226,084, #5,247,579, #5,491,772, #5,517,511 
 * and under other U.S. and foreign patents pending, in the 
 * AMBE^(TM) speech coding technology embodied in this software.  
 * Any use of this software or technology requires a separate 
 * written license from DVSI.
 * 	
 * AMBE is a trademark of Digital Voice Systems, Inc.
 ******************************************************************************
 * $Name:  $
 * $Revision: 1.1 $
 * $RCSfile: timer.h,v $
 * $Date: 2008/05/05 19:29:36 $
 *****************************************************************************/
#ifndef __timer_h__
#define __timer_h__



/*
 * timer registers are memory mapped
 */
#define timer0_control		(*(volatile int *)0x01940000)
#define timer0_period		(*(volatile int *)0x01940004)
#define timer0_counter		(*(volatile int *)0x01940008)
#define timer1_control		(*(volatile int *)0x01980000)
#define timer1_period		(*(volatile int *)0x01980004)
#define timer1_counter		(*(volatile int *)0x01980008)
#define timer2_control		(*(volatile int *)0x01ac0000)
#define timer2_period		(*(volatile int *)0x01ac0004)
#define timer2_counter		(*(volatile int *)0x01ac0008)

/*
 * bit masks for bits in timer control registers
 */
#define TCTL_RSVD1	0xfffff000
#define TCTL_TSTAT	0x00000800
#define TCTL_INVINP	0x00000400
#define TCTL_CLKSRC	0x00000200
#define TCTL_CP		0x00000100
#define TCTL_HLD	0x00000080
#define TCTL_GO		0x00000040
#define TCTL_RSVD0	0x00000020
#define TCTL_PWID	0x00000010
#define TCTL_DATIN	0x00000008
#define TCTL_DATOUT	0x00000004
#define TCTL_INVOUT	0x00000002
#define TCTL_FUNC	0x00000001

#define TCTL_RSVD	(TCTL_RSVD0 | TCTL_RSVD1)

/*
 * functions in timer.c
 */

inline void
   start_timer( void )
{
	timer1_control &= ~TCTL_HLD;		//hold=0
	timer1_period = 0xffffffff;
	timer1_control |= TCTL_CLKSRC;
	timer1_control |= (TCTL_HLD | TCTL_GO);	//hold=1,go=1 (go autoclears)
}

inline int
   get_timer( void )
{
	return( timer1_counter * 8 );
}

inline int
   get_timer_max( int max )
{
	int time = get_timer();
	if (time > max)
		max = time;
	return( max );
}

inline void
   donops( void )
{
	/*  execute 400 nops */
	asm(" nop 8\n nop 8\n nop 8\n nop 8\n nop 8\n");
	asm(" nop 8\n nop 8\n nop 8\n nop 8\n nop 8\n");
	asm(" nop 8\n nop 8\n nop 8\n nop 8\n nop 8\n");
	asm(" nop 8\n nop 8\n nop 8\n nop 8\n nop 8\n");
	asm(" nop 8\n nop 8\n nop 8\n nop 8\n nop 8\n");
	asm(" nop 8\n nop 8\n nop 8\n nop 8\n nop 8\n");
	asm(" nop 8\n nop 8\n nop 8\n nop 8\n nop 8\n");
	asm(" nop 8\n nop 8\n nop 8\n nop 8\n nop 8\n");
	asm(" nop 8\n nop 8\n nop 8\n nop 8\n nop 8\n");
	asm(" nop 8\n nop 8\n nop 8\n nop 8\n nop 8\n");
}

inline int timenops( void )
{
	start_timer();
	donops();
	return get_timer();
}
#endif

