/**
 * ============================================================================
 * @file	dprintf.h
 * @brief	This file contains definition for real time print outs from DSP
 * 			called dprintf. This is for DSP side.
 * @note	none.
 *
 * @author	DVSI
 * @author	Cal(it)2
 * @author	Anthony Nwokafor <aanwokafor@ucsd.edu>
 *
 * @verbatim
 * CAL(IT)2 CONTROL INFORMATION
 *
 * Copyright ©2009-2011, The Regents of the University of California
 * All Rights Reserved
 * See UCSD-license.h for more information
 *
 * Project Name:    APCO-P25 Linux Implementation
 * @endverbatim
 * ============================================================================
 */

#ifndef DPRINTF_H
#define DPRINTF_H

#include <std.h>
#include "dprintf_p.h"

/**
 * ----------------------------------------------------------------------------
 * @func	dprintf
 * @brief	outputs messages to ARM host processor.
 * ----------------------------------------------------------------------------
 */
void dprintf (const char * format, ...);

/**
 * ----------------------------------------------------------------------------
 * @func	dprintf_dsp_callback
 * @brief	callback function for the DSP BIOS NOTIFY mechanism.
 * ----------------------------------------------------------------------------
 */
void dprintf_dsp_callback (Uint32 eventNo, Ptr arg, Ptr info);

/**
 * ----------------------------------------------------------------------------
 * @func	dprintf_init_dsp
 * @brief	Initialize the DSP side of dprintf.
 * ----------------------------------------------------------------------------
 */
int dprintf_init_dsp ();

#endif //DPRINTF_H
